
use strict;
use Date::Parse;
use CGI::Carp qw(fatalsToBrowser);

my $DEBUG      = $ENV{DEBUG};
my $errlog     = $ENV{ERRLOG};
my $xport      = $ENV{EXPORT_TO_CSV};
my $webdir     = $ENV{WEBDIR};
my $basedir    = $ENV{INPUTDIR};
my $detail_yes = 1;
my $detail_no  = 0;
my $tmp_dir    = "$basedir/tmp";
my $wrkdir     = "$basedir/data";
my $time_u     = time();

# CGI-BIN HTML header
print "Content-type: text/html\n\n";

open( OUT, ">> $errlog" ) if $DEBUG == 2;

my @items_sorted = "sum_io_total sum_io sum_io_b io_rate ops_rate sum_data_total sum_data sum_data_b sum_resp_t sum_resp_t_b sum_busy sum_busy_frontend sum_busy_backend sum_busy_rdf data_rate read_io write_io read_ops write_ops io_pending read write sum_capacity total_usage cache_hit r_cache_hit w_cache_hit r_cache_usage w_cache_usage read_pct resp resp_t resp_t_r resp_t_w io_rate_b read_io_b write_io_b data_rate_b read_b write_b resp_t_b resp_t_r_b resp_t_w_b sys busy compress pprc_rio pprc_wio pprc_data_r pprc_data_w pprc_rt_r pprc_rt_w tier0 tier0-nl tier1 tier2 sum_repl repl_rate read_repl write_repl read_data_job write_data_job read_copy_job write_copy_job dedupe_ratio used real io_rate-subsys data_rate-subsys read_io-subsys write_io-subsys read-subsys write-subsys io_rate-FC data_rate-FC read_io-FC write_io-FC read-FC write-FC data_cntl io_cntl ssd_r_cache_hit data_in data_out frames_in frames_out credits errors enc_out disc_c3 link_fail loss_sync operating_rate write_pend clean_usage middle_usage phys_usage wafl_exempt-kahuna-exempt-hostOS-nwk_legacy cifs_io_rate-3_io_rate-4_io_rate-iscsi_io_rate-fcp_io_rate cifs_read_io-3_read_io-4_read_io-iscsi_read_io-fcp_read_io cifs_write_io-3_write_io-4_write_io-iscsi_write_io-fcp_write_io cifs_data_rate-3_data_rate-4_data_rate-iscsi_data_rate-fcp_data_rate cifs_read-3_read-4_read-iscsi_read-fcp_read cifs_write-3_write-4_write-iscsi_write-fcp_write cifs_resp_t-3_resp_t-4_resp_t-iscsi_resp_t-fcp_resp_t packet_rate read_packet write_packet read_error write_error io_rate_ini data_rate_ini resp_t_ini usage_rate dnlc_hit buffer_hit qdepth calls_rate write_calls read_calls dirty_cache_pages b2b crc WP alloc tier_alloc_prct io_rate_frontend data_rate_frontend read_io_frontend write_io_frontend read_frontend write_frontend busy_frontend resp_t_r_frontend resp_t_w_frontend io_rate_backend data_rate_backend read_io_backend write_io_backend read_backend write_backend busy_backend io_rate_rdf data_rate_rdf read_io_rdf write_io_rdf read_rdf write_rdf busy_rdf cfc_dirty cfc_max del_ack compress_cap compress_virt_cap uncompress_cap cifs_ops nfs_ops queue sum_capacity_aggregated r_transfer_size w_transfer_size transfer_size hit_ratio ssd_hit_ratio total_read total_write queue_full_count sum_net net_rate write_net read_net sum_bound_pre_comp sum_bound_net outbound_pre_comp inbound_pre_comp outbound_net inbound_net phase user system total free idle nfs_read_ops nfs_write_ops nfs_ops_rate nfsv4_read_ops nfsv4_write_ops nfsv4_ops_rate smb1_rate smb2_rate total_comp_factor r_cache_miss w_cache_miss memory apply_lat commit_lat pgs virt tier_ENT tier_SSD tier_NL tier_SATA tier_Flash tier_HighCapFlash";

# get QUERY_STRING
use Env qw(QUERY_STRING);
print OUT "-- $QUERY_STRING\n" if $DEBUG == 2;

#print STDERR "$QUERY_STRING\n";

################ get URL parameters (could be GET or POST) and return them as hash refference
#                multiple values with same name create an array
sub getURLparams {
  my ( $buffer, @pairs, $pair, $name, $value, %PAR );

  if ( defined $ENV{'REQUEST_METHOD'} ) {
    if ( lc $ENV{'REQUEST_METHOD'} eq "post" ) {
      read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
    }
    else {
      $buffer = $ENV{'QUERY_STRING'};
    }

    # Split information into name/value pairs
    @pairs = split( /&/, $buffer );
    foreach $pair (@pairs) {
      ( $name, $value ) = split( /=/, $pair );
      $value =~ tr/+/ /;
      $value =~ s/%(..)/pack("C", hex($1))/eg;
      if ( exists $PAR{$name} ) {
        if ( ref $PAR{$name} eq "ARRAY" ) {
          push @{ $PAR{$name} }, $value;
        }
        else {
          my $firstVal = $PAR{$name};
          $PAR{$name} = [];
          push @{ $PAR{$name} }, $firstVal;
          push @{ $PAR{$name} }, $value;
        }
      }
      else {
        $PAR{$name} = $value;
      }
    }
    return \%PAR;
  }
  else {
    return 0;
  }
} ## end sub getURLparams

sub urlencode {
  my $s = shift;
  $s =~ s/([^a-zA-Z0-9!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
  return $s;
} ## end sub urlencode

my %params;   # hash containing URL parameters. Use like this: $params{server}

my $qs = getURLparams;
if ( $qs ) {
  %params = %{ $qs };
  if ( $params{debug} ) {
    use Data::Dumper;
    print "<pre style='font-size: small'>QS: ". $ENV{'QUERY_STRING'} . "\n";
    print Data::Dumper->Dump( [ \%params ], [ qw(*params) ] );
    print "</pre>";
  }
}

# assign values from %params for back compatibility
my ( $host, $type, $name, $st_type, $item, $gui, $referer );
$host    = $params{host};
$type    = $params{type};
$name    = $params{name};
$st_type = $params{storage};
$item    = $params{item};
$gui     = $params{gui};
$referer = $params{referer};

# ( my $host, my $type, my $name, my $st_type, my $item, my $gui, my $referer ) = split( /&/, $QUERY_STRING );

# no URL decode here, as it goes immediately into URL again

my $host_url = urlencode($host);

#print STDERR "$host $type $name $item\n";

#print STDERR "$host $name $type $st_type $item\n";

#print STDERR "$QUERY_STRING : $name\n";
if ( !$gui ) {
  $gui = 0;
}


# http_base must be passed through cgi-bin script to get location of jquery scripts and others
# it is taken from HTTP_REFERER first time and then passed in the HTML GET
my $html_base = "";
$referer =~ s/referer=//;    #for paging aggregated : PH: not only for paging!!

if ( $referer ne "" && $referer !~ m/none=/ ) {

  # when is referer set then it is already html_base --> not call html_base($refer) again
  $referer =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
  $referer =~ s/\+/ /g;
  $html_base = $referer;
} ## end if ( $referer ne "" &&...)
else {
  $referer   = $ENV{HTTP_REFERER};
  $html_base = html_base($referer);    # it must be here behind $base setting
}

if ( $name =~ m/^top$/ ) {

  # volumes text tables
  # must be before $item =~ m/^sum$/
  volumes_top_all( $host, $type, $item, $st_type, $name );
  exit(0);
} ## end if ( $name =~ m/^top$/)

if ($gui == 2){
  make_re_group($host, $type, $name, $st_type, $item);
  exit(0);
}

if ( $item =~ m/^sum$/ ) {

  # aggregated graphs
  make_agg( $host, $type, $name, $item, $st_type );
  exit(0);
} ## end if ( $item =~ m/^sum$/)

# individual "item" graohs
if ( $st_type =~ "^SAN-" ) {
  create_tab_san( $host, $type, $name, $st_type, $item );
}
elsif ( $st_type =~ /^LAN-(CISCO|UNKNOWN)$/ ) {
  create_tab_lan( $host, $type, $name, $st_type, $item );
}
else {
  if ( $item =~ m/^custom$/ ) {

    # Custom group
    create_tab_custom( $host, $type, $name, $st_type, $item );
  }
  else {
    # default stuff
    create_tab( $host, $type, $name, $st_type, $item );
  }
} ## end else [ if ( $st_type =~ "^SAN-")]

sub make_re_group{

  my ( $host, $type, $name, $st_type, $item ) = @_;
  my $tab_number = 0;
  my $data_type = "tabfrontend";
  my $item_high = "";
  my $name_act = "";
  my @items = ();

  print "<div  id=\"tabs\"> <ul>\n";

  print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">daily</a></li>\n";
  $tab_number++;

  print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">weekly</a></li>\n";
  $tab_number++;

  print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">monthly</a></li>\n";
  $tab_number++;

  print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">yearly</a></li>\n";
  $tab_number++;

  print "</ul>\n";


  $tab_number = 0;
  print "<div id=\"tabs-$tab_number\"><center>\n";
  $tab_number++;

  @items = get_items_re_group($name, $host, $type, "d");

  create_tab_re_group($host, $type, $name, $st_type, \@items, "d");
  print "</div>\n";

  print "<div id=\"tabs-$tab_number\"><center>\n";
  $tab_number++;

  @items = get_items_re_group($name, $host, $type, "w");

  create_tab_re_group($host, $type, $name, $st_type, \@items, "w");
  print "</div>\n";

  print "<div id=\"tabs-$tab_number\"><center>\n";
  $tab_number++;

  @items = get_items_re_group($name, $host, $type, "m");

  create_tab_re_group($host, $type, $name, $st_type, \@items, "m");
  print "</div>\n";

  print "<div id=\"tabs-$tab_number\"><center>\n";
  $tab_number++;

  @items = get_items_re_group($name, $host, $type, "y");

  create_tab_re_group($host, $type, $name, $st_type, \@items, "y");
  print "</div>\n";



  print "</div>\n";

}

sub get_items_re_group{

  my $name = shift;
  my $host = shift;
  my $type = shift;
  my $time = shift;
  my @items = ();


  if ($name =~ m/read|read_io|resp_t_r|r_transfer_size/){
    @items = ("read_io", "read", "resp_t_r", "r_transfer_size");
  }

  if ($name =~ m/write|write_io|resp_t_w|w_transfer_size/){
    @items = ("write_io", "write", "resp_t_w", "w_transfer_size");
  }

  if ($name =~ m/io_rate|data_rate|resp_t/){
    my $file = "$tmp_dir/$host/$type-io_rate-$time.cmd";
    if (-f $file){
      push(@items,"io_rate");
    }
    else{
      push(@items,"sum_io");
    }
    $file = "$tmp_dir/$host/$type-data_rate-$time.cmd";
    if (-f $file){
      push(@items,"data_rate");
    }
    else{
      push(@items,"sum_data");
    }
    $file = "$tmp_dir/$host/$type-resp_t-$time.cmd";
    if (-f $file){
      push(@items,"resp_t");
    }
    else{
      push(@items,"sum_resp_t");
    }

  }

  ### exeption for sum test exist cmd file
  if ($name =~ m/sum_io|sum_data|sum_resp_t/){
    my $file = "$tmp_dir/$host/$type-sum_io-$time.cmd";
    if (-f $file){
      push(@items,"sum_io");
    }
    else{
      push(@items,"io_rate");
    }
    $file = "$tmp_dir/$host/$type-sum_data-$time.cmd";
    if (-f $file){
      push(@items,"sum_data");
    }
    else{
      push(@items,"data_rate");
    }
    $file = "$tmp_dir/$host/$type-sum_resp_t-$time.cmd";
    if (-f $file){
      push(@items,"sum_resp_t");
    }
    else{
      push(@items,"resp_t");
    }
  }

  return @items;
}

sub create_tab_re_group{

  my ($host, $type, $name, $st_type, $item_tmp, $time  ) = @_;
  my @items = @{$item_tmp};

  print "<div class=\'regroup bck\'><a href=\'$ENV{WEB_CGI_PATH}/detail.sh?host=$host_url&type=$type&name=$name&storage=$st_type&item=sum&gui=1&none=none::\' alt=\'Back\' title=\'Back\'></a></div>\n";
  print "<table align=\"center\" class='regrp' summary=\"Graphs\">\n";

  foreach my $item (@items){
    if (-f "$tmp_dir/$host/$type-$item-$time.cmd"){
      print "<tr>\n";
      print_item( $host, $type, $item, "sum", $time, $detail_yes, "re-group");
      print "</tr>\n";
    }
  }

  print "</table>\n";

  return 1;


}

sub create_tab_san {

  #host=ASAN11&type=PORT&name=port9&storage=SAN-BRCD&item=san&gui=1&none=none::
  #( my $host,my $type, my $name, my $st_type, my $item, my $gui, my $referer) = split(/&/,$QUERY_STRING);

  my ( $host, $type, $name, $st_type, $item ) = @_;

  if ( $item eq "san" ) {
    my @graph = ( "$item\_data", "$item\_io", "$item\_io_size", "$item\_credits", "$item\_errors", "$item\_enc_out", "$item\_disc_c3", "$item\_link_fail", "$item\_loss_sync" );

    # print out tabs
    my $tab_number = 0;
    print "<div  id=\"tabs\"> <ul>\n";
    foreach $item (<@graph>) {

      if ( $st_type eq "SAN-CISCO" && ($item eq "san_enc_out" || $item eq "san_disc_c3" || $item eq "san_link_fail" || $item eq "san_loss_sync") ) { next; } # only for Brocade

      # Show BBCredits only for fc ports
      if ( $item eq "san_credits" && $st_type eq "SAN-CISCO" ) {
        my $go_bbc = 0;
        if ( $name =~ "^portfc" || $name =~ "portChannel" || $name =~ "bay" || $name =~ "ext" ) { $go_bbc = 1; }
        if ( $go_bbc == 0 ) { next; }
      }

      # data source back/fron end
      my $data_type = "tabfrontend";
      my $tab_name = text_tab( $item, "item" );
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)
    print "   </ul> \n";

    $tab_number = 0;
    foreach $item (<@graph>) {
      my $item_name = $item;

      # Show BBCredits only for fc ports
      if ( $item eq "san_credits" && $st_type eq "SAN-CISCO" ) {
        my $go_bbc = 0;
        if ( $name =~ "^portfc" || $name =~ "portChannel" || $name =~ "bay" || $name =~ "ext" ) { $go_bbc = 1; }
        if ( $go_bbc == 0 ) { next; }
      }

      print "<div id=\"tabs-$tab_number\"><center>\n";
      if ( $gui == 0 ) {
        print "<center><h3>$item_name</h3></center>";
      }

      print "<table align=\"center\" summary=\"Graphs\">\n";

      print "<tr>\n";
      print_item( $host, $type, $name, $item, "d", $detail_yes );
      print_item( $host, $type, $name, $item, "w", $detail_yes );
      print "</tr><tr>\n";
      print_item( $host, $type, $name, $item, "m", $detail_yes );
      print_item( $host, $type, $name, $item, "y", $detail_yes );
      print "</tr>\n";

      #if ( $item_name eq "san_credits" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of times that the transmit credit has reached 0.</td></tr>\n"; }
      if ( $item_name eq "san_errors" ) {
        print "<tr><td align=\"center\" colspan=\"2\">Counts the number of CRC errors detected for frames received.</td></tr>\n";

        #print "<tr><td align=\"left\" colspan=\"2\"><b>swFCPortRxEncOutFrs</b> : Counts the number of encoding error or disparity error outside frames received.</td></tr>\n";
      }

      # BBCredits a href
      if ( $item eq "san_credits" ) {
        print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/BBCredit.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'What is BBCredits?\' title=\'What is BBCredits?\'></a></div>\n";
      }

      print "</table></center>\n";
      print "</div>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)

    print "</div><br>\n";
    if ( $gui == 0 ) {
      print "</BODY></HTML>";
    }
  } ## end if ( $item eq "san" )

  if ( $item eq "san_data_sum" || $item eq "san_io_sum" || $item eq "san_cpu_mem" ) {
    my @graph = ( "$item\_in", "$item\_out" );

    if ( $item eq "san_io_sum" ) {
      @graph = ( "$item\_in", "$item\_out", "$item\_credits", "$item\_crc_errors", "$item\_enc_out", "$item\_disc_c3", "$item\_link_fail", "$item\_loss_sync" );
    }

    if ( $item eq "san_cpu_mem" ) {
      @graph = ( "san_cpu", "san_mem" );
    }

    # print out tabs
    my $tab_number = 0;
    print "<div  id=\"tabs\"> <ul>\n";
    foreach $item (<@graph>) {

      if ( $st_type eq "SAN-CISCO" && ($item =~ m/enc_out/ || $item =~ m/disc_c3/ || $item =~ m/link_fail/ || $item =~ m/loss_sync/) ) { next; } # only for Brocade

      # data source back/fron end
      my $data_type = "tabfrontend";
      my $tab_name = text_tab( $item, "item" );
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)
    print "   </ul> \n";

    $tab_number = 0;
    foreach $item (<@graph>) {
      my $item_name = $item;

      print "<div id=\"tabs-$tab_number\"><center>\n";
      if ( $gui == 0 ) {
        print "<center><h3>$item_name</h3></center>";
      }

      print "<table align=\"center\" summary=\"Graphs\">\n";

      print "<tr>\n";
      print_item( $host, $type, $name, $item, "d", $detail_yes );
      print_item( $host, $type, $name, $item, "w", $detail_yes );
      print "</tr><tr>\n";
      print_item( $host, $type, $name, $item, "m", $detail_yes );
      print_item( $host, $type, $name, $item, "y", $detail_yes );
      print "</tr>\n";

      if ( $item_name eq "san_io_sum_crc_errors" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of CRC errors detected for frames received.</td></tr>\n"; }

      #if ( $item_name eq "san_io_sum_credits" )         { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of times that the transmit credit has reached 0.</td></tr>\n"; }
      #if ( $item_name eq "san_io_sum_encoding_errors" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of encoding error or disparity error outside frames received.</td></tr>\n"; }
      # BBCredits a href
      if ( $item eq "san_io_sum_credits" ) {
        print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/BBCredit.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'What is BBCredits?\' title=\'What is BBCredits?\'></a></div>\n";
      }

      print "</table></center>\n";
      print "</div>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)

    print "</div><br>\n";
    if ( $gui == 0 ) {
      print "</BODY></HTML>";
    }
  } ## end if ( $item eq "san_data_sum"...)

  if ( $item =~ "san_data_sum_tot" || $item =~ "san_io_sum_tot" || $item eq "san_cpu_mem_tot" ) {
    my @graph = ( "$item\_in", "$item\_out" );

    if ( $item eq "san_io_sum_tot" ) {
      @graph = ( "$item\_in", "$item\_out", "$item\_credits", "$item\_crc_errors", "$item\_enc_out", "$item\_disc_c3", "$item\_link_fail", "$item\_loss_sync" );
    }

    if ( $item eq "san_cpu_mem_tot" ) {
      @graph = ( "san_cpu_tot", "san_mem_tot" );
    }

    # BNA
    if ( $item =~ "san_data_sum_tot_bna" ) {
      @graph = ( "san_data_sum_tot_in_bna", "san_data_sum_tot_out_bna" );
    }
    if ( $item =~ "san_io_sum_tot_bna" ) {
      @graph = ( "san_io_sum_tot_in_bna", "san_io_sum_tot_out_bna", "san_io_sum_tot_credits_bna", "san_io_sum_tot_crc_errors_bna", "san_io_sum_tot_enc_out_bna", "san_io_sum_tot_disc_c3_bna", "san_io_sum_tot_link_fail_bna", "san_io_sum_tot_loss_sync_bna" );
    }

    # try if there are some Brocade switches, if not, then skip some metrics which are available only for BRCD
    my $brcd_found   = 0;
    my @switches_all = <$wrkdir/*\/SAN-*>;
    foreach my $line (@switches_all) {
      chomp $line;
      if ( $line =~ m/\/SAN-BRCD$/ ) {
        $brcd_found = 1;
        last;
      }
    }

    # print out tabs
    my $tab_number = 0;
    print "<div  id=\"tabs\"> <ul>\n";
    foreach $item (<@graph>) {

      if ( $brcd_found == 0 && ($item =~ m/enc_out/ || $item =~ m/disc_c3/ || $item =~ m/link_fail/ || $item =~ m/loss_sync/) ) { next; } # only for Brocade

      # data source back/fron end
      my $data_type = "tabfrontend";
      my $tab_name = text_tab( $item, "item" );
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)
    print "   </ul> \n";

    $tab_number = 0;
    foreach $item (<@graph>) {
      my $item_name = $item;

      print "<div id=\"tabs-$tab_number\"><center>\n";
      if ( $gui == 0 ) {
        print "<center><h3>$item_name</h3></center>";
      }

      print "<table align=\"center\" summary=\"Graphs\">\n";

      print "<tr>\n";
      print_item( $host, $type, $name, $item, "d", $detail_yes );
      print_item( $host, $type, $name, $item, "w", $detail_yes );
      print "</tr><tr>\n";
      print_item( $host, $type, $name, $item, "m", $detail_yes );
      print_item( $host, $type, $name, $item, "y", $detail_yes );
      print "</tr>\n";

      # SAN totals data in|out trends
      if ( $item =~ "san_data_sum_tot_in" || $item =~ "san_data_sum_tot_out" ) {
        print "<tr>\n";
        print_item( $host, $type, $name, "$item\_trend", "y", $detail_no );
        print "</tr>\n";
      }

      if ( $item_name =~ "san_io_sum_tot_crc_errors" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of CRC errors detected for frames received.</td></tr>\n"; }

      #if ( $item_name eq "san_io_sum_tot_credits" )         { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of times that the transmit credit has reached 0.</td></tr>\n"; }
      #if ( $item_name eq "san_io_sum_tot_encoding_errors" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of encoding error or disparity error outside frames received.</td></tr>\n"; }
      # BBCredits a href
      if ( $item =~ "san_io_sum_tot_credits" ) {
        print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/BBCredit.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'What is BBCredits?\' title=\'What is BBCredits?\'></a></div>\n";
      }

      print "</table></center>\n";
      print "</div>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)

    print "</div><br>\n";
    if ( $gui == 0 ) {
      print "</BODY></HTML>";
    }
  } ## end if ( $item eq "san_data_sum_tot"...)

  if ( $item =~ "san_fabric" ) {
    my @graph = ( "$item\_data_in", "$item\_frames_in", "$item\_credits", "$item\_crc_errors", "$item\_enc_out", "$item\_disc_c3", "$item\_link_fail", "$item\_loss_sync", "$item\_conf" );

    # BNA
    if ( $item =~ "san_fabric_bna" ) {
      @graph = ( "san_fabric_data_in_bna", "san_fabric_frames_in_bna", "san_fabric_credits_bna", "san_fabric_crc_errors_bna", "san_fabric_enc_out_bna", "san_fabric_disc_c3_bna", "san_fabric_link_fail_bna", "san_fabric_loss_sync_bna", "san_fabric_conf_bna" );
    }

    # print out tabs
    my $tab_number = 0;
    print "<div  id=\"tabs\"> <ul>\n";
    foreach $item (<@graph>) {

      if ( $st_type eq "SAN-CISCO" && ($item =~ m/enc_out/ || $item =~ m/disc_c3/ || $item =~ m/link_fail/ || $item =~ m/loss_sync/) ) { next; } # only for Brocade

      # data source back/fron end
      my $data_type = "tabfrontend";
      my $tab_name = text_tab( $item, "item" );
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)
    print "   </ul> \n";

    $tab_number = 0;
    foreach $item (<@graph>) {
      my $item_name = $item;

      print "<div id=\"tabs-$tab_number\"><center>\n";
      if ( $gui == 0 ) {
        print "<center><h3>$item_name</h3></center>";
      }

      if ( $item_name =~ "san_fabric_conf" ) {
        if ( $item_name eq "san_fabric_conf" ) {
          if ( -f "$webdir/fabric_cfg.html" ) {
            open( FAC, "< $webdir/fabric_cfg.html" ) || error( "file does not exists : $webdir/fabric_cfg.html " . __FILE__ . ":" . __LINE__ ) && return 0;
            my @lines = <FAC>;
            close(FAC);

            print @lines;
          } ## end if ( -f "$webdir/fabric_cfg.html")
          else {
            print "Fabric configuration table not found!";
          }
        }
        if ( $item_name eq "san_fabric_conf_bna" ) {
          if ( -f "$webdir/fabric_cfg_bna.html" ) {
            open( FAC, "< $webdir/fabric_cfg_bna.html" ) || error( "file does not exists : $webdir/fabric_cfg_bna.html " . __FILE__ . ":" . __LINE__ ) && return 0;
            my @lines = <FAC>;
            close(FAC);

            print @lines;
          }
          else {
            print "Fabric configuration table not found!";
          }
        }
      }
      else {
        print "<table align=\"center\" summary=\"Graphs\">\n";

        print "<tr>\n";
        print_item( $host, $type, $name, $item, "d", $detail_yes );
        print_item( $host, $type, $name, $item, "w", $detail_yes );
        print "</tr><tr>\n";
        print_item( $host, $type, $name, $item, "m", $detail_yes );
        print_item( $host, $type, $name, $item, "y", $detail_yes );
        print "</tr>\n";

        if ( $item_name =~ "san_fabric_crc_errors" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of CRC errors detected for frames received.</td></tr>\n"; }

        #if ( $item_name eq "san_fabric_credits" )         { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of times that the transmit credit has reached 0.</td></tr>\n"; }
        #if ( $item_name eq "san_fabric_encoding_errors" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of encoding error or disparity error outside frames received.</td></tr>\n"; }
        # BBCredits a href
        if ( $item =~ "san_fabric_credits" ) {
          print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/BBCredit.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'What is BBCredits?\' title=\'What is BBCredits?\'></a></div>\n";
        }

        print "</table></center>\n";
        print "</div>\n";
      } ## end else [ if ( $item_name eq "san_fabric_conf")]
      $tab_number++;
    } ## end foreach $item (<@graph>)

    print "</div><br>\n";
    if ( $gui == 0 ) {
      print "</BODY></HTML>";
    }
  } ## end if ( $item eq "san_fabric")

  if ( $item =~ "san_isl" ) {
    my @graph = ( "$item\_data_in", "$item\_data_out", "$item\_frames_in", "$item\_frames_out", "$item\_frame_size_in", "$item\_frame_size_out", "$item\_credits", "$item\_crc_errors", "$item\_enc_out", "$item\_disc_c3", "$item\_link_fail", "$item\_loss_sync" );

    # BNA
    if ( $item =~ "san_isl_bna" ) {
      @graph = ( "san_isl_data_in_bna", "san_isl_data_out_bna", "san_isl_frames_in_bna", "san_isl_frames_out_bna", "san_isl_frame_size_in_bna", "san_isl_frame_size_out_bna", "san_isl_credits_bna", "san_isl_crc_errors_bna", "san_isl_enc_out_bna", "san_isl_disc_c3_bna", "san_isl_link_fail_bna", "san_isl_loss_sync_bna" );
    }

    # print out tabs
    my $tab_number = 0;
    print "<div  id=\"tabs\"> <ul>\n";
    foreach $item (<@graph>) {

      if ( $st_type eq "SAN-CISCO" && ($item =~ m/enc_out/ || $item =~ m/disc_c3/ || $item =~ m/link_fail/ || $item =~ m/loss_sync/) ) { next; } # only for Brocade

      # data source back/fron end
      my $data_type = "tabfrontend";
      my $tab_name = text_tab( $item, "item" );
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)
    print "   </ul> \n";

    $tab_number = 0;
    foreach $item (<@graph>) {
      my $item_name = $item;

      print "<div id=\"tabs-$tab_number\"><center>\n";
      if ( $gui == 0 ) {
        print "<center><h3>$item_name</h3></center>";
      }

      print "<table align=\"center\" summary=\"Graphs\">\n";

      print "<tr>\n";
      print_item( $host, $type, $name, $item, "d", $detail_yes );
      print_item( $host, $type, $name, $item, "w", $detail_yes );
      print "</tr><tr>\n";
      print_item( $host, $type, $name, $item, "m", $detail_yes );
      print_item( $host, $type, $name, $item, "y", $detail_yes );
      print "</tr>\n";

      if ( $item_name =~ "san_isl_crc_errors" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of CRC errors detected for frames received.</td></tr>\n"; }

      #if ( $item_name eq "san_isl_credits" )         { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of times that the transmit credit has reached 0.</td></tr>\n"; }
      #if ( $item_name eq "san_fabric_encoding_errors" ) { print "<tr><td align=\"center\" colspan=\"2\">Counts the number of encoding error or disparity error outside frames received.</td></tr>\n"; }
      # BBCredits a href
      if ( $item =~ "san_isl_credits" ) {
        print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/BBCredit.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'What is BBCredits?\' title=\'What is BBCredits?\'></a></div>\n";
      }

      print "</table></center>\n";
      print "</div>\n";
      $tab_number++;
    } ## end foreach $item (<@graph>)

    print "</div><br>\n";
    if ( $gui == 0 ) {
      print "</BODY></HTML>";
    }
  } ## end if ( $item eq "san_isl")

  if ( $item eq "HS" ) {
    my $hs_file_html    = "$webdir/$host/health_status.html";
    my $event_file_html = "$webdir/$host/event.html";
    if ( $type eq "HS-total" ) {
      $hs_file_html    = "$webdir/total_health_status.html";
      $event_file_html = "$webdir/SAN_total_events.html";
    }

    print "<div  id=\"tabs\"> <ul>\n";
    print "  <li><a href=\"#tabs-0\">Ports</a></li>\n";
    if ( -f "$event_file_html" ) {
      print "  <li><a href=\"#tabs-1\">Events</a></li>\n";
    }
    print "   </ul> \n";

    print "<div id=\"tabs-0\" class=\"blacktext\"><center>\n";
    if ( -f "$hs_file_html" ) {
      open( HTM, "< $hs_file_html" ) || error( "Can't open $hs_file_html : $! " . __FILE__ . ":" . __LINE__ ) && return "";
      my @file = <HTM>;
      close(HTM);

      print @file;
    } ## end if ( -f "$hs_file_html")
    print "</div>\n";

    if ( -f "$event_file_html" ) {
      print "<div id=\"tabs-1\"><center>\n";

      open( HTM, "< $event_file_html" ) || error( "Can't open $event_file_html : $! " . __FILE__ . ":" . __LINE__ ) && return "";
      my @file = <HTM>;
      close(HTM);

      print @file;
      print "</div>\n";
    } ## end if ( -f "$event_file_html")

    if ( $gui == 0 ) {
      print "</BODY></HTML>";
    }
  } ## end if ( $item eq "HS" )

  return 1;
} ## end sub create_tab_san

sub create_tab_lan {
  my ( $host, $type, $name, $st_type, $item ) = @_;
  my @graph;

  if ( $item =~ /^lan$/ ) {
    @graph = ( "$item\_data", "$item\_Ucast_pkts", "$item\_errors" );
    #"$item\_Mcast_pkts", "$item\_Bcast_pkts",
  }
  elsif ( $item =~ /^lan_sum_data(_tot)?$/ ) {
    @graph = ( "$item\_in", "$item\_out" );
  }
  elsif ( $item =~ /^lan_sum_pkts(_tot)?$/ ) {
    @graph = ( "$item\_Ucast_in", "$item\_Ucast_out", "$item\_errors_in", "$item\_errors_out" );
    # "$item\_Mcast_in", "$item\_Mcast_out", "$item\_Bcast_in", "$item\_Bcast_out"
  }
  elsif ( $item =~ /^lan_cpu_mem$/ ) {
    if ( glob( "$wrkdir/$host/*.lcd" ) ) { push @graph, 'lan_cpu'; }
    if ( glob( "$wrkdir/$host/*.lmd" ) ) { push @graph, 'lan_mem'; }
  } ## end elsif ( $item =~ /^lan_cpu_mem(_tot)?$/ )
  elsif ( $item =~ /^lan_sum_port_channel$/ ) {
    @graph = ( 'lan_sum_port_channel_data_in', 'lan_sum_port_channel_data_out' );
  }
  elsif ( $item =~ /^lan_sum_fexport$/ ) {
    @graph = ( 'lan_sum_fexport_data_in', 'lan_sum_fexport_data_out' );
  }
  elsif ( $item =~ /^lan_sum_vlan$/ ) {
    if ( $st_type eq 'LAN-CISCO' ) {
      if ( glob( "$wrkdir/$host/VLAN*.rrd" ) ) {
        push @graph, 'lan_sum_vlan_unrouted_data_in', 'lan_sum_vlan_unrouted_data_out';
      }
      if ( glob( "$wrkdir/$host/Vl*.rrd" ) ) {
        push @graph, 'lan_sum_vlan_routed_data_in', 'lan_sum_vlan_routed_data_out';
      }
    }
    elsif ( $st_type eq 'LAN-UNKNOWN' ) {
      @graph = ( 'lan_sum_vlan_data_in', 'lan_sum_vlan_data_out' );
    }
  } ## end elsif ( $item =~ /^lan_cpu_mem(_tot)?$/ )

  # print out tabs
  my $tab_number = 0;
  print "<div id=\"tabs\"> <ul>\n";
  foreach $item (<@graph>) {
    # data source back/front end
    my $data_type = "tabfrontend";
    my $tab_name = text_tab( $item, "item" );
    print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
    $tab_number++;
  } ## end foreach $item (<@graph>)
  print "   </ul> \n";

  $tab_number = 0;
  foreach $item (<@graph>) {
    my $item_name = $item;

    print "<div id=\"tabs-$tab_number\"><center>\n";
    if ( $gui == 0 ) {
      print "<center><h3>$item_name</h3></center>";
    }

    print "<table align=\"center\" summary=\"Graphs\">\n";

    print "<tr>\n";
    print_item( $host, $type, $name, $item, "d", $detail_yes );
    print_item( $host, $type, $name, $item, "w", $detail_yes );
    print "</tr><tr>\n";
    print_item( $host, $type, $name, $item, "m", $detail_yes );
    print_item( $host, $type, $name, $item, "y", $detail_yes );
    print "</tr>\n";

    print "</table></center>\n";
    print "</div>\n";
    $tab_number++;
  } ## end foreach $item (<@graph>)

  print "</div><br>\n";
  if ( $gui == 0 ) {
    print "</BODY></HTML>";
  }

  return 1;
} ## end sub create_tab_lan

sub create_tab {
  my ( $host, $type, $name, $st_type, $item ) = @_;
  my $item       = "data_rate";
  my @graph      = "";
  my $graph_indx = 0;

  foreach $item (<@items_sorted>) {

    #print STDERR "01 $item $host $type : $tmp_dir/$host/$type-$item-d.cmd : $QUERY_STRING\n";

    if ( $item =~ m/-subsys/ || $item =~ m/-FC$/ ) {
      next;    # skip Storwize  PORT subsys graphs per port, they are only in agg graphs
    }

    if ( !-f "$tmp_dir/$host/$type-$item-d.cmd" ) {
      if ( !( $st_type eq "NETAPP" && $type eq "QTREE" && $item =~ m/^cifs_ops$|^nfs_ops$/ ) && !( $st_type eq "SWIZ" && $type eq "POOL" && $item =~ m/^data_rate$|^io_rate$/)) {
        next;    # non existing item
      }
    }
    if ($st_type eq "DATADOMAIN" && $type eq "FS" && $name eq "clean" && $item =~ m/^ops_rate$|^data_rate$|^write$|^read$|^read_ops$|^write_ops$/){
      next;
    }
    if ($st_type eq "DATADOMAIN" && $type eq "FS" && $name ne "clean" && $item =~ m/^phase$/){
      next;
    }
    if ($st_type eq "DS8K" && $type eq "POOL" && $item =~ m/^tier_ENT$|^tier_SSD$|^tier_Flash$|^tier_HighCapFlash$|^tier_NL$|^tier_SATA$/){next;}


    if ( $name !~ m/^sum/ && $item =~ m/^sum/  && !( $st_type eq "DATADOMAIN" && $type eq "MTREE" && $item eq "sum_capacity" && -f "$basedir/data/$host/$type/$name.rrc") ) {
      next;      # avoid sum graphs for normail item details
    }

    if ( $item =~ m/^sum_capacity$/ && $st_type =~ m/DS8K/ ) {
      next;
    }
    if ( $item =~ m/^cfc_dirty$|^cfc_max$|^del_ack$/ && $st_type eq "3PAR" && $type eq "NODE" ) {
      next;
    }

    if ( $item =~ m/_cntl$/ ) {
      next;      # controller info is only for aggregated, not for individual pools or volumes
    }


    if ( $type =~ m/^POOL$/ && $st_type eq "VMAX" && $item !~ m/_b$/ && $item !~ m/^used$/ && $item !~ m/^alloc$/ ) { next; }    # only back-end tabs for VMAXs single pool

    #if ( $type eq "POOL" && $st_type eq "STOREONCE" && $name ne "Catalyst" && $item =~ m/job/){
    #  next;  ### job metrics only Catalyst pool
    #}

    # VMAX - FAST-SG (only for SG in $wrkdir/data/$host/FAST-SG/)
    if ( $st_type eq "VMAX" && $type eq "VOLUME" && $item eq "tier_alloc_prct" ) {

      #my @fast_sg_files = <$wrkdir/$host/FAST-SG/$name-*.rrd>;
      #my $fast_sg_count = grep {/$wrkdir\/$host\/FAST-SG\/$name-*.rrd/} @fast_sg_files;
      my $fast_sg_count = grep {/FAST-SG\/$name-.+\.rrd/} <$wrkdir/$host/FAST-SG/$name-*.rrd>;
      if ( defined $fast_sg_count && $fast_sg_count eq "0" ) {
        next;
      }
    } ## end if ( $st_type eq "VMAX"...)

    if ( $type =~ m/^POOL$/ ) {
      if ( $item =~ m/^read$/ && $st_type !~ m/XIV/ && $st_type !~ m/DS8K/ && $st_type !~ m/HUS/ && $st_type !~ m/NETAPP/ && $st_type !~ m/3PAR/ && $st_type !~ m/VSPG/ && $st_type !~ m/^VNX$/ && $st_type !~ m/^VNXF$/ && $st_type !~ m/UNITY/ && $st_type !~ m/^VNXE$/ && $st_type !~ m/^DOTHILL$/ && $st_type !~ m/^COMPELLENT$/ && $st_type !~ m/^VPLEX$/ && $st_type !~ m/^OCEANSTOR$/ && $st_type !~ m/^PURE$/ && $st_type !~ m/^SWIZ$/ && $st_type !~ m/^NIMBLE$/ && $st_type !~ m/^INFINIBOX$/ && $st_type !~ m/^DATADOMAIN$/ && $st_type !~ m/^ETERNUS$/ && $st_type !~ m/^ISILON$/ && $st_type !~ m/^DATACORE$/ && $st_type !~ m/^XTREMIO$/ && $st_type !~ m/^STOREONCE$/ && $st_type !~ m/^STOREVIRTUAL$/ && $st_type !~ m/^CEPH$/ && $st_type !~ m/^PUREBLADE$/ ) {
        $item = "data_rate";
      }
      if ( $item =~ m/^read_io$/ && $st_type !~ m/XIV/ && $st_type !~ m/DS8K/ && $st_type !~ m/HUS/ && $st_type !~ m/NETAPP/ && $st_type !~ m/3PAR/ && $st_type !~ m/VSPG/ && $st_type !~ m/^VNX$/ && $st_type !~ m/^VNXF$/ && $st_type !~ m/UNITY/ && $st_type !~ m/^VNXE$/ && $st_type !~ m/^DOTHILL$/ && $st_type !~ m/^OCEANSTOR$/ && $st_type !~ m/^COMPELLENT$/ && $st_type !~ m/^VPLEX$/ && $st_type !~ m/^PURE$/ && $st_type !~ m/^SWIZ$/ && $st_type !~ m/^NIMBLE$/ && $st_type !~ m/^INFINIBOX$/ && $st_type !~ m/^DATADOMAIN$/ && $st_type !~ m/^ETERNUS$/ && $st_type !~ m/^DATACORE$/ && $st_type !~ m/^XTREMIO$/ && $st_type !~ m/^STOREONCE$/ && $st_type !~ m/^STOREVIRTUAL$/ && $st_type !~ m/^CEPH$/ && $st_type !~ m/^PUREBLADE$/) {
        $item = "io_rate";
      }
      if ( $st_type =~ m/SWIZ/ ) {
        if ( $item !~ m/^read_io_b$/
          && $item !~ m/^write_io_b$/
          && $item !~ m/^read_b$/
          && $item !~ m/^write_b$/
          && $item !~ m/^resp_t_r_b$/
          && $item !~ m/^resp_t_w_b$/
          && $item !~ m/tier/
          && $item !~ m/^sum_capacity$/
          && $item !~ m/^io_rate$/
          && $item !~ m/^data_rate$/
          && $item !~ m/resp/
          && $item !~ m/compress_/
          && $item !~ m/^read_io$/
          && $item !~ m/^write_io$/
          && $item !~ m/^read$/
          && $item !~ m/^write$/ )
        {
          next;
        } ## end if ( $item !~ m/^read_io_b$/...)
      } ## end if ( $st_type =~ m/SWIZ/)
      if ( $st_type =~ m/DS8K/ ) {
        if ( $item !~ m/tier/
          && $item !~ m/^read_io$/
          && $item !~ m/^write_io$/
          && $item !~ m/^read_io_b$/
          && $item !~ m/^write_io_b$/
          && $item !~ m/^resp_t_r$/
          && $item !~ m/^resp_t_w$/
          && $item !~ m/^read$/
          && $item !~ m/^write$/
          && $item !~ m/^read_b$/
          && $item !~ m/^write_b$/
          && $item !~ m/^resp_t_r_b$/
          && $item !~ m/^resp_t_w_b$/
          && $item !~ m/^real$/)
        {
          next;
        } ## end if ( $item !~ m/tier/ ...)
      } ## end if ( $st_type =~ m/DS8K/)
      if ( $st_type =~ m/DS5K/ && $item =~ m/cache/ ) {
        next;
      }
    } ## end if ( $type =~ m/^POOL$/)
    if ( $st_type =~ m/DS8K/ && $type =~ m/^PORT$/ && $item =~ /^pprc/ ) {
      if ( !-f "$basedir/data/$host/$type/$name.rrp" ) {

        # display PPRC tabs only where are some PPRC data (data file *.rrp)
        next;
      }
    } ## end if ( $st_type =~ m/DS8K/...)
    if ( $type =~ m/^HOST$/ && $st_type =~ m/VSPG/ ) {
      my $host_org = host_data_selection( $wrkdir, $host, $type, $st_type );
      if ( $host_org == 1 ) {
        if ( $item !~ /^data_rate$/ && $item !~ /^io_rate$/ && $item !~ /^resp_t$/ ) {
          next;
        }
      }
      else {
        if ( $item =~ /^resp_t$/ ) {
          next;    # volumes do not have resp_t
        }
      }
    } ## end if ( $type =~ m/^HOST$/...)

    # UNITY PORTS
    if ( $st_type =~ m/UNITY/ && $type =~ m/^PORT$/ ) {
      if ( $name =~ /_fc[0-9]+$/ || $name =~ m/iscsinode_/ ) {    #FC ports and iSCSI ports
        if ( $item =~ m/^read_packet$/ || $item =~ m/^write_packet$/ || $item =~ m/^packet_rate$/ ) {
          next;
        }
      }
      if ( $name =~ /_eth[0-9]+$/ && $name !~ m/iscsinode_/ ) {    #ETHERNET ports
        if ( $item =~ m/^read_io$/ || $item =~ m/^write_io$/ || $item =~ m/^io_rate$/ ) {
          next;
        }
      }
    } ## end if ( $st_type =~ m/UNITY/...)

    # VNXE PORTS
    if ( $st_type =~ m/VNXE/ && $type =~ m/^PORT$/ ) {
      if ( $name =~ /_fc[0-9]+$/ || $name =~ m/iscsinode_/ ) {     #FC ports and iSCSI ports
        if ( $item =~ m/^read_packet$/ || $item =~ m/^write_packet$/ || $item =~ m/^packet_rate$/ ) {
          next;
        }
      }
      if ( $name =~ /_eth[0-9]+$/ && $name !~ m/iscsinode_/ ) {    #ETHERNET ports
        if ( $item =~ m/^read_io$/ || $item =~ m/^write_io$/ || $item =~ m/^io_rate$/ ) {
          next;
        }
      }
    } ## end if ( $st_type =~ m/VNXE/...)

    # NIMBLE fc ports have only data statistics
    if ( $st_type eq "NIMBLE" && $type eq "PORT" && $name =~ /fc[0-9]+$/ && ( $item ne "data_rate" && $item ne "read" && $item ne "write" ) ) {
      next;
    }

    # VMAX directors (front-end, backend, rdf)
    if ( $st_type eq "VMAX" && ( $type eq "NODE" || $type eq "PORT" ) && $name =~ m/^[FS][A-Z]-/ ) {
      if ( $item =~ m/_backend$/ || $item =~ m/_rdf$/ ) {
        next;
      }
      if ( $item =~ m/_frontend$/ ) {
        $item =~ s/_frontend$//;
      }
    } ## end if ( $st_type eq "VMAX"...)
    if ( $st_type eq "VMAX" && ( $type eq "NODE" || $type eq "PORT" ) && $name =~ m/^D[A-Z]-/ ) {
      if ( $item =~ m/_frontend$/ || $item =~ m/_rdf$/ ) {
        next;
      }
      if ( $item =~ m/_backend$/ ) {
        $item =~ s/_backend$//;
      }
    } ## end if ( $st_type eq "VMAX"...)
    if ( $st_type eq "VMAX" && ( $type eq "NODE" || $type eq "PORT" ) && $name =~ m/^R[A-Z]-/ ) {
      if ( $item =~ m/_frontend$/ || $item =~ m/_backend$/ ) {
        next;
      }
      if ( $item =~ m/_rdf$/ ) {
        $item =~ s/_rdf$//;
      }
    } ## end if ( $st_type eq "VMAX"...)

    # VPLEX ports (front-end, backend)
    if ( $st_type eq "VPLEX" && $type eq "PORT" && $name =~ m/-[A-B]0-/ ) {
      if ( $item =~ m/_backend$/ ) {
        next;
      }
      if ( $item =~ m/_frontend$/ ) {
        $item =~ s/_frontend$//;
      }
    } ## end if ( $st_type eq "VPLEX"...)
    if ( $st_type eq "VPLEX" && $type eq "PORT" && $name =~ m/-[A-B]1-/ ) {
      if ( $item =~ m/_frontend$/ ) {
        next;
      }
      if ( $item =~ m/_backend$/ ) {
        $item =~ s/_backend$//;
      }
    } ## end if ( $st_type eq "VPLEX"...)

    ### DATADOMAIN ethernet ports do not have io metrics
    if ($st_type eq "DATADOMAIN" && $type eq "PORT" && $item =~ m/^io_rate$|^read_io$|^write_io$/){
      my $check_ethernet_port = check_ethernet_port("$basedir/data/$host/$type/port.cfg", $name);
      if ($check_ethernet_port == 1){next;}
    }

    ###

    if ( $type =~ m/^HOST$/ ) {
      my $wrkdir_space = check_space_name("$basedir/data");
      my $host_space   = check_space_name($host);
      my @files        = <$wrkdir_space/$host_space/$type/*>;
      my $rrd          = grep {/\.rrd$/} @files;
      if ( $rrd == 0 ) {
        if ( !-f "$basedir/data/$host/$type/hosts.cfg" || find_vols( $host, $type, $name ) == 0 ) {

          # when a host does not have any attached volume then print message and exit
          non_existing_data( $host, $name );
          exit(0);
        } ## end if ( !-f "$basedir/data/$host/$type/hosts.cfg"...)
      } ## end if ( $rrd == 0 )

      # add "?" with How it works link
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/host_docu.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
    } ## end if ( $type =~ m/^HOST$/)
    if ( $st_type =~ m/SWIZ/ && $type =~ m/^PORT$/ ) {

      # add "?" with How it works link
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/storwize_ports.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'Port list?\' title=\'Port list\'></a></div>\n";
    }
    if ( $st_type =~ m/3PAR/ && $type =~ m/^VOLUME$/ ) {
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/3PAR-volume.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
    }

    #print STDERR "02 $item $host $type : $tmp_dir/$host/$type-$item-d.cmd : $QUERY_STRING\n";
    $graph[$graph_indx] = $item;
    $graph_indx++;
    if ( $st_type =~ m/SWIZ/ && $item =~ /^sys$/ ) {

      # add compress
      $graph[$graph_indx] = "compress";
      $graph_indx++;
    } ## end if ( $st_type =~ m/SWIZ/...)
  } ## end foreach $item (<@items_sorted>)

  # print out tabs
  my $tab_number = 0;
  print "<div  id=\"tabs\"> <ul>\n";
  foreach $item (<@graph>) {

    # data source back/fron end
    my $data_type = "tabfrontend";
    if ( $item =~ m/_b$/ || $type =~ m/DRIVE/ ) {
      $data_type = "tabbackend";
    }
    if ( $type =~ m/TIER/ && $st_type =~ m/^3PAR/ ) {
      $data_type = "tabbackend";
    }
    my $tab_name = text_tab( $item, "item", $st_type, $type );
    print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
    $tab_number++;
    if ( $type eq "VOLUME" && $st_type eq "3PAR" && $item eq "resp_t_w" && -f "$tmp_dir/$host/LUN-$item-d.cmd" ) {
      $tab_name = text_tab( "resp_t_vlun", "item", $st_type, $type );
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
      $graph[$graph_indx] = "resp_t_vlun";
      $graph_indx++;
      $tab_name = text_tab( "resp_t_r_vlun", "item", $st_type, $type );
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
      $graph[$graph_indx] = "resp_t_r_vlun";
      $graph_indx++;
      $tab_name = text_tab( "resp_t_w_vlun", "item", $st_type, $type );
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
      $graph[$graph_indx] = "resp_t_w_vlun";
      $graph_indx++;

    } ## end if ( $type eq "VOLUME"...)
  } ## end foreach $item (<@graph>)
  print "   </ul> \n";

  $tab_number = 0;
  foreach $item (<@graph>) {
    my $item_name = $item;

    print "<div id=\"tabs-$tab_number\"><center>\n";
    if ( $gui == 0 ) {
      print "<center><h3>$item_name</h3></center>";
    }

    # add "?" with How it works link
    if ( $type =~ m/^NODE$/ && $st_type eq "VNX" && $item eq "dirty_cache_pages" ) {
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/dirty_cache_pages.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
    }
    if ( $type =~ m/^NODE$/ && $st_type eq "CEPH" && $item eq "commit_lat") {
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/Ceph-latency.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
    }
    if ( $type =~ m/^NODE$/ && $st_type eq "CEPH" && $item eq "apply_lat") {
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/Ceph-latency.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
    }
    if ( $type =~ m/^NODE$/ && $st_type eq "CEPH" && $name eq "latency" ) {
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/Ceph-latency.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
    }

    print "<table align=\"center\" summary=\"Graphs\">\n";

    if ($st_type eq "DATADOMAIN" && $type eq "MTREE"){
      if ($item eq "sum_capacity"){
        print "<tr>\n";
        print_item( $host, $type, $name, "local_capacity", "y", $detail_yes );
        print "</tr>\n";
        print "<tr>\n";
        print_item( $host, $type, $name, "local_capacity_trend", "y", $detail_no );
        print "</tr>\n";
      }
      elsif ($item eq "total_comp_factor"){
        print "<tr>\n";
        print_item( $host, $type, $name, $item, "y", $detail_yes );
        print "</tr>\n";
      }
      else{
        print "<tr>\n";
        print_item( $host, $type, $name, $item, "d", $detail_yes );
        print_item( $host, $type, $name, $item, "w", $detail_yes );
        print "</tr><tr>\n";
        print_item( $host, $type, $name, $item, "m", $detail_yes );
        print_item( $host, $type, $name, $item, "y", $detail_yes );
        print "</tr>\n";
      }

    }
    else{

      if ($type eq "DRIVE" && $item eq "busy"){
        $item = "disk_busy";
      }

      print "<tr>\n";
      print_item( $host, $type, $name, $item, "d", $detail_yes );
      print_item( $host, $type, $name, $item, "w", $detail_yes );
      print "</tr><tr>\n";
      print_item( $host, $type, $name, $item, "m", $detail_yes );
      print_item( $host, $type, $name, $item, "y", $detail_yes );
      print "</tr>\n";

    }

    print "</table></center>\n";
    print "</div>\n";
    $tab_number++;
  } ## end foreach $item (<@graph>)

  print "</div><br>\n";
  if ( $gui == 0 ) {
    print "</BODY></HTML>";
  }
}    # create_tab

sub html_base {
  my $refer = shift;

  # Print link to full lpar cfg (must find out at first html_base
  # find out HTML_BASE
  # renove from the path last 3 things
  # http://nim.praha.cz.ibm.com/lpar2rrd/hmc1/PWR6B-9117-MMA-SN103B5C0%20ttt/pool/top.html
  # --> http://nim.praha.cz.ibm.com/lpar2rrd
  my $html_base = "";
  my @full_path = split( /\//, $refer );
  my $k         = 0;
  foreach my $path (@full_path) {
    $k++;
  }
  $k--;
  if ( $refer !~ m/topten-glo/ && $refer !~ m/cpu_max_check/ ) {    # when top10 global then just once
    $k--;
    $k--;
  }
  my $j = 0;
  foreach my $path (@full_path) {
    if ( $j < $k ) {
      if ( $j == 0 ) {
        $html_base .= $path;
      }
      else {
        $html_base .= "/" . $path;
      }
      $j++;
    } ## end if ( $j < $k )
  } ## end foreach my $path (@full_path)

  return $html_base;
} ## end sub html_base

sub make_agg {
  my $host       = shift;
  my $type       = shift;
  my $name       = shift;
  my $item       = shift;
  my $st_type    = shift;
  my $cache_once = 0;



  # list of all items with their priority
  my @items_high = "io data packet error resp cap comp cache busy cpu cache-node pprc operating wafl_exempt-kahuna-exempt-hostOS-nwk_legacy calls other WP tier_alloc_prct tier queue ops transfer_size net total bound repl protocol job memory latency";

  # print out tabs
  my $tab_number = 0;
  print "<div  id=\"tabs\"> <ul>\n";

  # go though all @items_high and check if at least one is available
  my $data_string = "sum_data sum_data_b sum_resp_t sum_resp_t_b data data_rate read write read_b write_b ";

  # create tab header
  foreach my $item_high (<@items_high>) {

    #print STDERR "00 - $item_high - $name -\n";
    if ( $name !~ m/$item_high/ ) {
      next;
    }
    #print STDERR "01 - $item_high - $name -\n";
    foreach my $name_act (<@items_sorted>) {

      #print STDERR "01 $item_high,$name_act \n";
      if ( !-f "$tmp_dir/$host/$type-$name_act-d.cmd" ) {
        next;    # non existing metric
      }
      if ( $item_high !~ m/^cpu$/ && $item_high !~ m/^cap$/ && $name_act !~ m/data/ && $name_act !~ m/^read$/ && $name_act !~ m/^write$/ && $name_act !~ m/$item_high/ && $name_act !~ m/^read_b$/ && $name_act !~ m/^write_b$/ && $name_act !~ m/-subsys$/ && $name_act !~ m/-FC$/ && $item_high !~ m/^cache-node$/ && $item_high !~ m/^operating$/ && $name_act !~ m/cifs_read-3_read-4_read/ && $name_act !~ m/cifs_write-3_write-4_write/ && $name_act !~ m/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ && $name_act !~ m/^b2b/ && $name_act !~ m/^crc/ && $name_act !~ m/^read_frontend$/ && $name_act !~ m/^read_backend$/ && $name_act !~ m/^read_rdf$/ && $name_act !~ m/^write_frontend$/ && $name_act !~ m/^write_backend$/ && $name_act !~ m/^write_rdf$/ && $name_act !~ m/^cfc_max$/ && $name_act !~ m/^del_ack$/ && $name_act !~ m/^cfc_dirty$/ && $name_act !~ m/^hit_ratio$/ && $name_act !~ m/^ssd_hit_ratio$/ && $name_act !~ m/^nfs_ops_rate$/ && $name_act !~ m/^nfsv4_ops_rate$/ && $name_act !~ m/^smb1_rate$/ && $name_act !~ m/^smb2_rate$/ && $name_act !~ m/^apply_lat$/ && $name_act !~ m/^commit_lat$/) {
        next;
      }
      if ( ( $name_act =~ m/^cfc_max$/ || $name_act =~ m/^del_ack$/ || $name_act =~ m/^cfc_dirty$/ ) && $item_high =~ m/^cache$/ ) {
        next;
      }
      if (  $name_act =~ m/^nfs_ops_rate$|nfsv4_ops_rate|smb1_rate|smb2_rate/  && $item_high =~ m/^data$/ ) {
        next;
      }

      if ($name_act =~ m/^hit_ratio$|^ssd_hit_ratio$/ && $item_high !~ m/^cache$/){next;}

      if ( $item_high !~ m/^data$/ && ( $name_act =~ m/cifs_read-3_read-4_read/ || $name_act =~ m/cifs_write-3_write-4_write/ ) ) {

        # Due to NetApp
        next;
      }
      if ( ( $item_high =~ m/data/ || $item_high =~ m/io/ ) && $name_act =~ m/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) {
        next;    # skip kahuna for IO & data graphs
      }

      #if ($type eq "POOL" && $name_act =~ m/job/){
      #  next;  ### STOREONCE skip job graphs
      #}

      if ( $name_act =~ m/data/ || $name_act =~ m/^read$/ || $name_act =~ m/^write$/ || $name_act =~ m/^read_b$/ || $name_act =~ m/^write_b$/ || $name_act =~ m/^read_frontend$/ || $name_act =~ m/^read_backend$/ || $name_act =~ m/^read_rdf$/ || $name_act =~ m/^write_frontend$/ || $name_act =~ m/^write_backend$/ || $name_act =~ m/^write_rdf$/ ) {
        if ( $data_string !~ m/$item_high / && $name_act !~ m/pprc/ && $name_act !~ m/-subsys$/ && $name_act !~ m/-FC$/ && $name_act !~ m/job$/ ) {
          next;
        }
      }
      if ($name_act  =~ m/read_data_job|write_data_job/ && $item_high eq "data"){
        next;
      }

      if ( $name_act =~ m/-subsys$/ && $item_high =~ m/io/ ) {
        if ( $name_act !~ m/io_rate-subsys/ && $name_act !~ m/write_io-subsys/ && $name_act !~ m/read_io-subsys/ ) {
          next;
        }
      }
      if ( $name_act =~ m/-subsys$/ && $item_high =~ m/data/ ) {
        if ( $name_act !~ m/data_rate-subsys/ && $name_act !~ m/write-subsys/ && $name_act !~ m/read-subsys/ ) {
          next;
        }
      }
      if ( $name_act =~ m/-FC$/ && $item_high =~ m/io/ ) {
        if ( $name_act !~ m/io_rate-FC/ && $name_act !~ m/write_io-FC/ && $name_act !~ m/read_io-FC/ ) {
          next;
        }
      }
      if ( $name_act =~ m/-FC$/ && $item_high =~ m/data/ ) {
        if ( $name_act !~ m/data_rate-FC/ && $name_act !~ m/write-FC/ && $name_act !~ m/read-FC/ ) {
          next;
        }
      }

      if ( $name_act =~ m/-subsys$/ && $item_high =~ m/resp/ ) {
        next;
      }
      if ( $name_act =~ m/-FC$/ && $item_high =~ m/resp/ ) {
        next;
      }

      if ( $item_high =~ m/operating/ && $name_act !~ m/operating/ ) {
        next;
      }

      if ( $item_high =~ m/^other$/ && $name_act !~ m/^b2b/ && $name_act !~ m/^crc/ ) {
        next;    # SWIZ PORT "other" : b2b & crc
      }
      if ( ( $item_high =~ m/^io$/ || $item_high =~ m/^data$/ ) && ( $name_act =~ m/^b2b/ || $name_act =~ m/^crc/ ) ) {
        next;    # SWIZ PORT "other" : b2b & crc
      }

      #print STDERR "04 $item_high,$name_act: $item_high - $data_string\n";
      # capacity POOL
      if ( $item_high =~ m/^cap$/ ) {
        if ( $name_act !~ m/^tier/ && $name_act !~ m/^used$/ && $name_act !~ m/^sum_capacity$/ && $name_act !~ m/^real$/ && $name_act !~ m/^alloc$/ && $name_act !~ m/^sum_capacity_aggregated$/ && $name_act !~ m/total_comp_factor/ && $name_act !~ m/^virt$/ ) {
          next;
        }
      }
      if ( $item_high =~ m/comp/ ) {
        if ( $name_act !~ m/compress_cap/ && $name_act !~ m/compress_virt/ ) {
          next;
        }
      }

      #print STDERR "05 $item_high,$name_act: $item_high - $data_string\n";

      # port and data_rate && io_rate vrs PPRC ....
      if ( $item_high =~ m/data/ || $item_high =~ m/io/ ) {
        if ( $type =~ /PORT/ && $name_act =~ m/pprc/ ) {
          next;
        }
      }

      if ( $type =~ m/^HOST$/ && $st_type =~ m/VSPG/ && $name_act !~ /^data_rate$/ && $name_act !~ /^io_rate$/ && $name_act !~ /^resp_t$/ ) {
        next;    # agg graphs only for native host data, there might be touched fake host cmd files for volume usage, then skip then and use only real one for host
      }
      if ( $type =~ m/^NODE-CACHE$/ && $st_type =~ m/VNXF/ && $item_high =~ m/^cache$/ ) {
        next;
      }
      if ( $type =~ m/^NODE$/ && $st_type =~ m/COMPELLENT|OCEANSTOR/ && $item_high eq "cpu" && $name_act !~ /^sys$/ ) {
        ### compellent node cpu
        next;
      }

      # VPLEX POOL (only "^sum_" graphs with read and write)
      if ( $type eq "POOL" && $st_type eq "VPLEX" && $name_act !~ m/^sum_/ ) {
        next;
      }

      ### COMPELLENT POOL
      if ( $type eq "POOL" && $st_type eq "COMPELLENT" && $name_act =~ m/read_io|data_rate|write_io|read|write|io_rate|real/ ) {
        next;
      }

      ### DATADOMAIN POOL
      if ( $type eq "POOL" && $st_type eq "DATADOMAIN" && $name_act =~ m/read_io|data_rate|write_io|read|write|io_rate|real|read_repl|write_repl|repl_rate/ ) {
        next;
      }
      if ( $type eq "POOL" && $st_type eq "DATADOMAIN" && $name_act eq "resp_t" && -f "$wrkdir/$host/DATADOMAIN-V6"){
        next;
      }
      if ( $type eq "POOL" && $st_type eq "DATADOMAIN" && $name_act eq "sum_io_total" && -f "$wrkdir/$host/DATADOMAIN-V6"){
        next;
      }

      if ( $type eq "DDBOOST" && $st_type eq "DATADOMAIN" && $name_act =~ m/read_net|data_rate|write_net|read|write|net_rate|outbound_pre_comp|inbound_pre_comp|outbound_net|inbound_net/ ) {
        next;
      }
      if ($type eq "DDBOOST" && $st_type eq "DATADOMAIN" && $name_act =~ m/sum_bound_net/ && $name eq "net" ){
        next;
      }

      # only sum and cap graphs for XTREMIO total pool
      if ( $type eq "POOL" && $st_type eq "XTREMIO" && ($name_act =~ m/_rate$/ || $name_act =~ m/^read/ || $name_act =~ m/^write/ || $name_act =~ m/^resp_t/) ) { next;}



      ### for PURE pool only sum_io sum_data
      if ($st_type eq "PURE" && $type eq "POOL" && $name_act =~ m/write_io|read_io|io_rate|data_rate|write|read/){next;}

      ## for PUREBLADE pool, nsf, smb, s3
      if ($st_type eq "PUREBLADE" && $type =~ m/POOL|NFS|SMB|S3/ && $name_act =~ m/write_io|read_io|io_rate|data_rate|write|read/){next;}
      if ($st_type eq "PUREBLADE" && $type =~ m/S3/ && $name_act =~ m/^resp_t$/){next;}

      # found it, create the page and tabs

      # data source back/fron end
      my $data_type = "tabfrontend";
      #if ( $type =~ /^POOL$/ && $st_type !~ m/^DS8K$/ && $st_type !~ m/^SWIZ$/ ) {
      #  $data_type = "tabbackend";
      #} --> all POOL data is front-end!!!
      if ( $st_type =~ m/^DS5K$/ || $st_type =~ m/^HUS$/ || $st_type =~ m/^NETAPP/ || $st_type =~ m/^3PAR/ || $st_type =~ m/^VSPG/ || $st_type =~ m/^VNX/ ) {
        $data_type = "tabfrontend";
      }
      if ( $type =~ /^RANK$/ || $type =~ /DRIVE/ || $name_act =~ m/_b$/ ) {
        $data_type = "tabbackend";
      }
      if ( $type =~ /^NODE-CACHE$/ ) {
        $data_type = "tabfrontend";
      }
      if ( $st_type =~ m/^VMAX$/ && $type =~ /^POOL$/ && $name_act !~ m/_b$/ ) {
        $data_type = "tabfrontend";
      }
      if ( $type =~ m/TIER/ && $st_type =~ m/^3PAR/ ) {
        $data_type = "tabbackend";
      }
      if ( $item_high =~ m/^cache$/ && $cache_once == 0 ) {

        # cache for storwirwize, table, not graphs
        if ( -f "$webdir/$host/cache-r_cache_hit.html" ) {
          print "  <li class=\"$data_type\"><a href=\"$host/cache-r_cache_hit.html\">read hit avrg</a></li>\n";
        }
        if ( -f "$webdir/$host/cache-w_cache_hit.html" ) {
          print "  <li class=\"$data_type\"><a href=\"$host/cache-w_cache_hit.html\">write hit avrg</a></li>\n";
        }
        if ( -f "$webdir/$host/cache-cache_hit.html" ) {
          print "  <li class=\"$data_type\"><a href=\"$host/cache-cache_hit.html\">cache hit avrg</a></li>\n";
        }
        if ( -f "$webdir/$host/cache-read_pct.html" ) {
          print "  <li class=\"$data_type\"><a href=\"$host/cache-read_pct.html\">read percent avrg</a></li>\n";
        }
        if ( -f "$webdir/$host/cache-ssd_r_cache_hit.html" ) {
          print "  <li class=\"$data_type\"><a href=\"$host/cache-ssd_r_cache_hit.html\">SSD read cache hit avrg</a></li>\n";
        }
        $cache_once++;
      } ## end if ( $item_high =~ m/^cache$/...)
      if ( $st_type =~ m/SWIZ/ && $type =~ m/VOLUME/ && $name_act =~ m/^r_cache_hit$/ ) {

        # workaround for "total usage" as name does not contain cache and it must be under cache menu item
        my $tab_name = text_tab( "usage", "aggregated", $st_type, $type );
        print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
        $tab_number++;
      } ## end if ( $st_type =~ m/SWIZ/...)
      my $tab_name = text_tab( $name_act, "aggregated", $st_type, $type );
      #print STDERR "$tab_name $name_act $st_type $type\n";
      print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
      $tab_number++;
      if ($st_type eq "DATADOMAIN" && $name_act eq "sum_capacity" && -f "$tmp_dir/$host/TIER-sum_capacity-d.cmd" && $type eq "POOL"){
        my $tab_name = text_tab( $name_act, "aggregated", $st_type, "TIER" );
        print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
        $tab_number++;
      }
    } ## end foreach my $name_act (<@items_sorted>)
  } ## end foreach my $item_high (<@items_high>)
  print "</ul>\n";
  my $tab_number = 0;
  $cache_once = 0;

  # create body of the tabs
  foreach my $item_high (<@items_high>) {
    if ( $name !~ m/$item_high/ ) {
      next;
    }
    foreach my $name_act (<@items_sorted>) {
      if ( !-f "$tmp_dir/$host/$type-$name_act-d.cmd" ) {
        next;    # non existing metric
      }

      if ( $item_high !~ m/^cpu$/ && $item_high !~ m/^cap$/ && $name_act !~ m/data/ && $name_act !~ m/^read$/ && $name_act !~ m/^write$/ && $name_act !~ m/$item_high/ && $name_act !~ m/^read_b$/ && $name_act !~ m/^write_b$/ && $name_act !~ m/-subsys$/ && $name_act !~ m/-FC$/ && $item_high !~ m/^cache-node$/ && $item_high !~ m/^operating$/ && $name_act !~ m/cifs_read-3_read-4_read/ && $name_act !~ m/cifs_write-3_write-4_write/ && $name_act !~ m/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ && $name_act !~ m/^b2b/ && $name_act !~ m/^crc/ && $name_act !~ m/^read_frontend$/ && $name_act !~ m/^read_backend$/ && $name_act !~ m/^read_rdf$/ && $name_act !~ m/^write_frontend$/ && $name_act !~ m/^write_backend$/ && $name_act !~ m/^write_rdf$/ && $name_act !~ m/^cfc_max$/ && $name_act !~ m/^del_ack$/ && $name_act !~ m/^cfc_dirty$/ && $name_act !~ m/^hit_ratio$/ && $name_act !~ m/^ssd_hit_ratio$/ && $name_act !~ m/^nfs_ops_rate$/ && $name_act !~ m/^nfsv4_ops_rate$/ && $name_act !~ m/^smb1_rate$/ && $name_act !~ m/^smb2_rate$/ && $name_act !~ m/^apply_lat$/ && $name_act !~ m/^commit_lat$/) {
        next;
      }
      if ( ( $name_act =~ m/^cfc_max$/ || $name_act =~ m/^del_ack$/ || $name_act =~ m/^cfc_dirty$/ ) && $item_high =~ m/^cache$/ ) {
        next;
      }
      if ($name_act =~ m/^hit_ratio$|^ssd_hit_ratio$/ && $item_high ne "cache" ){next;}


      if ( $item_high !~ m/^data$/ && ( $name_act =~ m/cifs_read-3_read-4_read/ || $name_act =~ m/cifs_write-3_write-4_write/ ) ) {

        # Due to NetApp
        next;
      }
      if (  $name_act =~ m/^nfs_ops_rate$|nfsv4_ops_rate|smb1_rate|smb2_rate/  && $item_high =~ m/^data$/ ) {
        next;
      }
      if ( ( $item_high =~ m/data/ || $item_high =~ m/io/ ) && $name_act =~ m/wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) {
        next;    # skip kahuna for IO & data graphs
      }
      if ( $name_act =~ m/data/ || $name_act =~ m/^read$/ || $name_act =~ m/^write$/ || $name_act =~ m/^read_b$/ || $name_act =~ m/^write_b$/ || $name_act =~ m/^read_frontend$/ || $name_act =~ m/^read_backend$/ || $name_act =~ m/^read_rdf$/ || $name_act =~ m/^write_frontend$/ || $name_act =~ m/^write_backend$/ || $name_act =~ m/^write_rdf$/ ) {
        if ( $data_string !~ m/$item_high / && $name_act !~ m/pprc/ && $name_act !~ m/-subsys$/ && $name_act !~ m/-FC$/ && $name_act !~ m/job$/ ) {
          next;
        }
      }
      if ($name_act  =~ m/read_data_job|write_data_job/ && $item_high eq "data"){
        next;
      }
      if ( $type =~ m/^NODE-CACHE$/ && $st_type =~ m/VNXF/ && $item_high =~ m/^cache$/ ) {
        next;
      }

      if ( $name_act =~ m/-subsys$/ && $item_high =~ m/io/ ) {
        if ( $name_act !~ m/io_rate-subsys/ && $name_act !~ m/write_io-subsys/ && $name_act !~ m/read_io-subsys/ ) {
          next;
        }
      }
      if ( $name_act =~ m/-subsys$/ && $item_high =~ m/data/ ) {
        if ( $name_act !~ m/data_rate-subsys/ && $name_act !~ m/write-subsys/ && $name_act !~ m/read-subsys/ ) {
          next;
        }
      }
      if ( $name_act =~ m/-FC$/ && $item_high =~ m/io/ ) {
        if ( $name_act !~ m/io_rate-FC/ && $name_act !~ m/write_io-FC/ && $name_act !~ m/read_io-FC/ ) {
          next;
        }
      }
      if ( $name_act =~ m/-FC$/ && $item_high =~ m/data/ ) {
        if ( $name_act !~ m/data_rate-FC/ && $name_act !~ m/write-FC/ && $name_act !~ m/read-FC/ ) {
          next;
        }
      }

      if ( $name_act =~ m/-subsys$/ && $item_high =~ m/resp/ ) {
        next;
      }
      if ( $name_act =~ m/-FC$/ && $item_high =~ m/resp/ ) {
        next;
      }

      if ( $item_high =~ m/operating/ && $name_act !~ m/operating/ ) {
        next;
      }
      if ( $item_high =~ m/^other$/ && $name_act !~ m/^b2b/ && $name_act !~ m/^crc/ ) {
        next;    # SWIZ PORT "other" : b2b & crc
      }
      if ( ( $item_high =~ m/^io$/ || $item_high =~ m/^data$/ ) && ( $name_act =~ m/^b2b/ || $name_act =~ m/^crc/ ) ) {
        next;    # SWIZ PORT "other" : b2b & crc
      }

      # capacity POOL
      if ( $item_high =~ m/^cap$/ ) {
        if ( $name_act !~ m/^tier/ && $name_act !~ m/^used$/ && $name_act !~ m/^sum_capacity$/ && $name_act !~ m/^real$/ && $name_act !~ m/^alloc$/ && $name_act !~ m/^sum_capacity_aggregated$/ && $name_act !~ m/^total_comp_factor$/ && $name_act !~ m/^virt$/ ) {
          next;
        }
      }
      if ( $item_high =~ m/comp/ ) {
        if ( $name_act !~ m/compress_cap/ && $name_act !~ m/compress_virt/ ) {
          next;
        }
      }

      # port and data_rate && io_rate vrs PPRC ....
      if ( $item_high =~ m/data/ || $item_high =~ m/io/ ) {
        if ( $type =~ /PORT/ && $name_act =~ m/pprc/ ) {
          next;
        }
      }
      if ( $type =~ m/^HOST$/ && $st_type =~ m/VSPG/ && $name_act !~ /^data_rate$/ && $name_act !~ /^io_rate$/ && $name_act !~ /^resp_t$/ ) {
        next;    # agg graphs only for native host data, there might be touched fake host cmd files for volume usage, then skip then and use only real one for host
      }
      if ( $type =~ m/^NODE$/ && $st_type =~ m/COMPELLENT|OCEANSTOR/ && $item_high eq "cpu" && $name_act !~ /^sys$/ ) {
        ### compellent node cpu
        next;
      }

      # VPLEX POOL (only "^sum_" graphs with read and write)
      if ( $type eq "POOL" && $st_type eq "VPLEX" && $name_act !~ m/^sum_/ ) {
        next;
      }
      ### COMPELLENT POOL
      if ( $type eq "POOL" && $st_type eq "COMPELLENT" && $name_act =~ m/read_io|data_rate|write_io|read|write|io_rate|real/ ) {
        next;
      }

      ### for PURE pool only sum_io
      if ($st_type eq "PURE" && $type eq "POOL" && $name_act =~ m/write_io|read_io|io_rate|data_rate|write|read/){next;}

      ### for PUREBLADE pool, nfs, smb, s3 only sum_io
      if ($st_type eq "PUREBLADE" && $type =~ m/POOL|NFS|SMB|S3/ && $name_act =~ m/write_io|read_io|io_rate|data_rate|write|read/){next;}
      if ($st_type eq "PUREBLADE" && $type =~ m/S3/ && $name_act =~ m/^resp_t$/){next;}


      ### DATADOMAIN POOL
      if ( $type eq "POOL" && $st_type eq "DATADOMAIN" && $name_act =~ m/read_io|data_rate|write_io|read|write|io_rate|real|repl_rate/ ) {
        next;
      }
      if ( $type eq "POOL" && $st_type eq "DATADOMAIN" && $name_act eq "resp_t" && -f "$wrkdir/$host/DATADOMAIN-V6"){
        next;
      }
      if ( $type eq "POOL" && $st_type eq "DATADOMAIN" && $name_act eq "sum_io_total" && -f "$wrkdir/$host/DATADOMAIN-V6"){
        next;
      }
      if ( $type eq "DDBOOST" && $st_type eq "DATADOMAIN" && $name_act =~ m/read_net|data_rate|write_net|read|write|net_rate|outbound_pre_comp|inbound_pre_comp|outbound_net|inbound_net/ ) {
        next;
      }
      if ($type eq "DDBOOST" && $st_type eq "DATADOMAIN" && $name_act =~ m/sum_bound_net/ && $name eq "net" ){
        next;
      }

      # found it, create the page and tabs

      # data source back/fron end
      my $data_type = "tabfrontend";
      if ( $type =~ /^POOL$/ && $st_type !~ m/^DS8K$/ && $st_type !~ m/^SWIZ$/ ) {
        $data_type = "tabbackend";
      }
      if ( $st_type =~ m/^DS5K$/ || $st_type =~ m/^HUS$/ || $st_type =~ m/^NETAPP/ || $st_type =~ m/^3PAR/ || $st_type =~ m/^VSPG/ || $st_type =~ m/^VNX/ ) {
        $data_type = "tabfrontend";
      }
      if ( $type =~ /^RANK$/ || $name_act =~ m/_b$/ || $type =~ /DRIVE/ ) {
        $data_type = "tabbackend";
      }
      if ( $type =~ /^NODE-CACHE$/ ) {
        $data_type = "tabfrontend";
      }
      if ( $type =~ m/TIER/ && $st_type =~ m/^3PAR/ ) {
        $data_type = "tabbackend";
      }

      if ( $st_type =~ m/SWIZ/ && $type =~ m/VOLUME/ && $name_act =~ m/^r_cache_hit$/ ) {

        # workaround for "total usage" as name does not contain cache and it must be under cache menu item
        print "<div id=\"tabs-$tab_number\"><center>\n";
        $tab_number++;
        create_tab_agg( $item_high, $host, $type, "total_usage", $st_type, $item );
        print "</div>\n";
      } ## end if ( $st_type =~ m/SWIZ/...)

      print "<div id=\"tabs-$tab_number\"><center>\n";
      $tab_number++;
      create_tab_agg( $item_high, $host, $type, $name_act, $st_type, $item );
      print "</div>\n";

      if ($st_type eq "DATADOMAIN" && $item_high eq "cap" && -f "$tmp_dir/$host/TIER-sum_capacity-d.cmd" && $type eq "POOL"){
        print "<div id=\"tabs-$tab_number\"><center>\n";
        $tab_number++;
        create_tab_agg( $item_high, $host, "TIER", $name_act, $st_type, $item );
        print "</div>\n";
      }

    } ## end foreach my $name_act (<@items_sorted>)
  } ## end foreach my $item_high (<@items_high>)
  print "</div>\n";

} ## end sub make_agg

sub create_tab_agg {
  my ( $item_high, $host, $type, $name, $st_type, $item ) = @_;

  if ( $st_type =~ m/3PAR/ && $type =~ m/^POOL$/ && ( $name =~ m/^read/ || $name =~ m/^write/ || $name =~ m/^resp_t_w/ || $name =~ m/^resp_t_r/ ) ) {
    print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/HPE-3PAR-pool.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
  }
  if ( $st_type =~ m/NETAPP/ && ( $type =~ m/VOLUME/ || $type =~ m/POOL/ || $type =~ m/NODE/ || $type =~ m/LUN/ ) && $name =~ m/io/ ) {

    # add "?" with How it works link : netapp oothers IOPS
    print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/NetApp-IO.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
  }

  # add "?" with How it works link
  if ( $type =~ m/^NODE$/ && $st_type eq "VNX" && $name eq "dirty_cache_pages" ) {
    print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/dirty_cache_pages.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
  }
  if ( $st_type =~ m/SWIZ/ && $type =~ m/^PORT$/ ) {
    if ( $name =~ m/b2b/ ) {
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/storwize_b2b.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'Port list?\' title=\'Zero B2B\'></a></div>\n";
    }
    else {
      print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/storwize_ports.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'Port list?\' title=\'Port list\'></a></div>\n";
    }
  } ## end if ( $st_type =~ m/SWIZ/...)
  if ( $st_type =~ m/3PAR/ && $type =~ m/^VOLUME$/ ) {
    print "<div id=\'hiw\'><a href=\'http://www.stor2rrd.com/3PAR-volume.htm\' target=\'_blank\'><img src=\'css/images/help-browser.gif\' alt=\'How it works?\' title=\'How it works?\'></a></div>\n";
  }
  # DIV for page re-grouping (by Date <-> by metric)
  my $rg_class = "";
  if ($name =~ m/^resp_t$|^resp_t_r$|^resp_t_w$|^data_rate$|^read$|^write$|^io_rate$|^read_io$|^write_io$|^sum_io|^sum_data$|^sum_resp_t$/){
    print "<div class=\'regroup fwd\'><a href=\'$ENV{WEB_CGI_PATH}/detail.sh?host=$host_url&type=$type&name=$name&storage=$st_type&item=$item&gui=2&none=none::\' alt=\'Regroup\' title=\'Regroup\'></a></div>\n";
  }
  print "<table align=\"center\" summary=\"Graphs\">\n";

  if ($st_type eq "DATADOMAIN" && $type eq "MTREE" ){
    if ($name eq "sum_capacity"){
      print "<tr>\n";
      #print_item( $host, $type, $name, $item, "y", $detail_yes );
      print_item( $host, $type, "local_capacity", $item, "y", $detail_yes );
      print "</tr>\n";
      print_item( $host, $type, "$name\_trend", $item, "y", $detail_no );
    }
    elsif ($name eq "total_comp_factor"){
      print "<tr>\n";
      print_item( $host, $type, "$name", $item, "y", $detail_yes );
      print "</tr>\n";
    }
    else{
      print "<tr>\n";
      print_item( $host, $type, $name, $item, "d", $detail_yes );
      print_item( $host, $type, $name, $item, "w", $detail_yes );
      print "</tr><tr>\n";
      print_item( $host, $type, $name, $item, "m", $detail_yes );
      print_item( $host, $type, $name, $item, "y", $detail_yes );
      print "</tr>\n";
    }

  }
  else{
    if ($type eq "DRIVE" && $name eq "busy"){
      $name = "disk_busy";
    }

  print "<tr>\n";
  print_item( $host, $type, $name, $item, "d", $detail_yes );
  print_item( $host, $type, $name, $item, "w", $detail_yes );
  print "</tr><tr>\n";
  print_item( $host, $type, $name, $item, "m", $detail_yes );
  print_item( $host, $type, $name, $item, "y", $detail_yes );
  print "</tr>\n";

  }

  if ( $name =~ /sum_io/ || $name =~ /sum_data/ ) {    # POOL total trend
    print_item( $host, $type, "$name\_trend", $item, "y", $detail_no );
  }
  if ( ($name eq "used" || $name eq "real" || $name eq "virt") && $item eq "sum" ) {           # POOL used capacity trend
    print_item( $host, $type, "$name\_trend", $item, "y", $detail_no );
  }


  print "</table>\n";

  return 1;
} ## end sub create_tab_agg

sub print_item {
  my ( $host, $type, $name, $item, $time, $detail, $re_group ) = @_;
  my $refresh      = "";
  my $legend_class = "nolegend";

  if ( $item !~ /^sum/ ) {
    $legend_class = "";
  }
  if ( $name =~ /^sum/ ) {
    $legend_class = "";
  }
  if ( ( $name =~ m/^sum_data$/ )
    || ( $name =~ m/^sum_data_b$/ )
    || ( $name =~ m/^sum_io$/ )
    || ( $name =~ m/^sum_resp/ )
    || ( $name =~ m/^sum_io_b$/ )
    || ( $name =~ m/^sum_io_total$/ )
    || ( $name =~ m/^sum_data_total$/ )
    || ( $name =~ m/^sum_capacity$/ )
    || ( $name =~ m/compress_cap/ )
    || ( $name =~ m/uncompress_cap/ )
    || ( $name =~ m/uncompress_virt_cap/ )
    || ( $name =~ m/^tier0$/ )
    || ( $name =~ m/^tier0-nl$/ )
    || ( $name =~ m/^tier1$/ )
    || ( $name =~ m/^tier2$/ )
    || ( $name =~ m/^io_rate-subsys$/ )
    || ( $name =~ m/^data_rate-subsys$/ )
    || ( $name =~ m/^read_io-subsys$/ )
    || ( $name =~ m/^write_io-subsys$/ )
    || ( $name =~ m/^read-subsys$/ )
    || ( $name =~ m/^write-subsys$/ )
    || ( $name =~ m/^used$/ )
    || ( $name =~ m/^real$/ )
    || ( $name =~ m/^sys$/ )
    || ( $name =~ m/^busy$/ )
    || ( $name =~ m/^sum_busy/ )
    || ( $name =~ m/^alloc$/ )
    || ( $name =~ m/^WP$/ )
    || ( $name =~ m/^sum_net$/ )
    || ( $name =~ m/^memory$/ )
    || ( $name =~ m/^virt$/ )
    || ( $name =~ m/^tier_ENT$|^tier_SSD$|^tier_Flash$|^tier_HighCapFlash$|^tier_NL$|^tier_SATA$/ ) )
  {
    $legend_class = "";
  } ## end if ( ( $name =~ m/^sum_data$/...))
  if ( $st_type eq "VMAX" && $type eq "POOL" && $name !~ m/_b$/ ) {
    $legend_class = "";
  }
  if ( $st_type eq "VMAX" && ( $type eq "PORT" || $type eq "NODE" ) && $name eq "busy" ) {
    $legend_class = "nolegend";
  }
  if ( $st_type eq "VPLEX" && $type eq "NODE" && $name eq "busy" ) {
    $legend_class = "nolegend";
  }
  if ( $st_type eq "DATADOMAIN" && $type eq "DRIVE" && $name eq "busy" ) {
    $legend_class = "nolegend";
  }
  if ( $st_type eq "ETERNUS" && $type eq "NODE" && $name eq "busy" ) {
    $legend_class = "nolegend";
  }
  if ( $st_type eq "ETERNUS" && $type eq "DRIVE" && $name eq "busy" ) {
    $legend_class = "nolegend";
  }
  if ( $st_type eq "ISILON" && $type eq "DRIVE" && $name eq "busy" ) {
    $legend_class = "nolegend";
  }
  if ( $st_type eq "ISILON" && $type eq "CPU-CORE" && $name eq "busy" ) {
    $legend_class = "nolegend";
  }
  if ( $st_type eq "DATADOMAIN" && $type eq "MTREE" && $name eq "sum_capacity" ) {
    $legend_class = "nolegend";
  }


  if ( $type =~ m/HOST/ ) {
    $legend_class = "nolegend";
  }
  if ( $type =~ m/HOST/ && $item !~ /^sum/ && ( $st_type eq "VMAX" || $st_type eq "COMPELLENT" || $st_type eq "OCEANSTOR" || $st_type eq "3PAR" || $st_type eq "VSPG" || $st_type eq "PURE" || $st_type eq "INFINIBOX" || $st_type eq "DATADOMAIN" || $st_type eq "DATACORE" || $st_type eq "XTREMIO" ) ) {
    $legend_class = "";
  }
  if ( $type =~ m/^HOST$/ && $st_type =~ m/VSPG/ ) {
    my $host_org = host_data_selection( $wrkdir, $host, $type, $st_type );
    if ($host_org == 0){
      $legend_class = "nolegend";
    }
  }
  if ( $st_type eq "VMAX" && $type eq "VOLUME" && $item eq "tier_alloc_prct" ) {
    $legend_class = "nolegend";
  }
  if ( $st_type =~ "^SAN-" ) {
    if ( $item eq "san_data" || $item eq "san_io" || $item eq "san_io_size" || $item eq "san_errors" || $item eq "san_credits" || $item eq "san_cpu" || $item eq "san_mem" || $item eq "san_enc_out" || $item eq "san_disc_c3" || $item eq "san_link_fail" || $item eq "san_loss_sync" ) {
      $legend_class = "";
    }
    else {
      $legend_class = "nolegend";
    }
  } ## end if ( $st_type =~ "^SAN-")
  # my @graph = ( "$item\_data", "$item\_Ucast_pkts", "$item\_Mcast_pkts", "$item\_Bcast_pkts", "$item\_errors" );
  if ( $st_type =~ /^LAN-(CISCO|UNKNOWN)$/ ) {
    if ( $item eq "lan_data" || $item eq "lan_Ucast_pkts" || $item eq "lan_Mcast_pkts" || $item eq "lan_Bcast_pkts" || $item eq "lan_errors" ) {
      $legend_class = "";
    }
    else {
      $legend_class = "nolegend";
    }
  }
  if ( $item =~ "^custom-group" ) {
    $legend_class = "nolegend";
  }
  if ( $item =~ /^custom-group.*-trend$/ ) {
    $legend_class = "";
  }

  # It must be here otherwise does notwork for example "#" in the $name
  $name =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;
  $host =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;

  #print STDERR "009 print_item enter:$host,$type,$name,$item,$time,$detail,$legend_class,\n";

  my $html_title_detail = 9;
  if (defined $re_group && $re_group eq "re-group"){
    $html_title_detail = 8; # re-group has different graph size
  }
  my $centered_class = "";
  if (($name eq "sum_capacity_trend" || $name eq "local_capacity") && $item eq "sum" && $type eq "MTREE"){
    $centered_class = "centered";
  }
  if ( $detail > 0 ) {
    print "          <td class=\"relpos $centered_class\">
            <div>
              <div class=\"g_title\">
                <div class=\"favs favoff\"></div>
                $refresh
                <div class=\"popdetail\"></div>
                <div class=\"g_text\" data-server=\"$host\" data-name=\"$name\" data-type=\"$type\" data-item=\"$item\" data-time=\"$time\"><span class=\"tt_span\"></span></div>
              </div>
              <a class=\"detail\" href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=$item&time=$time&detail=1&none=$time_u\">
                <div title=\"Click to show detail\"><img class=\"lazy $legend_class\" border=\"0\" data-src=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=$item&time=$time&detail=$html_title_detail&none=$time_u\" src=\"css/images/sloading.gif\">
                  <div class=\"zoom\" title=\"Click and drag to select range\"></div>
                </div>
              </a>
              <div class=\"legend\"></div>
              <div class=\"updated\"></div>
            </div>
          </td>\n";
  } ## end if ( $detail > 0 )
  else {
    print "<td align=\"center\" valign=\"top\" colspan=\"2\"><div><img class=\"lazy\" border=\"0\" data-src=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=$item&time=$time&detail=0&none=$time_u\" src=\"css/images/sloading.gif\"></div></td>\n";
  }
  return 1;
} ## end sub print_item

sub basename {
  my $full = shift;
  my $out  = "";

  # basename without direct function
  my @base = split( /\//, $full );
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
} ## end sub basename

sub text_tab {
  my $item     = shift;
  my $type_out = shift;
  my $st_type  = shift;
  my $type     = shift;
  my $text_out = $item;

  if ( $type_out =~ m/^aggregated$/ ) {
    if ( $item =~ m/^packet_rate$/ )                                { $text_out = "total"; }
    if ( $item =~ m/^read_packet$/ )                                { $text_out = "read"; }
    if ( $item =~ m/^write_packet$/ )                               { $text_out = "write"; }
    if ( $item =~ m/^read_error$/ )                                 { $text_out = "read"; }
    if ( $item =~ m/^write_error$/ )                                { $text_out = "write"; }
    if ( $item =~ m/^io_rate_b$/ )                                  { $text_out = "total back"; }
    if ( $item =~ m/^cifs_io_rate-3_io_rate-4_io_rate/ )            { $text_out = "Protocols total"; }
    if ( $item =~ m/^cifs_read_io-3_read_io-4_read_io/ )            { $text_out = "Protocols read"; }
    if ( $item =~ m/^cifs_write_io-3_write_io-4_write_io/ )         { $text_out = "Protocols write"; }
    if ( $item =~ m/^cifs_data_rate-3_data_rate-4_data_rate/ )      { $text_out = "Protocols total"; }
    if ( $item =~ m/^cifs_read-3_read-4_read/ )                     { $text_out = "Protocols read"; }
    if ( $item =~ m/^cifs_write-3_write-4_write/ )                  { $text_out = "Protocols write"; }
    if ( $item =~ m/^cifs_resp_t-3_resp_t-4_resp_t/ )               { $text_out = "Protocols total"; }
    if ( $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) { $text_out = "CPU"; }
    if ( $item =~ m/^sum_io$/ )                                     { $text_out = "total"; }
    if ( $item =~ m/^sum_io_b$/ )                                   { $text_out = "total back"; }
    if ( $item =~ m/^sum_io_total$/ )                               { $text_out = "total"; }
    if ( $item =~ m/^sum_data$/ )                                   { $text_out = "total"; }
    if ( $item =~ m/^sum_repl$/ )                                   { $text_out = "total"; }
    if ( $item =~ m/^sum_bound_pre_comp$/ )                         { $text_out = "pre-comp"; }
    if ( $item =~ m/^sum_bound_net$/ )                              { $text_out = "network"; }
    if ( $item =~ m/^sum_resp_t$/ )                                 { $text_out = "total"; }
    if ( $item =~ m/^sum_resp_t_b$/ )                               { $text_out = "total back"; }
    if ( $item =~ m/^sum_data_b$/ )                                 { $text_out = "total back"; }
    if ( $item =~ m/^sum_data_total$/ )                             { $text_out = "total"; }
    if ( $item =~ m/^sum_net$/ )                                     { $text_out = "total"; }
    if ( $item =~ m/^io_rate$/ )                                    { $text_out = "total"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/DS5K/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/HUS/ )        { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/NETAPP/ )     { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/3PAR/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/CEPH/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/RANK/ && $st_type =~ m/HUS/ )        { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/VSPG/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/COMPELLENT/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/OCEANSTOR/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/INFINIBOX/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/DATADOMAIN/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/PURE/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/^VNX$/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/VNXF/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/DOTHILL/ )    { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/UNITY/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/VNXE$/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/NIMBLE/ )     { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/ETERNUS/ )     { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/DATACORE/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/POOL/ && $st_type =~ m/STOREVIRTUAL/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^resp_t$/   && $type =~ m/POOL/ ) { $text_out = "per pool"; }
    if ( $item =~ m/^hit_ratio$/   && $type =~ m/NODE/ ) { $text_out = "hit ratio"; }
    if ( $item =~ m/^total_read$/   && $type =~ m/DRIVE/ ) { $text_out = "total read"; }
    if ( $item =~ m/^total_write$/   && $type =~ m/DRIVE/ ) { $text_out = "total write"; }
    if ( $item =~ m/^ssd_hit_ratio$/   && $type =~ m/NODE/ ) { $text_out = "ssd hit ratio"; }
    if ( $item =~ m/^queue_full_count$/ && $type =~ m/DRIVE/ ){$text_out = "disk queue full count"}
    if ( $st_type eq "DATADOMAIN" && $item =~ m/^resp_t$/   && $type =~ m/POOL/ ) { $text_out = "total"; }
    if ( $item =~ m/^r_transfer_size$/ && $type =~ m/VOLUME/ ) { $text_out = "read transfer size"; }
    if ( $item =~ m/^w_transfer_size$/ && $type =~ m/VOLUME/ ) { $text_out = "write transfer size"; }
    if ( $item =~ m/^r_transfer_size$/ && $type =~ m/IO-GROUP/ ) { $text_out = "read transfer size"; }
    if ( $item =~ m/^w_transfer_size$/ && $type =~ m/IO-GROUP/ ) { $text_out = "write transfer size"; }
    if ( $item =~ m/^r_transfer_size$/ && $type =~ m/CLUSTER/ ) { $text_out = "read transfer size"; }
    if ( $item =~ m/^w_transfer_size$/ && $type =~ m/CLUSTER/ ) { $text_out = "write transfer size"; }
    if ( $item =~ m/^resp_t_b$/ && $type =~ m/POOL/ ) { $text_out = "per pool back"; }
    if ( $item =~ m/^data_rate$/ ) { $text_out = "total"; }
    if ( $item =~ m/^data_rate_ini$|^io_rate_ini$|^resp_t_ini$/ ) { $text_out = "total initiator"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/DS5K/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/HUS/ )        { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/VSPG/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/COMPELLENT/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/INFINIBOX/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/OCEANSTOR/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/NETAPP/ )     { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/3PAR/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/RANK/ && $st_type =~ m/HUS/ )        { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/^VNX$/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/VNXF/ )       { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/DOTHILL/ )    { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/UNITY/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/VNXE$/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/PURE$/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/NIMBLE/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/DATADOMAIN$/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/ETERNUS/ )     { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/ISILON/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/CEPH/ )      { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/DATACORE/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/STOREONCE/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type =~ m/POOL/ && $st_type =~ m/STOREVIRTUAL/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate_b$/ )   { $text_out = "total back"; }
    if ( $item =~ m/^sum_capacity$/ )  { $text_out = "capacity"; }
    if ( $item =~ m/^sum_capacity$/ && $type =~ m/TIER/ && $st_type =~ m/DATADOMAIN$/ )  { $text_out = "tier"; }
    if ( $item =~ m/^sum_capacity_aggregated$/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^read_io$/ )       { $text_out = "read"; }
    if ( $item =~ m/^write_io$/ )      { $text_out = "write"; }
    if ( $item =~ m/^write_calls$/ )   { $text_out = "write"; }
    if ( $item =~ m/^b2b$/ )           { $text_out = "B2B"; }
    if ( $item =~ m/^crc$/ )           { $text_out = "CRC"; }
    if ( $item =~ m/^read_calls$/ )    { $text_out = "read"; }
    if ( $item =~ m/^calls_rate$/ )    { $text_out = "total"; }
    if ( $item =~ m/^net_rate$/ )      { $text_out = "total"; }
    if ( $item =~ m/^read_net$/ )      { $text_out = "read"; }
    if ( $item =~ m/^write_net$/ )     { $text_out = "write"; }
    if ( $item =~ m/^read$/ )          { $text_out = "read"; }
    if ( $item =~ m/^write$/ )         { $text_out = "write"; }
    if ( $item =~ m/^r_cache_hit$/ )   { $text_out = "read hit"; }
    if ( $item =~ m/^w_cache_hit$/ )   { $text_out = "write hit"; }
    if ( $item =~ m/^r_cache_miss$/ )   { $text_out = "read miss"; }
    if ( $item =~ m/^w_cache_miss$/ )   { $text_out = "write miss"; }
    if ( $item =~ m/^r_cache_usage$/ ) { $text_out = "read usage"; }
    if ( $item =~ m/^w_cache_usage$/ ) { $text_out = "write usage"; }
    if ( $item =~ m/^total_usage$/ )   { $text_out = "usage"; }
    if ( $item =~ m/^read_repl$/ )      { $text_out = "read"; }
    if ( $item =~ m/^write_repl$/ )     { $text_out = "write"; }
    if ( $item =~ m/^repl_rate$/ && $type =~ m/POOL/)     { $text_out = "aggregated"; }
    if ( $item =~ m/^repl_rate$/ && $type !~ m/POOL/)     { $text_out = "total"; }
    if ( $item =~ m/^resp_t$/ && $type !~ m/POOL/ ) { $text_out = "total"; }
    if ( $item =~ m/^resp_t$/ && ( $type =~ m/DRIVE/ || $type =~ m/RANK/ ) && $st_type =~ m/DOTHILL/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^resp_t$/ && $type =~ m/DRIVE/ && $st_type =~ m/COMPELLENT|OCEANSTOR|DATADOMAIN/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^resp_t$/ && $type =~ m/VOLUME-FOLDER/ && $st_type =~ m/COMPELLENT/ ) { $text_out = "total"; }
    if ( $item =~ m/^resp_t$/ && $type =~ m/S3/ && $st_type =~ m/PUREBLADE/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^ops_rate$/ )  { $text_out = "total"; }
    if ( $item =~ m/^read_ops$/ )  { $text_out = "read"; }
    if ( $item =~ m/^write_ops$/ ) { $text_out = "write"; }
    if ( $item =~ m/^resp_t$/  && $type =~ m/PORT/ && $st_type =~ m/DOTHILL/ ) { $text_out = "per port"; }
    if ( $item =~ m/^resp_t$/  && $type =~ m/PORT/ && $st_type =~ m/DS8K/ )    { $text_out = "per port"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/PORT/ && $st_type =~ m/DS8K/ )    { $text_out = "aggregated"; }
    if ( $item =~ m/^resp_t_b$/ && $type !~ m/POOL/ ) { $text_out = "total"; }
    if ( $item =~ m/^resp_t_r$/ )          { $text_out = "read"; }
    if ( $item =~ m/^resp_t_w$/ )          { $text_out = "write"; }
    if ( $item =~ m/^read_io_b$/ )         { $text_out = "read back"; }
    if ( $item =~ m/^write_io_b$/ )        { $text_out = "write back"; }
    if ( $item =~ m/^read_b$/ )            { $text_out = "read back"; }
    if ( $item =~ m/^write_b$/ )           { $text_out = "write back"; }
    if ( $item =~ m/^resp_t_r_b$/ )        { $text_out = "read back"; }
    if ( $item =~ m/^resp_t_w_b$/ )        { $text_out = "write back"; }
    if ( $item =~ m/^sys$/ )               { $text_out = "CPU"; }
    if ( $item =~ m/^compress$/ )          { $text_out = "compress"; }
    if ( $item =~ m/^busy$/ )              { $text_out = "CPU busy"; }
    if ( $item =~ m/^busy$/ && $type =~ m/DRIVE/)                   { $text_out = "busy"; }
    if ( $item =~ m/^pprc_rio$/ )          { $text_out = "IO read"; }
    if ( $item =~ m/^pprc_wio$/ )          { $text_out = "IO write"; }
    if ( $item =~ m/^pprc_data_r$/ )       { $text_out = "read"; }
    if ( $item =~ m/^pprc_data_w$/ )       { $text_out = "write"; }
    if ( $item =~ m/^pprc_rt_r$/ )         { $text_out = "resp read"; }
    if ( $item =~ m/^pprc_rt_w$/ )         { $text_out = "resp write"; }
    if ( $item =~ m/^tier0$/ )             { $text_out = "tier 0"; }
    if ( $item =~ m/^tier0-nl$/ )          { $text_out = "tier 0-NL"; }
    if ( $item =~ m/^tier1$/ )             { $text_out = "tier 1"; }
    if ( $item =~ m/^tier2$/ )             { $text_out = "tier 2"; }
    if ( $item =~ m/^compress_cap$/ )      { $text_out = "total"; }
    if ( $item =~ m/^uncompress_cap$/ )    { $text_out = "uncompressed"; }
    if ( $item =~ m/^compress_virt_cap$/ ) { $text_out = "virtual"; }
    if ( $item =~ m/^used$/ )              { $text_out = "used"; }
    if ( $item =~ m/^virt$/ )              { $text_out = "virtual"; }
    if ( $item =~ m/^real$/ )              { $text_out = "used"; }
    if ( $item =~ m/^tier_ENT$|^tier_SSD$|^tier_Flash$|^tier_HighCapFlash$|^tier_NL$|^tier_SATA$/){$text_out = $item; $text_out =~ s/tier_//g;}
    if ( $item =~ m/^cache_hit$/ )         { $text_out = "cache hit"; }
    if ( $item =~ m/^data_cntl$/ )         { $text_out = "controller"; }
    if ( $item =~ m/^io_cntl$/ )           { $text_out = "controller"; }
    if ( $item =~ m/^data_cntl$/ && $st_type =~ m/^VNX$/ ) { $text_out = "Storage Processor"; }
    if ( $item =~ m/^io_cntl$/   && $st_type =~ m/^VNX$/ ) { $text_out = "Storage Processor"; }
    if ( $item =~ m/^ssd_r_cache_hit$/ )   { $text_out = "SSD read cache"; }
    if ( $item =~ m/^write_pend$/ )        { $text_out = "write pending"; }
    if ( $item =~ m/^clean_usage$/ )       { $text_out = "clean queue"; }
    if ( $item =~ m/^dnlc_hit$/ )          { $text_out = "dnlc hit"; }
    if ( $item =~ m/^buffer_hit$/ )        { $text_out = "buffer hit"; }
    if ( $item =~ m/^middle_usage$/ )      { $text_out = "middle queue"; }
    if ( $item =~ m/^phys_usage$/ )        { $text_out = "physical queue"; }
    if ( $item =~ m/^operating_rate$/ )    { $text_out = "operating"; }
    if ( $item =~ m/^dirty_cache_pages$/ ) { $text_out = "Cache"; }
    if ( $item =~ m/^io_rate$/ && $type =~ m/NODE/ && $st_type =~ m/^VNX$/ ) { $text_out = "IO"; }
    if ( $item =~ m/^busy$/              && $st_type =~ m/^VMAX$/ )  { $text_out = "aggregated"; }
    if ( $item =~ m/^sum_busy$/          && $st_type =~ m/^VMAX$/ )  { $text_out = "total"; }
    if ( $item =~ m/^busy$/              && $st_type =~ m/^VPLEX$/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^busy$/              && $st_type =~ m/^DATADOMAIN$|^ISILON$/ && $type =~ m/DRIVE/ ){ $text_out = "aggregated"; }
    if ( $item =~ m/^sum_busy$/          && $st_type =~ m/^VPLEX$/ ) { $text_out = "total"; }
    if ( $item =~ m/^sum_busy$/          && $st_type =~ m/^ISILON$/ && $type =~ m/^CPU-CORE$/) { $text_out = "total CPU busy"; }
    if ( $item =~ m/^sum_busy$/          && $st_type =~ m/^DATADOMAIN$|^ISILON$/ && $type =~ m/DRIVE/ ) { $text_out = "total"; }
    if ( $item =~ m/^sum_busy_frontend$/ && $st_type =~ m/^VMAX$/ )  { $text_out = "FE"; }
    if ( $item =~ m/^sum_busy_backend$/  && $st_type =~ m/^VMAX$/ )  { $text_out = "BE"; }
    if ( $item =~ m/^sum_busy_rdf$/      && $st_type =~ m/^VMAX$/ )  { $text_out = "RDF"; }
    if ( $item =~ m/^alloc$/ )              { $text_out = "allocated"; }
    if ( $item =~ m/^io_rate_frontend$/ )   { $text_out = "FE"; }
    if ( $item =~ m/^io_rate_backend$/ )    { $text_out = "BE"; }
    if ( $item =~ m/^io_rate_rdf$/ )        { $text_out = "RDF"; }
    if ( $item =~ m/^read_io_frontend$/ )   { $text_out = "FE read"; }
    if ( $item =~ m/^read_io_backend$/ )    { $text_out = "BE read"; }
    if ( $item =~ m/^read_io_rdf$/ )        { $text_out = "RDF read"; }
    if ( $item =~ m/^write_io_frontend$/ )  { $text_out = "FE write"; }
    if ( $item =~ m/^write_io_backend$/ )   { $text_out = "BE write"; }
    if ( $item =~ m/^write_io_rdf$/ )       { $text_out = "RDF write"; }
    if ( $item =~ m/^data_rate_frontend$/ ) { $text_out = "FE"; }
    if ( $item =~ m/^data_rate_backend$/ )  { $text_out = "BE"; }
    if ( $item =~ m/^data_rate_rdf$/ )      { $text_out = "RDF"; }
    if ( $item =~ m/^read_frontend$/ )      { $text_out = "FE read"; }
    if ( $item =~ m/^read_backend$/ )       { $text_out = "BE read"; }
    if ( $item =~ m/^read_rdf$/ )           { $text_out = "RDF read"; }
    if ( $item =~ m/^write_frontend$/ )     { $text_out = "FE write"; }
    if ( $item =~ m/^write_backend$/ )      { $text_out = "BE write"; }
    if ( $item =~ m/^write_rdf$/ )          { $text_out = "RDF write"; }
    if ( $item =~ m/^busy_frontend$/ )      { $text_out = "FE aggregated"; }
    if ( $item =~ m/^busy_backend$/ )       { $text_out = "BE aggregated"; }
    if ( $item =~ m/^busy_rdf$/ )           { $text_out = "RDF aggregated"; }
    if ( $item =~ m/^resp_t_r_frontend$/ )  { $text_out = "FE read"; }
    if ( $item =~ m/^resp_t_w_frontend$/ )  { $text_out = "FE write"; }
    if ( $item =~ m/^data_rate$/  && $type =~ m/DRIVE/ && $st_type =~ m/3PAR/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/    && $type =~ m/DRIVE/ && $st_type =~ m/3PAR/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^resp_t$/     && $type =~ m/DRIVE/ && $st_type =~ m/3PAR/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/  && $type =~ m/TIER/  && $st_type =~ m/3PAR/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^io_rate$/    && $type =~ m/TIER/  && $st_type =~ m/3PAR/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^resp_t$/     && $type =~ m/TIER/  && $st_type =~ m/3PAR/ ) { $text_out = "aggregated"; }
    if ( $item eq "resp_t_r_vlun" && $type eq "VOLUME" && $st_type eq "3PAR" )  { $text_out = "resp read vLUN"; }
    if ( $item eq "resp_t_w_vlun" && $type eq "VOLUME" && $st_type eq "3PAR" )  { $text_out = "resp write vLUN"; }
    if ( $item eq "resp_t_vlun"   && $type eq "VOLUME" && $st_type eq "3PAR" )  { $text_out = "resp time vLUN"; }
    if ( $item =~ m/^io_rate$/    && $type =~ m/TIER/  && $st_type =~ m/STOREVIRTUAL/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^resp_t$/     && $type =~ m/TIER/  && $st_type =~ m/STOREVIRTUAL/ ) { $text_out = "aggregated"; }
    if ( $item =~ m/^io_pending$/ ) { $text_out = "pending"; }
    if ( $item =~ m/^io_rate$/ && $type eq "PORT" && $st_type eq "VNX" )         { $text_out = "aggregated"; }
    if ( $item =~ m/^data_rate$/ && $type eq "PORT" && $st_type eq "VNX" )       { $text_out = "aggregated"; }
    if ( $item =~ m/^cfc_dirty$/ )  { $text_out = "cfc dirty"; }
    if ( $item =~ m/^cfc_max$/ )    { $text_out = "cfc max"; }
    if ( $item =~ m/^del_ack$/ )    { $text_out = "del ack"; }
    if ( $item =~ m/^apply_lat$/ ){$text_out = "apply latency";}
    if ( $item =~ m/^commit_lat$/ ){$text_out = "commit latency";}
    if ( $item =~ m/^nfsv4_ops_rate$/ ){$text_out = "nfs4";}
    if ( $item =~ m/^nfs_ops_rate$/ ){$text_out = "nfs";}
    if ( $item =~ m/^smb1_rate$/ ){$text_out = "smb1";}
    if ( $item =~ m/^smb2_rate$/ ){$text_out = "smb2";}
    if ( $item =~ m/^total_comp_factor$/ ){$text_out = "compression";}
    if ( $item =~ m/^read_data_job$/ )  { $text_out = "read data job"; }
    if ( $item =~ m/^write_data_job$/ ) { $text_out = "write data job"; }
    if ( $item =~ m/^read_copy_job$/ )  { $text_out = "read copy job"; }
    if ( $item =~ m/^write_copy_job$/ ) { $text_out = "write copy job"; }
  } ## end if ( $type_out =~ m/^aggregated$/)
  else {
    if ( $item =~ m/^packet_rate$/ )                                { $text_out = "packet"; }
    if ( $item =~ m/^read_packet$/ )                                { $text_out = "packet read"; }
    if ( $item =~ m/^write_packet$/ )                               { $text_out = "packet write"; }
    if ( $item =~ m/^read_error$/ )                                 { $text_out = "error read"; }
    if ( $item =~ m/^write_error$/ )                                { $text_out = "error write"; }
    if ( $item =~ m/^io_rate_b$/ )                                  { $text_out = "total back"; }
    if ( $item =~ m/^cifs_io_rate-3_io_rate-4_io_rate/ )            { $text_out = "Prot IO"; }
    if ( $item =~ m/^cifs_read_io-3_read_io-4_read_io/ )            { $text_out = "Prot IO read"; }
    if ( $item =~ m/^cifs_write_io-3_write_io-4_write_io/ )         { $text_out = "Prot IO write"; }
    if ( $item =~ m/^cifs_data_rate-3_data_rate-4_data_rate/ )      { $text_out = "Prot data"; }
    if ( $item =~ m/^cifs_read-3_read-4_read/ )                     { $text_out = "Prot read"; }
    if ( $item =~ m/^cifs_write-3_write-4_write/ )                  { $text_out = "Prot write"; }
    if ( $item =~ m/^cifs_resp_t-3_resp_t-4_resp_t/ )               { $text_out = "Protocols resp"; }
    if ( $item =~ m/^wafl_exempt-kahuna-exempt-hostOS-nwk_legacy/ ) { $text_out = "CPU"; }
    if ( $item =~ m/^sum_io$/ )                                     { $text_out = "IO"; }
    if ( $item =~ m/^sum_io_b$/ )                                   { $text_out = "IO back"; }
    if ( $item =~ m/^sum_io_total$/ )                               { $text_out = "IO"; }
    if ( $item =~ m/^sum_data$/ )                                   { $text_out = "data"; }
    if ( $item =~ m/^outbound_pre_comp$/ )                          { $text_out = "outbound pre comp"; }
    if ( $item =~ m/^inbound_pre_comp$/ )                           { $text_out = "inbound pre comp"; }
    if ( $item =~ m/^outbound_net$/ )                               { $text_out = "outbound network"; }
    if ( $item =~ m/^inbound_net$/ )                                { $text_out = "inbound network"; }
    if ( $item =~ m/^sum_resp_t$/ )                                 { $text_out = "resp"; }
    if ( $item =~ m/^sum_resp_t_b$/ )                               { $text_out = "resp back"; }
    if ( $item =~ m/^sum_data_b$/ )                                 { $text_out = "data back"; }
    if ( $item =~ m/^sum_data_total$/ )                             { $text_out = "data"; }
    if ( $item =~ m/^sum_net$/ )                                     { $text_out = "Net"; }
    if ( $item =~ m/^io_rate$/ )                                    { $text_out = "IO"; }
    if ( $item =~ m/^data_rate$/ )                                  { $text_out = "data"; }
    if ( $item =~ m/^data_rate_ini$/ )                              { $text_out = "data initiator"; }
    if ( $item =~ m/^io_rate_ini$/ )                                { $text_out = "IO initiator"; }
    if ( $item =~ m/^io_rate_b$/ )                                  { $text_out = "total back"; }
    if ( $item =~ m/^data_rate_b$/ )                                { $text_out = "total back"; }
    if ( $item =~ m/^sum_capacity$/ )                               { $text_out = "capacity"; }
    if ( $item =~ m/^read_io$/ )                                    { $text_out = "IO read"; }
    if ( $item =~ m/^write_io$/ )                                   { $text_out = "IO write"; }
    if ( $item =~ m/^calls_rate$/ )                                 { $text_out = "calls"; }
    if ( $item =~ m/^read_calls$/ )                                 { $text_out = "calls read"; }
    if ( $item =~ m/^write_calls$/ )                                { $text_out = "calls write"; }
    if ( $item =~ m/^net_rate$/ )                                   { $text_out = "net"; }
    if ( $item =~ m/^read_net$/ )                                   { $text_out = "net read"; }
    if ( $item =~ m/^write_net$/ )                                  { $text_out = "net write"; }
    if ( $item =~ m/^b2b$/ )                                        { $text_out = "B2B"; }
    if ( $item =~ m/^crc$/ )                                        { $text_out = "CRC"; }
    if ( $item =~ m/^read$/ )                                       { $text_out = "read"; }
    if ( $item =~ m/^write$/ )                                      { $text_out = "write"; }
    if ( $item =~ m/^r_cache_hit$/ )                                { $text_out = "read cache hit"; }
    if ( $item =~ m/^w_cache_hit$/ )                                { $text_out = "write cache hit"; }
    if ( $item =~ m/^r_cache_miss$/ )                               { $text_out = "read cache miss"; }
    if ( $item =~ m/^w_cache_miss$/ )                               { $text_out = "write cache miss"; }
    if ( $item =~ m/^r_cache_usage$/ )                              { $text_out = "read usage"; }
    if ( $item =~ m/^w_cache_usage$/ )                              { $text_out = "write usage"; }
    if ( $item =~ m/^total_usage$/ )                                { $text_out = "usage"; }
    if ( $item =~ m/^resp_t$/ )                                     { $text_out = "resp time"; }
    if ( $item =~ m/^resp_t_ini$/ )                                 { $text_out = "resp time initiator"; }
    if ( $item =~ m/^resp_t_r$/ )                                   { $text_out = "resp read"; }
    if ( $item =~ m/^resp_t_w$/ )                                   { $text_out = "resp write"; }
    if ( $item =~ m/^read_io_b$/ )                                  { $text_out = "IO read back"; }
    if ( $item =~ m/^write_io_b$/ )                                 { $text_out = "IO write back"; }
    if ( $item =~ m/^read_b$/ )                                     { $text_out = "read back"; }
    if ( $item =~ m/^write_b$/ )                                    { $text_out = "write back"; }
    if ( $item =~ m/^resp_t_b$/ )                                   { $text_out = "resp time back"; }
    if ( $item =~ m/^resp_t_r_b$/ )                                 { $text_out = "resp read back"; }
    if ( $item =~ m/^resp_t_w_b$/ )                                 { $text_out = "resp write back"; }
    if ( $item =~ m/^r_transfer_size$/ && $type =~ m/VOLUME/ ) { $text_out = "read transfer size"; }
    if ( $item =~ m/^w_transfer_size$/ && $type =~ m/VOLUME/ ) { $text_out = "write transfer size"; }
    if ( $item =~ m/^r_transfer_size$/ && $type =~ m/IO-GROUP/ ) { $text_out = "read transfer size"; }
    if ( $item =~ m/^w_transfer_size$/ && $type =~ m/IO-GROUP/ ) { $text_out = "write transfer size"; }
    if ( $item =~ m/^r_transfer_size$/ && $type =~ m/CLUSTER/ ) { $text_out = "read transfer size"; }
    if ( $item =~ m/^w_transfer_size$/ && $type =~ m/CLUSTER/ ) { $text_out = "write transfer size"; }
    if ( $item =~ m/^sys$/ )                                        { $text_out = "CPU"; }
    if ( $item =~ m/^compress$/ )                                   { $text_out = "compress"; }
    if ( $item =~ m/^busy$/ )                                       { $text_out = "CPU busy"; }
    if ( $item =~ m/^busy$/ && $type =~ m/DRIVE/)                   { $text_out = "busy"; }
    if ( $item =~ m/^pprc_rio$/ )                                   { $text_out = "PPRC IO read"; }
    if ( $item =~ m/^pprc_wio$/ )                                   { $text_out = "PPRC IO write"; }
    if ( $item =~ m/^pprc_data_r$/ )                                { $text_out = "PPRC read"; }
    if ( $item =~ m/^pprc_data_w$/ )                                { $text_out = "PPRC write"; }
    if ( $item =~ m/^pprc_rt_r$/ )                                  { $text_out = "PPRC resp read"; }
    if ( $item =~ m/^pprc_rt_w$/ )                                  { $text_out = "PPRC resp write"; }
    if ( $item =~ m/^tier0$/ )                                      { $text_out = "tier 0"; }
    if ( $item =~ m/^tier0-nl$/ )                                   { $text_out = "tier 0-nl"; }
    if ( $item =~ m/^tier1$/ )                                      { $text_out = "tier 1"; }
    if ( $item =~ m/^tier2$/ )                                      { $text_out = "tier 2"; }
    if ( $item =~ m/^compress_cap$/ )                               { $text_out = "total"; }
    if ( $item =~ m/^uncompress_cap$/ )                             { $text_out = "uncompressed"; }
    if ( $item =~ m/^compress_virt_cap$/ )                          { $text_out = "virtual"; }
    if ( $item =~ m/^used$/ )                                       { $text_out = "used"; }
    if ( $item =~ m/^virt$/ )                                       { $text_out = "virtual"; }
    if ( $item =~ m/^real$/ )                                       { $text_out = "used"; }
    if ( $item =~ m/^read_pct$/ )                                   { $text_out = "read percent"; }
    if ( $item =~ m/^write_pct$/ )                                  { $text_out = "write percent"; }
    if ( $item =~ m/^cache_hit$/ )                                  { $text_out = "cache hit"; }
    if ( $item =~ m/^ssd_r_cache_hit$/ )                            { $text_out = "SSD read cache hit"; }
    if ( $item =~ m/^san_data_sum_in$/ )                            { $text_out = "IN"; }
    if ( $item =~ m/^san_data_sum_out$/ )                           { $text_out = "OUT"; }
    if ( $item =~ m/^san_io_sum_credits$/ )                         { $text_out = "BBCredits"; }
    if ( $item =~ m/^san_io_sum_crc_errors$/ )                      { $text_out = "CRC errors"; }
    if ( $item =~ m/^hit_ratio$/ ) { $text_out = "hit ratio"; }
    if ( $item =~ m/^total_read$/ ) { $text_out = "total read"; }
    if ( $item =~ m/^total_write$/  ) { $text_out = "total write"; }
    if ( $item =~ m/^ssd_hit_ratio$/ ) { $text_out = "ssd hit ratio"; }
    if ( $item =~ m/^queue_full_count$/ ){$text_out = "disk queue full count";}
    if ( $item =~ m/^apply_lat$/ ){$text_out = "apply latency";}
    if ( $item =~ m/^commit_lat$/ ){$text_out = "commit latency";}
    if ( $item =~ m/^nfsv4_ops_rate$/ ){$text_out = "nfs4";}
    if ( $item =~ m/^nfs_ops_rate$/ ){$text_out = "nfs";}
    if ( $item =~ m/^smb1_rate$/ ){$text_out = "smb1";}
    if ( $item =~ m/^smb2_rate$/ ){$text_out = "smb2";}
    if ( $item =~ m/^total_comp_factor$/ ){$text_out = "compression";}
    if ( $item =~ m/^repl_rate$/ ){$text_out = "replication rate";}
    if ( $item =~ m/^write_repl$/ ){$text_out = "replication write";}
    if ( $item =~ m/^read_repl$/ ){$text_out = "replication read";}
    if ( $item =~ m/^read_data_job$/ )  { $text_out = "read data job"; }
    if ( $item =~ m/^write_data_job$/ ) { $text_out = "write data job"; }
    if ( $item =~ m/^read_copy_job$/ )  { $text_out = "read copy job"; }
    if ( $item =~ m/^write_copy_job$/ ) { $text_out = "write copy job"; }
    if ( $item =~ m/^dedupe_ratio$/ ) { $text_out = "dedupe ratio"; }
    # SAN
    if ( $item =~ m/^san_io_sum_in$/ )            { $text_out = "IN"; }
    if ( $item =~ m/^san_io_sum_out$/ )           { $text_out = "OUT"; }
    if ( $item =~ m/^san_data_sum_tot_in/ )       { $text_out = "IN"; }
    if ( $item =~ m/^san_data_sum_tot_out/ )      { $text_out = "OUT"; }
    if ( $item =~ m/^san_io_sum_tot_credits/ )    { $text_out = "BBCredits"; }
    if ( $item =~ m/^san_io_sum_tot_crc_errors/ ) { $text_out = "CRC errors"; }
    if ( $item =~ m/^san_io_sum_tot_in/ )         { $text_out = "IN"; }
    if ( $item =~ m/^san_io_sum_tot_out/ )        { $text_out = "OUT"; }
    if ( $item =~ m/^san_data$/ )                 { $text_out = "Data"; }
    if ( $item =~ m/^san_io$/ )                   { $text_out = "Frames"; }
    if ( $item =~ m/^san_io_size$/ )              { $text_out = "Frame size"; }
    if ( $item =~ m/^san_credits$/ )              { $text_out = "BBCredits"; }
    if ( $item =~ m/^san_errors$/ )               { $text_out = "Errors"; }
    if ( $item =~ m/^san_fabric_data_in/ )        { $text_out = "Data in"; }
    if ( $item =~ m/^san_fabric_frames_in/ )      { $text_out = "Frames in"; }
    if ( $item =~ m/^san_fabric_credits/ )        { $text_out = "BBCredits"; }
    if ( $item =~ m/^san_fabric_crc_errors/ )     { $text_out = "CRC errors"; }
    if ( $item =~ m/^san_fabric_conf/ )           { $text_out = "Configuration"; }
    if ( $item =~ m/^san_isl_data_in/ )           { $text_out = "Data in"; }
    if ( $item =~ m/^san_isl_data_out/ )          { $text_out = "Data out"; }
    if ( $item =~ m/^san_isl_frames_in/ )         { $text_out = "Frames in"; }
    if ( $item =~ m/^san_isl_frames_out/ )        { $text_out = "Frames out"; }
    if ( $item =~ m/^san_isl_frame_size_in/ )     { $text_out = "Frame size in"; }
    if ( $item =~ m/^san_isl_frame_size_out/ )    { $text_out = "Frame size out"; }
    if ( $item =~ m/^san_isl_credits/ )           { $text_out = "BBCredits"; }
    if ( $item =~ m/^san_isl_crc_errors/ )        { $text_out = "CRC errors"; }
    if ( $item =~ m/^san_cpu/ )                   { $text_out = "CPU"; }
    if ( $item =~ m/^san_mem/ )                   { $text_out = "Memory"; }
    if ( $item =~ m/^dnlc_hit$/ )                 { $text_out = "dnlc hit"; }
    if ( $item =~ m/^buffer_hit$/ )               { $text_out = "buffer hit"; }
    if ( $item =~ m/^san.*_enc_out/ )             { $text_out = "Enc out"; }
    if ( $item =~ m/^san.*_disc_c3/ )             { $text_out = "Disc C3 frames"; }
    if ( $item =~ m/^san.*_link_fail/ )           { $text_out = "Link fail"; }
    if ( $item =~ m/^san.*_loss_sync/ )           { $text_out = "Loss sync"; }

    # LAN
    if ( $item =~ m/^lan_data$/ )                       { $text_out = "Data"; }
    if ( $item =~ m/^lan_Ucast_pkts$/ )                 { $text_out = "Unicast Packets"; }
    if ( $item =~ m/^lan_Mcast_pkts$/ )                 { $text_out = "Multicast Packets"; }
    if ( $item =~ m/^lan_Bcast_pkts$/ )                 { $text_out = "Broadcast Packets"; }
    if ( $item =~ m/^lan_errors$/ )                     { $text_out = "Errors"; }
    if ( $item =~ m/^lan_sum_data_in$/ )                { $text_out = "IN"; }
    if ( $item =~ m/^lan_sum_data_out$/ )               { $text_out = "OUT"; }
    if ( $item =~ m/^lan_sum_pkts_Ucast_in$/ )          { $text_out = "Unicast pkts IN"; }
    if ( $item =~ m/^lan_sum_pkts_Ucast_out$/ )         { $text_out = "Unicast pkts OUT"; }
    if ( $item =~ m/^lan_sum_pkts_Mcast_in$/ )          { $text_out = "Multicast pkts IN"; }
    if ( $item =~ m/^lan_sum_pkts_Mcast_out$/ )         { $text_out = "Multicast pkts OUT"; }
    if ( $item =~ m/^lan_sum_pkts_Bcast_in$/ )          { $text_out = "Broadcast pkts IN"; }
    if ( $item =~ m/^lan_sum_pkts_Bcast_out$/ )         { $text_out = "Broadcast pkts OUT"; }
    if ( $item =~ m/^lan_sum_pkts_errors_in$/ )         { $text_out = "Errors IN"; }
    if ( $item =~ m/^lan_sum_pkts_errors_out$/ )        { $text_out = "Errors OUT"; }
    if ( $item =~ m/^lan_sum_data_tot_in$/ )            { $text_out = "IN"; }
    if ( $item =~ m/^lan_sum_data_tot_out$/ )           { $text_out = "OUT"; }
    if ( $item =~ m/^lan_sum_pkts_tot_Ucast_in$/ )      { $text_out = "Unicast pkts IN"; }
    if ( $item =~ m/^lan_sum_pkts_tot_Ucast_out$/ )     { $text_out = "Unicast pkts OUT"; }
    if ( $item =~ m/^lan_sum_pkts_tot_Mcast_in$/ )      { $text_out = "Multicast pkts IN"; }
    if ( $item =~ m/^lan_sum_pkts_tot_Mcast_out$/ )     { $text_out = "Multicast pkts OUT"; }
    if ( $item =~ m/^lan_sum_pkts_tot_Bcast_in$/ )      { $text_out = "Broadcast pkts IN"; }
    if ( $item =~ m/^lan_sum_pkts_tot_Bcast_out$/ )     { $text_out = "Broadcast pkts OUT"; }
    if ( $item =~ m/^lan_sum_pkts_tot_errors_in$/ )     { $text_out = "Errors IN"; }
    if ( $item =~ m/^lan_sum_pkts_tot_errors_out$/ )    { $text_out = "Errors OUT"; }
    if ( $item =~ m/^lan_sum_vlan_unrouted_data_in$/ )  { $text_out = "Unrouted IN"; }
    if ( $item =~ m/^lan_sum_vlan_unrouted_data_out$/ ) { $text_out = "Unrouted OUT"; }
    if ( $item =~ m/^lan_sum_vlan_routed_data_in$/ )    { $text_out = "Routed IN"; }
    if ( $item =~ m/^lan_sum_vlan_routed_data_out$/ )   { $text_out = "Routed OUT"; }
    if ( $item =~ m/^lan_sum_vlan_data_in$/ )           { $text_out = "IN"; }
    if ( $item =~ m/^lan_sum_vlan_data_out$/ )          { $text_out = "OUT"; }
    if ( $item =~ m/^lan_sum_port_channel_data_in$/ )   { $text_out = "IN"; }
    if ( $item =~ m/^lan_sum_port_channel_data_out$/ )  { $text_out = "OUT"; }
    if ( $item =~ m/^lan_sum_fexport_data_in$/ )        { $text_out = "IN"; }
    if ( $item =~ m/^lan_sum_fexport_data_out$/ )       { $text_out = "OUT"; }
    if ( $item =~ m/^lan_cpu$/ )                        { $text_out = "CPU"; }
    if ( $item =~ m/^lan_mem$/ )                        { $text_out = "Memory"; }

    # custom groups
    if ( $item =~ m/^data_in$/ )    { $text_out = "Data in"; }
    if ( $item =~ m/^data_out$/ )   { $text_out = "Data out"; }
    if ( $item =~ m/^frames_in$/ )  { $text_out = "Frames in"; }
    if ( $item =~ m/^frames_out$/ ) { $text_out = "Frames out"; }
    if ( $item =~ m/^credits$/ )    { $text_out = "BBCredits"; }
    if ( $item =~ m/^errors$/ )     { $text_out = "CRC errors"; }
    if ( $item =~ m/^crc_errors$/ ) { $text_out = "CRC errors"; }
    if ( $item =~ m/^enc_out/ )     { $text_out = "Enc out"; }
    if ( $item =~ m/^disc_c3/ )     { $text_out = "Disc C3 frames"; }
    if ( $item =~ m/^link_fail/ )   { $text_out = "Link fail"; }
    if ( $item =~ m/^loss_sync/ )   { $text_out = "Loss sync"; }

    if ( $item =~ m/^write_pend$/ )        { $text_out = "write pending"; }
    if ( $item =~ m/^usage_rate$/ )        { $text_out = "usage rate"; }
    if ( $item =~ m/^clean_usage$/ )       { $text_out = "clean queue"; }
    if ( $item =~ m/^middle_usage$/ )      { $text_out = "middle queue"; }
    if ( $item =~ m/^phys_usage$/ )        { $text_out = "physical queue"; }
    if ( $item =~ m/^operating_rate$/ )    { $text_out = "operating"; }
    if ( $item =~ m/^qdepth$/ )            { $text_out = "queue depth"; }
    if ( $item =~ m/^dirty_cache_pages$/ ) { $text_out = "Cache"; }
    if ( $item =~ m/^busy$/ && $st_type =~ m/^VMAX$/ ) { $text_out = "Busy"; }
    if ( $item =~ m/^alloc$/ )              { $text_out = "allocated"; }
    if ( $item =~ m/^tier_alloc_prct$/ )    { $text_out = "allocated"; }
    if ( $item =~ m/^io_rate_frontend$/ )   { $text_out = "IO"; }
    if ( $item =~ m/^io_rate_backend$/ )    { $text_out = "IO"; }
    if ( $item =~ m/^io_rate_rdf$/ )        { $text_out = "IO"; }
    if ( $item =~ m/^read_io_frontend$/ )   { $text_out = "IO read"; }
    if ( $item =~ m/^read_io_backend$/ )    { $text_out = "IO read"; }
    if ( $item =~ m/^read_io_rdf$/ )        { $text_out = "IO read"; }
    if ( $item =~ m/^write_io_frontend$/ )  { $text_out = "IO write"; }
    if ( $item =~ m/^write_io_backend$/ )   { $text_out = "IO write"; }
    if ( $item =~ m/^write_io_rdf$/ )       { $text_out = "IO write"; }
    if ( $item =~ m/^data_rate_frontend$/ ) { $text_out = "Data"; }
    if ( $item =~ m/^data_rate_backend$/ )  { $text_out = "Data"; }
    if ( $item =~ m/^data_rate_rdf$/ )      { $text_out = "Data"; }
    if ( $item =~ m/^read_frontend$/ )      { $text_out = "read"; }
    if ( $item =~ m/^read_backend$/ )       { $text_out = "read"; }
    if ( $item =~ m/^read_rdf$/ )           { $text_out = "read"; }
    if ( $item =~ m/^write_frontend$/ )     { $text_out = "write"; }
    if ( $item =~ m/^write_backend$/ )      { $text_out = "write"; }
    if ( $item =~ m/^write_rdf$/ )          { $text_out = "write"; }
    if ( $item =~ m/^busy_frontend$/ )      { $text_out = "Busy"; }
    if ( $item =~ m/^busy_backend$/ )       { $text_out = "Busy"; }
    if ( $item =~ m/^busy_rdf$/ )           { $text_out = "Busy"; }
    if ( $item =~ m/^resp_t_r_frontend$/ )  { $text_out = "read"; }
    if ( $item =~ m/^resp_t_w_frontend$/ )  { $text_out = "write"; }
    if ( $item eq "resp_t_r_vlun" && $type eq "VOLUME" && $st_type eq "3PAR" ) { $text_out = "resp read vLUN"; }
    if ( $item eq "resp_t_w_vlun" && $type eq "VOLUME" && $st_type eq "3PAR" ) { $text_out = "resp write vLUN"; }
    if ( $item eq "resp_t_vlun"   && $type eq "VOLUME" && $st_type eq "3PAR" ) { $text_out = "resp time vLUN"; }
    if ( $item =~ m/^io_pending$/ ) { $text_out = "IO pending"; }
    if ( $item =~ m/^ops_rate$/ )   { $text_out = "OPS"; }
    if ( $item =~ m/^read_ops$/ )   { $text_out = "OPS read"; }
    if ( $item =~ m/^write_ops$/ )  { $text_out = "OPS write"; }
  } ## end else [ if ( $type_out =~ m/^aggregated$/)]
  return $text_out;
} ## end sub text_tab

sub non_existing_data {
  my $host = shift;
  my $name = shift;

  print "There is no any volume attached to that host: $name\n";

  return 0;
} ## end sub non_existing_data

sub check_ethernet_port{
  my $file = shift;
  my $id_port = shift;
  if ( !-f "$file" ) {
    return 0;
  }
  open( FHH, "< $file" ) || error( "Can't open $file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @ports = <FHH>;
  close(FHR);

  foreach my $line (@ports) {
    chomp($line);
    ( my $id, my $name ) = split( /:/, $line );
    if ($id eq $id_port && $name =~ m/ETH/ && $name =~ m/^eth/){
      return 1;
    }
  }
  return 0;
}

sub find_vols {
  my $host = shift;
  my $type = shift;
  my $lpar = shift;

  if ( !-f "$wrkdir/$host/$type/hosts.cfg" ) {
    return 0;
  }
  open( FHH, "< $wrkdir/$host/$type/hosts.cfg" ) || error( "Can't open $wrkdir/$host/$type/hosts.cfg : $! " . __FILE__ . ":" . __LINE__ ) && return "";
  my @hosts = <FHH>;
  close(FHR);

  foreach my $line (@hosts) {
    chomp($line);
    ( my $host_name, my $volumes ) = split( / : /, $line );
    if ( !defined($volumes) ) {
      next;
    }
    $volumes =~ s/ //g;

    # must be used this no regex construction otherwise m// does not work with names with ()
    if ( $host_name eq $lpar && length($volumes) > 0 ) {
      return 1;
    }
  } ## end foreach my $line (@hosts)
  return 0;
} ## end sub find_vols

sub volumes_top_all {
  my $host      = shift;
  my $type      = shift;
  my $item      = shift;
  my $st_type   = shift;
  my $name      = shift;
  my $time_unix = time();
  my $data_type = "tabfrontend";
  my $class     = "class=\"$data_type\"";
  my $class_url = "class=\"lazy\" src=\"$html_base/css/images/loading.gif\"";
  $class_url = "";

  # print tab page only
  print "<div  id=\"tabs\"> <ul>\n";
  print "  <li><a href='#tabs-0'>custom</a></li>\n";
  print "  <li $class><a $class_url href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=avrg&time=d&detail=0&none=$time_unix\">daily</a></li>\n";
  print "  <li $class><a $class_url href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=avrg&time=w&detail=0&none=$time_unix\">weekly</a></li>\n";
  print "  <li $class><a $class_url href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=avrg&time=m&detail=0&none=$time_unix\">monthly</a></li>\n";

  #print "  <li $class><a $class_url href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=avrg&time=y&detail=0&none=$time_unix\">yearly</a></li>\n";
  print "  <li $class><a $class_url href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=max&time=d&detail=0&none=$time_unix\">daily max</a></li>\n";
  print "  <li $class><a $class_url href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=max&time=w&detail=0&none=$time_unix\">weekly max</a></li>\n";
  print "  <li $class><a $class_url href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=max&time=m&detail=0&none=$time_unix\">monthly max</a></li>\n";

  #print "  <li $class><a $class_url href=\"$ENV{WEB_CGI_PATH}/detail-graph.sh?host=$host&type=$type&name=$name&item=max&time=y&detail=0&none=$time_unix\">yearly max</a></li>\n";
  print "</ul> \n";
  print <<_MARKER_;
  <div id='tabs-0'>
    <center>
      <div style="font-size: 0.8rem; margin-top: 1em">
      <form method="get" action="$ENV{WEB_CGI_PATH}/detail-graph.sh" id="volcustomform">
        <label for="from">From</label>
        <input type="text" id="fromTime" size="14">
        <label for="to">to</label>
        <input type="text" id="toTime" size="14">
        <select>
          <option value="AVG">AVG</option>
          <option value="MAX">MAX</option>
        </select>
        <input type='submit' style='font-weight: bold;' id='showvolumes' value='Show top volumes'>
      </form>
      </div>
      <div id="volresults"></div>
    </center>
  </div>
  </div>
_MARKER_

  return 1;
} ## end sub volumes_top_all

sub error {
  my $text     = shift;
  my $act_time = localtime();

  if ( $text =~ m/no new input files, exiting data load/ ) {
    print "ERROR          : $text \n";
    print STDERR "$act_time: $text \n";
  }
  else {
    print "ERROR          : $text : $!\n";
    print STDERR "$act_time: $text : $!\n";
  }

  return 1;
} ## end sub error

sub create_tab_custom {

  # $ENV{WEB_CGI_PATH}/detail.sh?host=na&type=VOLUME&name=Agrp1&storage=na&item=custom&gui=1&none=none
  #( my $host,my $type, my $name, my $st_type, my $item, my $gui, my $referer) = split(/&/,$QUERY_STRING);

  my ( $host, $type, $name, $st_type, $item ) = @_;

  # tab summary at first
  my $data_type  = "tabfrontend";
  my $tab_number = 0;
  print "<div  id=\"tabs\"> <ul>\n";
  foreach my $item_act (<@items_sorted>) {
    if ( !-f "$tmp_dir/custom-group/$name-$item_act-d.cmd" ) {
      next;
    }

    # print out tabs
    my $tab_name = text_tab( $item_act, "item" );
    print "  <li class=\"$data_type\"><a href=\"#tabs-$tab_number\">$tab_name</a></li>\n";
    $tab_number++;
  } ## end foreach my $item_act (<@items_sorted>)
  print "   </ul> \n";

  my $notice_file  = "$tmp_dir/custom-group/.$name-ll.cmd";
  my @final_notice = "";
  if ( -f $notice_file ) {
    open( FH, "$notice_file" ) || error( "Cannot open $notice_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    @final_notice = <FH>;
    close(FH);
  }

  # tab body
  $tab_number = 0;
  foreach my $item_act (<@items_sorted>) {
    if ( !-f "$tmp_dir/custom-group/$name-$item_act-d.cmd" ) {
      next;
    }
    print "<div id=\"tabs-$tab_number\"><center>\n";
    if ( $type ne "SANPORT" ) {
      print "<div id=\"hiw\"><a href=\"http://www.stor2rrd.com/custom_groups-metrics.htm\" target=\"_blank\" class=\"nowrap\"><img src=\"css/images/help-browser.gif\" alt=\"Help for Custom Groups\" title=\"Help for Custom Groups\"></img></a></div>";
    }
    print "<table align=\"center\" summary=\"Graphs\">\n";

    print "<tr>\n";
    print_item( $host, $type, $name, "custom-group-" . $item_act, "d", $detail_yes );
    print_item( $host, $type, $name, "custom-group-" . $item_act, "w", $detail_yes );
    print "</tr><tr>\n";
    print_item( $host, $type, $name, "custom-group-" . $item_act, "m", $detail_yes );
    print_item( $host, $type, $name, "custom-group-" . $item_act, "y", $detail_yes );
    print "</tr>\n";
    print "<tr><td align=\"center\" colspan=\"2\">@final_notice </td></tr>\n";

    if ( $item_act !~ m/^resp_t_/ && $item_act !~ m/^credits/ ) {
      print "<tr>\n";
      print_item( $host, $type, $name, "custom-group-" . $item_act . "-trend", "y", $detail_no );
      print "</tr>\n";
    } ## end if ( $item_act !~ m/^resp_t_/...)
    print "</table></center>\n";
    print "</div>\n";
    $tab_number++;
  } ## end foreach my $item_act (<@items_sorted>)

  print "</div><br>\n";

  return 1;
} ## end sub create_tab_custom

sub check_space_name {
  my $name       = shift;
  my $name_space = $name;
  if ( $name =~ m/ / ) {
    $name_space = "\"" . $name . "\"";    # it must be here to support space with server names
    return $name_space;
  }
  else {
    return $name;
  }
} ## end sub check_space_name

sub host_data_selection {
  my $wrkdir  = shift;
  my $host    = shift;
  my $type    = shift;
  my $st_type = shift;

  my $wrkdir_space = check_space_name($wrkdir);
  my $host_space   = check_space_name($host);
  my @files        = <$wrkdir_space/$host_space/$type/*>;
  my $rrd          = grep {/\.rrd$/} @files;

  my $VSPG_NATIVE_HOSTS = 1;
  if ( $st_type =~ m/VSPG/ && defined $ENV{VSPG_NATIVE_HOSTS} ) {

    # VSPG_NATIVE_HOSTS=0 forces to use original volume based host graphs (etc/.magic)
    $VSPG_NATIVE_HOSTS = $ENV{VSPG_NATIVE_HOSTS};
    if ( defined $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST} && $VSPG_NATIVE_HOSTS == 0 ) {
      $VSPG_NATIVE_HOSTS = 1;    # host are specified, switch default to host data back
      foreach my $storage_act ( split( / /, $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST} ) ) {
        if ( $host =~ m/^$storage_act$/ ) {
          $VSPG_NATIVE_HOSTS = 0;
        }
      }
    } ## end if ( defined $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST...})
  } ## end if ( $st_type =~ m/VSPG/...)

  if ( $rrd > 0 && $VSPG_NATIVE_HOSTS == 1 ) {
    return 1;                    # use host original
  }
  else {
    return 0;                    # use volume data instead
  }
} ## end sub host_data_selection
