#!/usr/bin/perl

use strict;
use warnings;
use JSON;
use Data::Dumper;

my $DEBUG   = $ENV{DEBUG};
my $errlog  = $ENV{ERRLOG};
my $webdir  = $ENV{WEBDIR};
my $basedir = $ENV{INPUTDIR};
my $tmp_dir = "$basedir/tmp";
my $wrkdir  = "$basedir/data";

open( OUT, ">> $errlog" ) if $DEBUG == 2;

# get QUERY_STRING
use Env qw(QUERY_STRING);
print OUT "-- $QUERY_STRING\n" if $DEBUG == 2;

#print STDERR "$QUERY_STRING\n";

( my $host, my $type, my $name, my $st_type, my $item, my $gui, my $referer ) = split( /&/, $QUERY_STRING );

# no URL decode here, as it goes immediately into URL again

$host =~ s/host=//;
my $host_url = $host;
$host =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
$host =~ s/\+/ /g;
$host =~ s/%23/\#/g;
$type =~ s/type=//;
$name =~ s/name=//;
$name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
$name =~ s/\+/ /g;
$name =~ s/%23/\#/g;
$item =~ s/item=//;
$st_type =~ s/storage=//;

#print STDERR "$host $name $type $st_type $item\n";
#print STDERR "$QUERY_STRING : $name\n";

my $gray_st   = "<img src=\"css/images/status_gray.png\">";
my $red_st    = "<img src=\"css/images/status_red.png\">";
my $orange_st = "<img src=\"css/images/status_orange.png\">";
my $green_st  = "<img src=\"css/images/status_green.png\">";

# CGI-BIN HTML header
print "Content-type: text/html\n\n";
print "<div style=\"text-align: center\">\n";
print "<div style=\"display: inline-block\">\n";

# health status for one LAN switch
if ( $type eq "HS" && defined $host && $host ne "" ) {
  generate_single_switch_page();
}

# health status for all LAN switches
if ( $type eq "HS-total" && $host eq "total" ) {
  generate_total_page();
}

print "</div>\n";
print "</div>\n";
exit;

sub generate_single_switch_page {
  my $switch_type = get_switch_type($host);
  print "<div id=\"tabs\"><ul>\n";
  print "<li><a href=\"#tabs-0\">Ports</a></li>\n";

  if ( -f "$wrkdir/$name/HEALTH_STATUS_SWITCH.txt" ) {
    print "<li><a href=\"#tabs-1\">Modules</a></li>\n";
  }

  print "</ul>\n";
  print "<div id=\"tabs-0\" class=\"blacktext\">\n";

  # parse and print HS table for one switch
  print "<br><br>\n";
  if ( $switch_type =~ /^(CISCO|UNKNOWN)$/ ) {
    print_hs_table_cisco($host);
    print "<br>\n";
    print_legend_cisco();
  }

  print "</div>\n";
  if ( -f "$wrkdir/$name/HEALTH_STATUS_SWITCH.txt" ) {
    print "<div id=\"tabs-1\" class=\"blacktext\">\n";
    print_modules_table();
    print "</div>\n";
  }

  return 1;
} ## end sub generate_single_switch_page

sub generate_total_page {
  print "<div id=\"tabs\"><ul>\n";
  print "<li><a href=\"#tabs-0\">Ports</a></li>\n";

  # add events if exists
  if ( -f "$webdir/LAN_total_events.html" ) {
    print "<li><a href=\"LAN_total_events.html\">Events</a></li>\n";
  }
  # add HS table for all switches if exists
  #  if ( -f "$webdir/LAN_HEALTH_STATUS.html" ) {
  my @component_files = <$wrkdir/*/HEALTH_STATUS_SWITCH.txt>;
  if ( scalar @component_files ) {
    print "<li><a href=\"#tabs-1\">Modules</a></li>\n";
  }

  print "</ul>\n";
  print "<div id=\"tabs-0\" class=\"blacktext\">\n";

  my @switches_cisco = <$wrkdir/*\/LAN-CISCO>;
  my $cisco_count    = @switches_cisco;

  # CISCO
  my $cisco_idx = 0;
  if ( defined $cisco_count && $cisco_count ne "0" ) {
    print "<br><br><b>Cisco LAN switches</b><br><br>\n";

    foreach my $switch_name (@switches_cisco) {
      chomp $switch_name;
      $switch_name =~ s/^$wrkdir\///g;
      $switch_name =~ s/\/LAN-CISCO$//g;

      $cisco_idx++;

      print "<p class='switchname'>$switch_name</p>\n";
      print_hs_table_cisco($switch_name);
      print "<br>\n";
    }
  }

  my @switches_unknown = <$wrkdir/*\/LAN-UNKNOWN>;
  my $unknown_count    = @switches_unknown;

  # UNKNOWN
  my $unknown_idx = 0;
  if ( defined $unknown_count && $unknown_count ne "0" ) {
    print "<br><br><b>Other LAN switches</b><br><br>\n";

    foreach my $switch_name (@switches_unknown) {
      chomp $switch_name;
      $switch_name =~ s/^$wrkdir\///g;
      $switch_name =~ s/\/LAN-UNKNOWN$//g;

      $unknown_idx++;

      print "<p class='switchname'>$switch_name</p>\n";
      print_hs_table_cisco($switch_name);
      print "<br>\n";
    }
  }

  print_legend_cisco();
  print "</div>\n";

  if ( scalar @component_files ) {
    print "<div id=\"tabs-1\" class=\"blacktext\">\n";
    print_total_switches_hs_table();
    print "</div>\n";
  }

  return 1;
} ## end generate_total_page

sub print_hs_table_cisco {
  my $switch_name = shift;

  if ( -f "$wrkdir/$switch_name/HEALTH_STATUS.txt" ) {
    open( HS, "<$wrkdir/$switch_name/HEALTH_STATUS.txt" ) || error( "Couldn't open file $wrkdir/$switch_name/HEALTH_STATUS.txt $!" . __FILE__ . ":" . __LINE__ ) && exit;
    my @lines = <HS>;
    (undef, undef, @lines) = @lines;
    close(HS);

    my %data;
    my $port_idx = 0;

    # port,Admin status, Operational status
    # ---------------------------------------
    # fc1-1,up,up,none

    foreach my $line (@lines) {
      chomp $line;
      $port_idx++;

      my ( $port_id, $admin_st, $oper_st ) = split( ",", $line );

      $data{'table'}{'id'}{$port_idx}{'port'}           = $port_id;
      $data{'table'}{'id'}{$port_idx}{'oper_st'}        = $oper_st;
      $data{'table'}{'id'}{$port_idx}{'admin_st'}       = $admin_st;

      # class for physical status (admin status)
      my $admin_st_class = "UN";
      if ( defined $admin_st && $admin_st ne "" ) {
        if ( $admin_st =~ "testing" || $admin_st =~ "down" ) {
          $admin_st_class = "st_orange";
        }
        if ( $admin_st =~ "up" ) {
          $admin_st_class = "st_green";
        }
      } ## end if ( defined $admin_st ...)

      # class for operational status
      my $oper_st_class = "UN";
      if ( defined $oper_st && $oper_st ne "" ) {
        if ( $oper_st =~ "unknown" || $oper_st =~ "notPresent" ) {
          $oper_st_class = "st_gray";
        }
        if ( $oper_st =~ "dormant" || $oper_st =~ "lowerLayerDown" ) {
          $oper_st_class = "st_red";
        }
        if ( $oper_st =~ "testing" || $oper_st =~ "down" ) {
          $oper_st_class = "st_orange";
        }
        if ( $oper_st =~ "up" ) {
          $oper_st_class = "st_green";
        }
      } ## end if ( defined $oper_st ...)

      $data{'table'}{'id'}{$port_idx}{'oper_st_class'}        = $oper_st_class;
      $data{'table'}{'id'}{$port_idx}{'admin_st_class'}       = $admin_st_class;

    } ## end foreach my $line (@lines)

    # print table
    my $print_table        = 0;
    my $port_table_line    = "";
    my $admin_table_line   = "";
    my $oper_table_line    = "";
    my $oper_r_table_line  = "";

    foreach my $id ( sort { $a <=> $b } keys( %{ $data{table}{id} } )) {
      my $port_name             = "";
      my $oper_st               = "";
      my $admin_st              = "";
      my $oper_st_class         = "";
      my $admin_st_class        = "";
      if ( defined $data{table}{id}{$id}{port} && $data{table}{id}{$id}{port} ne '' ) { $port_name = $data{table}{id}{$id}{port}; }
      if ( defined $data{table}{id}{$id}{oper_st} && $data{table}{id}{$id}{oper_st} ne '' ) { $oper_st = $data{table}{id}{$id}{oper_st}; }
      if ( defined $data{table}{id}{$id}{admin_st} && $data{table}{id}{$id}{admin_st} ne '' ) { $admin_st = $data{table}{id}{$id}{admin_st}; }
      if ( defined $data{table}{id}{$id}{oper_st_class} && $data{table}{id}{$id}{oper_st_class} ne '' ) { $oper_st_class = $data{table}{id}{$id}{oper_st_class}; }
      if ( defined $data{table}{id}{$id}{admin_st_class} && $data{table}{id}{$id}{admin_st_class} ne '' ) { $admin_st_class = $data{table}{id}{$id}{admin_st_class}; }

      my $port_num = $port_name;
      #$port_num    =~ s/[a-z]//g;
      #$port_num    =~ s/[A-Z]//g;

      $port_table_line  .= "<th class='rotate' title=\"$port_name\"><div><span>$port_num</div></span></th>";
      $admin_table_line .= "<td class=\"$admin_st_class\" title=\"$port_name : $admin_st\"><a href=\"$ENV{WEB_CGI_PATH}/detail.sh?host=$switch_name&type=LANPORT&name=$port_name&storage=LAN-CISCO&item=lan&gui=1&none=none\"></td>";
      $oper_table_line  .= "<td class=\"$oper_st_class\" title=\"$port_name : $oper_st\"><a href=\"$ENV{WEB_CGI_PATH}/detail.sh?host=$switch_name&type=LANPORT&name=$port_name&storage=LAN-CISCO&item=lan&gui=1&none=none\"></td>";

      if ( $id % 48 == 0 ) {
        print "<table class=\"san_stat_legend\">\n";
        print "<tr><td><b>Port</b></td>$port_table_line</tr>\n";
        print "<tr><td><b>Admin status</b></td>$admin_table_line</tr>";
        print "<tr><td><b>Operational status<b></td>$oper_table_line</tr>";
        print "</table>";

        $port_table_line   = "";
        $admin_table_line   = "";
        $oper_table_line   = "";
        $print_table       = 0;
      }
      else {
        $print_table = 1;
      }
    }
    if ( defined $print_table && $print_table == 1 ) {
      print "<table class=\"san_stat_legend\">\n";
      print "<tr><td><b>Port</b></td>$port_table_line</tr>\n";
      print "<tr><td><b>Admin status</b></td>$admin_table_line</tr>";
      print "<tr><td><b>Operational status<b></td>$oper_table_line</tr>";
      print "</table>";
    }

  } ## end if ( -f "$wrkdir/$switch_name/HEALTH_STATUS.txt")
  else {
    print "Health status data not found! LAN switch skipped!<br><br>\n";
    error( "$wrkdir/$switch_name/HEALTH_STATUS.txt not exists! " . __FILE__ . ":" . __LINE__ );
  }

  return 1;
} ## end sub print_hs_table_brcd

sub get_switch_type {
  my $switch_name = shift;
  my $switch_type = "UNKNOWN";

  if ( -f "$wrkdir/$switch_name/LAN-CISCO" ) {
    $switch_type = "CISCO";
  }

  return $switch_type;
} ## end sub get_switch_type

sub print_legend_cisco {
  print <<_MARKER_;
    <table class="legend_bottom">
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Admin status</b></td>
          </tr>
          <tr>
            <td colspan="2">The desired state of the interface.</td>
          </tr>
          <tr>
            <td class="st_green"></td>
            <td><b>up</b></td>
          </tr>
          <tr>
            <td class="st_orange"></td>
            <td><b>down</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>testing</b></td>
          </tr>
        </table>
      </td>
      <td>
        <table class="san_stat_legend">
          <tr>
            <td colspan="2"><b>Operational status</b></td>
          </tr>
          <tr>
            <td colspan="2">The current operational state of the interface.</td>
          </tr>
          <tr>
            <td class="st_green"></td>
            <td><b>up</b></td>
          </tr>
          <tr>
            <td class="st_gray"></td>
            <td><b>notPresent</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>unknown</b></td>
          </tr>
          <tr>
            <td class="st_orange"></td>
            <td><b>down</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>testing</b></td>
          </tr>
          <tr>
            <td class="st_red"></td>
            <td><b>lowerLayerDown</b></td>
          </tr>
          <tr>
            <td></td>
            <td><b>dormant</b></td>
          </tr>
        </table>
      </td>
    </table>
_MARKER_

  return 1;
} ## end sub print_legend_cisco

sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);
  print STDERR "$act_time: $text : $!\n";
  return 1;
} ## end sub error

sub read_json {
  my $file = shift;
  my $json = JSON->new;
  my $string;
  my $output;

  {
    local $/=undef;
    open ( FILE, '<', $file ) or return 0;
    $string = <FILE>;
    close FILE;
  }

  if ($string) {
    eval {
      $output = $json->decode($string);
      1;
    } or do {
      error("Couldn't parse json file $file $@ " . __FILE__ . ":" . __LINE__) && return 0;
    };
  }

  return $output;
} ## end sub read_json

sub generate_row {
  my @values = @_;
  my $row = '';

  $row .= "<tr>\n";
  foreach my $value ( @values ) {
    $row .=  "<td title=\"$value\" style=\"text-align:center; color:black;\" nowrap=\"\">$value</td>\n";
  }
  $row .=  "</tr>\n";

  return \$row;
} ## end sub generate_row

sub generate_table {
  my @header        = @{ shift @_ };
  my $rows          = ${ shift @_ };
  my $headline      = shift;
  my $table_classes = shift @_ ? 'tabcfgsum tablesortercfgsum' : 'tabcfgsum tablesorter tablesortercfgsum';
  my $acc           = "";

  if ( $rows eq '' ) {
    return \$acc;
  }

  $acc .= "<center>\n";
  $acc .= defined $headline ? "<br><br><b>$headline:</b>\n" : "<br>\n";
  $acc .= "<table class =\"$table_classes\">\n";
  $acc .= "<thead>\n";
  $acc .= "<tr>\n";

  foreach my $column ( @header ) {
    $acc .= "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">$column</th>\n";
  }

  $acc .= "</tr>\n";
  $acc .= "</thead>\n";
  $acc .= "<tbody>\n";
  $acc .= $rows;
  $acc .= "</tbody>\n";
  $acc .= "</table>\n";
  $acc .= "</center>\n";

  return \$acc;
} ## end sub generate_table

sub print_total_switches_hs_table {
  my @switches_header = ('Switch name', 'Overall operational status');
  my @files = <$wrkdir/*/HEALTH_STATUS_SWITCH.txt>;
  my $switches_rows = '';

  foreach my $switch_hs_file ( @files ) {
    $switch_hs_file =~ /$wrkdir\/(.*)\/HEALTH_STATUS_SWITCH.txt/;
    my $switch_name = $1;
    my $data_ref = read_json( $switch_hs_file );
    my %data;

    if ( $data_ref ) { %data = %{ $data_ref }; }
    else { next; }

    my $overall_status = $data{overall_status};
    if ( defined $switch_name && defined $overall_status ) {
      $switches_rows .= ${ generate_row( $switch_name, $overall_status ) };
    }
  }

  print ${ generate_table( \@switches_header, \$switches_rows ) };

  return 1;
}

sub print_modules_table {
  my $modules_hs_file = "$wrkdir/$name/HEALTH_STATUS_SWITCH.txt";
  my @modules_header = ('Module name', 'Module serial', 'Admin status', 'Operational status');
  my $last_update = localtime( ( stat($modules_hs_file) )[9] );
  my $modules_rows = '';
  my $data_ref = read_json( $modules_hs_file );
  my %data;

  if ( $data_ref ) { %data = %{ $data_ref }; }
  else { return 0; }

  foreach my $index ( sort { $a <=> $b } keys %{ $data{modules} } ) {
    my %modul        = %{ $data{modules}{ $index } };
    my $modul_name   = $modul{modul_name};
    my $modul_serial = $modul{modul_serial};
    my $admin_status = $modul{admin_status};
    my $oper_status  = $modul{oper_status};

    if ( defined $modul_name && defined $admin_status && defined $oper_status ) {
      $modules_rows .= ${ generate_row( $modul_name, $modul_serial, $admin_status, $oper_status ) };
    }
  }

  print ${ generate_table( \@modules_header, \$modules_rows ) };
  print "<br>\n";
  print "last update : $last_update";

  return 1;
} ## end sub print_modules_table
