#!/usr/bin/perl
#
# svcconfig.pl
#
# $Revision: 1.3.6 $  2018-10-03

my ($revision) = ( '$Revision: 1.3.6 $' =~ /([\d\.]+)/ );
# Changes:
# 1.3.1 Add FlashSystem 840/900 support
# 1.3.6 Correct FC / Global Mirror with Change Volume

# Modules
use strict;
use Data::Dumper;
use Storable qw(retrieve nstore);
use Date::Parse;
use Xorux_lib;

#use XML::Simple;
my $bindir   = $ENV{BINDIR};
my $webdir   = $ENV{WEBDIR};
my $inputdir = $ENV{INPUTDIR};
require "$bindir/xml.pl";    # it replaces above line and fixes an issue on Perl 5.10.1 on AIX 7100-00-06-1216

# Storage Hardware Types
my %storage_info = ( "is_svc" => 0, "is_storwize" => 0, "is_fs840" => 0, "is_fs9100" => 0, "is_v9000" => 0);

# SVC Hardware Types
my %svc_g1 = ( "CG8" => 1, "CF8" => 1, "8A4" => 1, "8G4" => 1, "8F4" => 1, "8F2" => 1, "4F2" => 1 );
my %svc_g2 = ( "DH8" => 1, "SV1" => 1 );

# Time Zone
my %tzoffset = (
  "A" => "-0100", "ACDT" => "+1030", "ACST" => "+0930", "ACT" => "-0500", "ACWDT" => "+0945", "ACWST" => "+0845", "ADDT" => "-0200", "ADT" => "-0300", "AEDT" => "+1100",
  "AEST" => "+1000", "AFT" => "+0430", "AHDT" => "-0900", "AHST" => "-1000", "AKDT" => "-0800", "AKST" => "-0900", "AKTST" => "+0600", "AKTT" => "+0500", "ALMST" => "+0700",
  "ALMT" => "+0600", "AMST" => "-0300", "AMT" => "-0400", "ANAST" => "+1200", "ANAT" => "+1200", "ANT" => "-0430", "APT" => "-0300", "AQTST" => "+0500", "AQTT" => "+0500", 
  "ARST" => "-0200", "ART" => "-0300", "ASHST" => "+0500", "ASHT" => "+0400", "AST" => "-0400", "AT-0200" => "", "AWDT" => "+0900", "AWST" => "+0800", "AWT" => "-0300",
  "AZOMT" => "+0000", "AZOST" => "+0000", "AZOT" => "-0100", "AZST" => "+0500", "AZT" => "+0400", "B" => "-0200", "BAKST" => "+0400", "BAKT" => "+0400", "BDST" => "+0700",
  "BDT" => "+0600", "BEAT" => "+0230", "BEAUT" => "+0245", "BNT" => "+0800", "BORT" => "+0800", "BORTST" => "+0820", "BOT" => "-0400", "BRST" => "-0200", "BRT" => "-0300", 
  "BST" => "+0100", "BT+0300" => "", "BTT" => "+0600", "BURT" => "+0630", "C" => "-0300", "CADT" => "+1030", "CANT" => "-0100", "CAPT" => "-0900", "CAST" => "+1100", "CAT" => "+0200",
  "CAWT" => "-0900", "CCT" => "+0630", "CDDT" => "-0400", "CDT" => "-0500", "CEMT" => "+0300", "CEST" => "+0200", "CET" => "+0100", "CGST" => "-0100", "CGT" => "-0200", "CHADT" => "+1345",
  "CHAST" => "+1245", "CHDT" => "-0530", "CHOST" => "+1000", "CHOT" => "+0800", "CHST" => "+1000", "CHUT" => "+1000", "CKHST" => "-0930", "CKT" => "-1000", "CLDT" => "-0300", "CLST" => "-0300",
  "CLT" => "-0300", "CMT" => "+0155", "COST" => "-0400", "COT" => "-0500", "CPT" => "-0500", "CST" => "-0600", "CUT" => "+0220", "CVST" => "-0100", "CVT" => "-0100", "CWT" => "-0500",
  "CXT" => "+0700", "D" => "-0400", "DACT" => "+0600", "DAVT" => "+0700", "DDUT" => "+1000", "DUSST" => "+0600", "DUST" => "+0600", "E" => "-0500", "EADT" => "+1100", "EASST" => "-0500", 
  "EAST" => "-0500", "EAT" => "+0300", "ECT" => "-0500", "EDDT" => "-0300", "EDT" => "-0400", "EEST" => "+0300", "EET" => "+0200", "EETDST" => "+0300", "EETEDT" => "+0300",
  "EGST" => "+0000", "EGT" => "-0100", "EHDT" => "-0430", "EPT" => "-0400", "EST" => "-0500", "EWT" => "-0400", "F" => "-0600", "FET" => "+0300", "FJST" => "+1300", "FJT" => "+1200",
  "FKST" => "-0300", "FKT" => "-0400", "FNST" => "-0100", "FNT" => "-0200", "FORT" => "+0500", "FRUST" => "+0600", "FRUT" => "+0600", "FST" => "+0200", "FWT" => "+0100", "G" => "-0700",
  "GALT" => "-0600", "GAMT" => "-0900", "GB+0100" => "", "GBGT" => "-0345", "GEST" => "+0400", "GET" => "+0400", "GFT" => "-0300", "GHST" => "+0020", "GILT" => "+1200", "GMT" => "+0000",
  "GMT+1" => "+0100", "GMT+10" => "+1000", "GMT+11" => "+1100", "GMT+12" => "+1200", "GMT+2" => "+0200", "GMT+3" => "+0300", "GMT+4" => "+0400", "GMT+5" => "+0500", "GMT+6" => "+0600",
  "GMT+7" => "+0700", "GMT+8" => "+0800", "GMT+9" => "+0900", "GMT-1" => "-0100", "GMT-10" => "-1000", "GMT-11" => "-1100", "GMT-12" => "-1200", "GMT-13" => "-1300", "GMT-14" => "-1400", 
  "GMT-2" => "-0200", "GMT-3" => "-0300", "GMT-4" => "-0400", "GMT-5" => "-0500", "GMT-6" => "-0600", "GMT-7" => "-0700", "GMT-8" => "-0800", "GMT-9" => "-0900", "GST" => "-0200",
  "GYT" => "-0400", "H" => "-0800", "HADT" => "-0900", "HAST" => "-1000", "HDT" => "-0930", "HKST" => "+0900", "HKT" => "+0800", "HOVST" => "+0800", "HOVT" => "+0700", "HST" => "-1000",
  "I" => "-0900", "ICT" => "+0700", "IDDT" => "+0400", "IDLE" => "+1200", "IDLW" => "-1200", "IDT" => "+0300", "IHST" => "+0600", "IOT" => "+0600", "IRDT" => "+0430", "IRKST" => "+0900",
  "IRKT" => "+0800", "IRST" => "+0330", "ISST" => "+0000", "IST" => "+0200", "IT+0330" => "", "JAVT" => "+0720", "JCST" => "+0900", "JDT" => "+1000", "JST" => "+0900", "JWST" => "+0800",
  "K" => "-1000", "KART" => "+0500", "KDT" => "+1000", "KGST" => "+0600", "KGT" => "+0600", "KIZST" => "+0600", "KIZT" => "+0500", "KOST" => "+1100", "KRAST" => "+0800", "KRAT" => "+0700",
  "KST" => "+0900", "KUYST" => "+0500", "KUYT" => "+0300", "KWAT" => "-1200", "L" => "-1100", "LHDT" => "+1100", "LHST" => "+1030", "LINT" => "+1400", "LKT" => "+0600", "M" => "-1200",
  "MADMT" => "+0100", "MADST" => "+0000", "MADT" => "-0100", "MAGST" => "+1200", "MAGT" => "+1000", "MALST" => "+0720", "MALT" => "+0730", "MART" => "-0930", "MAWT" => "+0500", "MDDT" => "-0500",
  "MDT" => "-0600", "MEST" => "+0200", "MESZ" => "+0200", "MET" => "+0100", "METDST" => "+0200", "MEWT" => "+0100", "MEZ" => "+0100", "MHT" => "+1200", "MIST" => "+1100", "MMT" => "+0630",
  "MOST" => "+0900", "MOT" => "+0800", "MPT" => "+1000", "MSD" => "+0400", "MSK" => "+0300", "MSM" => "+0500", "MST" => "-0700", "MUST" => "+0500", "MUT" => "+0400", "MVT" => "+0500",
  "MWT" => "-0600", "MYT" => "+0800", "N" => "+0100", "NCST" => "+1200", "NCT" => "+1100", "NDDT" => "-0130", "NDT" => "-0230", "NEGT" => "-0330", "NEST" => "+0120", "NET" => "+0020",
  "NFT" => "+1130", "NMT" => "+1112", "NOVST" => "+0700", "NOVT" => "+0600", "NPT" => "+0545", "NRT" => "+1200", "NST" => "-0330", "NT-1100" => "", "NUT" => "-1100", "NWT" => "-1000",
  "NZDT" => "+1300", "NZMT" => "+1130", "NZST" => "+1200", "NZT" => "+1200", "O" => "+0200", "OMSST" => "+0700", "OMST" => "+0600", "ORAST" => "+0500", "ORAT" => "+0500", "P" => "+0300",
  "PDDT" => "-0600", "PDT" => "-0700", "PEST" => "-0400", "PET" => "-0500", "PETST" => "+1200", "PETT" => "+1200", "PGT" => "+1000", "PHOT" => "+1300", "PHST" => "+0900", "PHT" => "+0800",
  "PKST" => "+0600", "PKT" => "+0500", "PMDT" => "-0200", "PMST" => "-0300", "PMT" => "+1000", "PNT" => "-0830", "PONT" => "+1100", "PPMT" => "-0449", "PPT" => "-0700", "PST" => "-0800",
  "PWT" => "+0900", "PYST" => "-0300", "PYT" => "-0400", "Q" => "+0400", "QMT" => "-0514", "QYZST" => "+0700", "QYZT" => "+0600", "R" => "+0500", "RET" => "+0400", "ROK" => "+0900",
  "ROTT" => "-0300", "S" => "+0600", "SAKST" => "+1100", "SAKT" => "+1000", "SAMST" => "+0400", "SAMT" => "+0400", "SAST" => "+0200", "SAT" => "-0400", "SBT" => "+1100", "SCT" => "+0400",
  "SDMT" => "-0440", "SDT" => "-1000", "SGT" => "+0800", "SHEST" => "+0600", "SHET" => "+0500", "SMT" => "+0216", "SRET" => "+1100", "SRT" => "-0300", "SST" => "-1100", "STAT" => "+0400",
  "SVEST" => "+0500", "SVET" => "+0400", "SWAT" => "+0130", "SWT" => "+0100", "SYOT" => "+0300", "T" => "+0700", "TAHT" => "-1000", "TASST" => "+0600", "TAST" => "+0600", "TBIST" => "+0400",
  "TBIT" => "+0400", "TFT" => "+0500", "TJT" => "+0500", "TKT" => "+1300", "TLT" => "+0900", "TMT" => "+0500", "TOST" => "+1400", "TOT" => "+1300", "TRST" => "+0400", "TRT" => "+0300",
  "TSAT" => "+0300", "TVT" => "+1200", "U" => "+0800", "ULAST" => "+0900", "ULAT" => "+0800", "URAST" => "+0500", "URAT" => "+0400", "UT+0000" => "", "UTC" => "+0000", "UYHST" => "-0230",
  "UYST" => "-0200", "UYT" => "-0300", "UZST" => "+0600", "UZT" => "+0500", "V" => "+0900", "VET" => "-0430", "VLAST" => "+1100", "VLAT" => "+1000", "VOLST" => "+0400", "VOLT" => "+0400",
  "VOST" => "+0600", "VUST" => "+1200", "VUT" => "+1100", "W" => "+1000", "WAKT" => "+1200", "WARST" => "-0300", "WART" => "-0400", "WAST" => "+0200", "WAT" => "+0100", "WEMT" => "+0200",
  "WEST" => "+0100", "WET" => "+0000", "WFT" => "+1200", "WGST" => "-0200", "WGT" => "-0300", "WIB" => "+0700", "WIT" => "+0900", "WITA" => "+0800", "WMT" => "+0124", "WSDT" => "+1400",
  "WSST" => "+1300", "X" => "+1100", "XJT" => "+0600", "Y" => "+1200", "YAKST" => "+1000", "YAKT" => "+0900", "YDDT" => "-0700", "YDT" => "-0800", "YEKST" => "+0600", "YEKT" => "+0500",
  "YERST" => "+0400", "YERT" => "+0400", "YPT" => "-0800", "YST" => "-0900", "YWT" => "-0800", "Z" => "+0000", "ZP4" => "+0400", "ZP5" => "+0500", "ZP6" => "+0600",
);

# Oprions and their default values
my $storage;    # SVC/Storwize V7000 alias
my $svc;        # SVC/Storwize V7000 cluster host name (IP address)
my $user = "admin";    # SVC/Storwize V7000 user name
my $key  = "";         # SSH key filename
my $dir  = "..";       # Data directory

my $debug      = 0;        # Debug mode 0:off, 1:on
my $fullcfgint = 86400;    # Full Config Interval (sec) ... 24H = 86400

if ( defined $ENV{STORAGE_NAME} ) {
  $storage = $ENV{STORAGE_NAME};
}
else {
  &message("SVC/Storwize V7000 storage name alias is required.");
  exit(1);
}
if ( defined $ENV{SVC_IP} ) {
  $svc = $ENV{SVC_IP};
}
else {
  &message("SVC/Storwize V7000 host name or IP address is required.");
  exit(1);
}
if ( defined $ENV{SVC_USER} ) { $user  = $ENV{SVC_USER} }
if ( defined $ENV{SVC_KEY} )  { $key   = $ENV{SVC_KEY} }
if ( defined $ENV{SVC_DIR} )  { $dir   = $ENV{SVC_DIR} }
if ( defined $ENV{DEBUG} )    { $debug = $ENV{DEBUG} }
if ( defined $ENV{SVC_DEBUG_FULL} ) { 
    if ( $ENV{SVC_DEBUG_FULL} =~ /^\s*(\d{1})\s*$/ ) { $debug = $1; }
    elsif ( $ENV{SVC_DEBUG_FULL} eq $storage ) { $debug = 1; }
    else { $debug = 0; }
}

# Main process

# Display parameters
if ($debug) {
  print( "
svcconfig.pl starts with the following parameters ...
\tRevision:                             $revision
\tSVC/Storwize V7000 cluster name:      $storage
\tSVC cluster host name:                $svc
\tSVC admin user name:                  $user
\tSVC admin user SSH key filename:      $key
\tBase Directory for output files:      $dir
\tDebug:                                $debug
\n" );
} ## end if ($debug)

#
my $fullcfgflg = 0;
my $cfgdata;
my %cfg;
my $cfg_ref = \%cfg;
my %fullcfg;
my $fullcfg_ref = \%fullcfg;
my %prevcfg;
my $prevcfg_ref = \%prevcfg;
my $ssh;
my $scp;
my $cmd_ls   = "/bin/ls";
my $cmd_mv   = "/bin/mv";
my $cmd_rm   = "/bin/rm";
my $cmd_grep = "/bin/grep";
my $putty    = 0;

my $ssh_cmd = $ENV{SSH};
if ( !defined($ssh_cmd) || $ssh_cmd eq '' ) {
  $ssh_cmd = "ssh -o ConnectTimeout=15 -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey -o SendEnv=no ";
}

if ($putty) {
  $cmd_ls   = "c:\\unix\\ls";
  $cmd_mv   = "c:\\unix\\mv";
  $cmd_rm   = "c:\\unix\\rm";
  $cmd_grep = "c:\\unix\\grep";

  #my $plink = "c:/Program Files (x86)/PuTTY/plink.exe";
  my $plink = "c:\\Progra~2\\PuTTY\\plink.exe";
  my $pscp  = "c:\\Progra~2\\PuTTY\\pscp.exe";
  $ssh = $plink . " -i $key $user\@$svc";

  # $scp = $pscp." -i $key $user\@$svc:/tmp/svc.config.backup.xml";
  $scp = $pscp . " -i $key $user\@$svc";
} ## end if ($putty)
elsif ( "x$key" eq "x" ) {    # with default keyfile (.ssh/id_rsa)
  $ssh = "$ssh_cmd $user\@$svc";

  # $scp = "scp $user\@$svc:/tmp/svc.config.backup.xml";
  $scp = "scp $user\@$svc";
} ## end elsif ( "x$key" eq "x" )
else {                        #
  $ssh = "$ssh_cmd -i $key $user\@$svc";

  # $scp = "scp -i $key  $user\@$svc:/tmp/svc.config.backup.xml";
  $scp = "scp -i $key  $user\@$svc";
} ## end else [ if ($putty) ]
my $html_chars = qr/[\x09\x0A\x0D\x20-\x{D7FF}\x{E000}-\x{FFFD}\x{10000}-\x{10FFFF}]/;
my @html_exclude_string;
if ( defined $ENV{SVC_HTML_EXCLUDE_STRING} ) {
  @html_exclude_string = split( / /, $ENV{SVC_HTML_EXCLUDE_STRING} );
}
else {
  @html_exclude_string = qw(&up; &dquot;);
}

# Date and Time
my $local_time  = localtime(time);
my $local_epoch = str2time($local_time);
my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = localtime(time);
$year += 1900;
$mon++;
my $date = sprintf( "%4d%2.2d%2.2d",   $year, $mon, $mday );
my $time = sprintf( "%2.2d%2.2d%2.2d", $hour, $min, $sec );
$cfg_ref->{date} = sprintf( "%2.2d.%2.2d.%4d", $mday, $mon, $year ) . " " . sprintf( "%2.2d:%2.2d:%2.2d", $hour, $min, $sec );

# Get Latest Config Data
my $tmp_conf_file = &GetConfigFilename();

if ( defined $tmp_conf_file ) {
  if ($debug) { &message( "Config Filename: " . $dir . "/" . $tmp_conf_file ) }
  if ( -r $dir . "/" . $tmp_conf_file ) {
    eval { $prevcfg_ref = retrieve( $dir . "/" . $tmp_conf_file ); };
    if ($@) {
      &message( $@ . __FILE__ . ":" . __LINE__ . " file:" . $dir . "/" . $tmp_conf_file );
      &message( "Retrieve data from file " . $dir . "/" . $tmp_conf_file . " failed." );
      system( "$cmd_rm -f " . $dir . "/" . $tmp_conf_file );
      $fullcfgflg = 1;
    } ## end if ($@)
    if ( !defined $prevcfg_ref ) {
      &message("Retrieve data from file $tmp_conf_file failed.");
      system( "$cmd_rm -f " . $dir . "/" . $tmp_conf_file );
      $fullcfgflg = 1;
    }
    elsif ( !scalar( %{$prevcfg_ref} ) ) {
      if ($debug) { &message("Previous configuration data is not valid.") }
      system( "$cmd_rm -f " . $dir . "/" . $tmp_conf_file );
      $fullcfgflg = 1;
    }
    elsif ( !defined $prevcfg_ref->{'fullcfg_epoch'} ) {
      if ($debug) { &message("Full Config Epoch is not find in previous data.") }
      $fullcfgflg = 1;
    }
    elsif ( $local_epoch - $prevcfg_ref->{'fullcfg_epoch'} >= $fullcfgint ) {
      $fullcfgflg = 1;
      if ($debug) {
        &message( "Full Config Time = " . localtime( $prevcfg_ref->{'fullcfg_epoch'} ) . " - epoch = " . $prevcfg_ref->{'fullcfg_epoch'} );
        my $diff = $local_epoch - $prevcfg_ref->{'fullcfg_epoch'};
        &message( "Full Config Interval = " . $diff );

      } ## end if ($debug)
    } ## end elsif ( $local_epoch - $prevcfg_ref...)
    else {
      if ($debug) {
        &message( "Full Config Time = " . localtime( $prevcfg_ref->{'fullcfg_epoch'} ) . " - epoch = " . $prevcfg_ref->{'fullcfg_epoch'} );
        my $diff = $local_epoch - $prevcfg_ref->{'fullcfg_epoch'};
        &message( "Full Config Interval = " . $diff );
      }
    } ## end else [ if ( !defined $prevcfg_ref)]
  } ## end if ( -r $dir . "/" . $tmp_conf_file)
  else {
    $fullcfgflg = 1;
    system( "$cmd_rm -f " . $dir . "/" . $tmp_conf_file );
  }
} ## end if ( defined $tmp_conf_file)
else {
  $fullcfgflg = 1;
  if ($debug) { &message("Config File is not present.") }
}

# Get SVC/Storwize Code Level
my $codelevel = &GetCodeLevel();
if ( !defined $codelevel ) {
  error( "svcconfig.pl: ERROR: Cannot get SVC/Storwize code level. Exiting.\n", 1 );
  exit 1;
}
my $codevrmn = $codelevel->{'version'} * 1000000 + $codelevel->{'release'} * 10000 + $codelevel->{'major'} * 100 + $codelevel->{'minor'};
if ( $codevrmn >= 6040000 && $codevrmn <= 7010001 ) {

  # Command svcconfig backup failed with error message
  # CMMVC6202E This command can only be run by the superuser.
  # Suggested Fixes
  # 7.1.0.2  IC90824  Configuration backup could only be run by superuser
  &InsertDataCli();
} ## end if ( $codevrmn >= 6040000...)
else {
  if ($fullcfgflg) {

    # Get SVC configuration file (XML file) from SVC cluster
    my $file_name = &GetXMLCronFile();
    if ( not defined $file_name ) {
      $file_name = &GetXMLConfigFile();
    }
    my $xml_epoch = &GetXMLConfigTime($file_name);
    if ( not defined $xml_epoch ) {
      error( "svcconfig.pl: ERROR: Cannot get time from XML file " . $file_name . ". Exiting.\n", 1 );
      exit 1;
    }
    my $pre_epoch = 0;
    if ( defined $prevcfg_ref->{'fullcfg_epoch'} ) {
      $pre_epoch = $prevcfg_ref->{'fullcfg_epoch'};
    }
    if ($debug) { &message( "XML Config File: " . $file_name . " Time: " . localtime($xml_epoch) . " Epoch: " . $xml_epoch . " Last epoch: " . $pre_epoch ); }
    if ( $pre_epoch + $fullcfgint / 2 < $xml_epoch ) {

      # new file
      $cfg_ref->{'fullcfg_epoch'} = $xml_epoch + 3600;
      &CheckConfigFile($file_name);
      $cfgdata = &ReadConfigFile($file_name);
      &InsertDataXml();
    } ## end if ( $pre_epoch + $fullcfgint...)
    else {
      # old file
      &InsertDataCli();
    }
  } ## end if ($fullcfgflg)
  else {
    &InsertDataCli();
  }
} ## end else [ if ( $codevrmn >= 6040000...)]

# Define configurations file name
my $svc_name = $cfg_ref->{svc_system}->{name};
$tmp_conf_file = $dir . "/" . $storage . "_svcconf_" . $date . "_" . $time . ".data";
my $tmp_conf_full_file = $dir . "/" . $storage . "_svcconf" . ".datafull";
my $out_conf_file      = $dir . "/" . $storage . "_svcconf_" . $date . "_" . $time . ".out";

&StoreData();
&RetriveData();

&PrintData();

### health status
my $error_status  = 0;
my $section_check = 0;
my $event_check   = 0;
my $summary_dir   = "$inputdir/tmp/health_status_summary";
my @error_message = ();

my %health_status = get_event_log($out_conf_file);

if (!-d "$webdir/$storage"){
  mkdir "$webdir/$storage" || error( "Cannot read $webdir/$storage: $!" . __FILE__ . ":" . __LINE__ );
}

#print Dumper \%health_status;
if ( $event_check != 0 && $section_check != 0 ) {
  my $table = get_health_status_html();
  write_health_status_html( "$webdir/$storage/health_status.html", $table );
  summary_health_status();
}
else {
  my $time = localtime();
  $time = $time . "<br>\n";
  my $report = "$time Storage $storage health status = Optimal\n";
  write_health_status_html( "$webdir/$storage/health_status.html", $report );
  summary_health_status();
} ## end else [ if ( $event_check != 0...)]

###

exit();

sub message {
  my $msg = shift;
  my $tm  = localtime();
  print( $tm. " - INFO    - svcconfig.pl: " . $msg . "\n" );
}

sub warning {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  print STDERR ( $tm . " - WARNING - svcconfig.pl: " . $msg . "\n" );
}

sub error {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  if ( defined $rc ) {
    print STDERR ( $tm . " - ERROR   - svcconfig.pl: " . $msg . " - Return code: " . $rc . "\n" );
  }
  else {
    print STDERR ( $tm . " - ERROR   - svcconfig.pl: " . $msg . "\n" );
  }
} ## end sub error

sub GetXMLConfigFile {
  if ($debug) { message("$ssh svcconfig backup"); }
  if ($putty) {
    system("$ssh svcconfig backup") == 0 or error( "svcconfig.pl: ERROR: Command \"$ssh svcconfig backup\" failed.\n", 1 );
    exit 1;
  }
  else {
    system("$ssh svcconfig backup 2>/dev/null >/dev/null") == 0 or die("svcconfig.pl: ERROR: Command \"$ssh svcconfig backup\" failed.\n");
  }

  if ($debug) { message( $scp . ":/tmp/svc.config.backup.xml " . $dir ); }
  system( $scp. ":/tmp/svc.config.backup.xml " . $dir ) == 0 or die("svcconfig.pl: ERROR: Command \"$scp $dir\" failed.\n");
  return ( $dir . "/svc.config.backup.xml" );
} ## end sub GetXMLConfigFile

sub ID_front_panel {
  my ( $node, $dumps_name ) = @_;

  # Create $front_panel_id
  foreach my $line_LSNODEVPD (`$ssh svcinfo lsnodevpd -delim : $node`) {

    #if($debug) {print("The svcinfo lsnodevpd command output: $line_LSNODEVPD");}
    my @field_LSNODEVPD = split( ':', $line_LSNODEVPD );
    if ( $field_LSNODEVPD[0] =~ $dumps_name ) {

      #if($debug) {message("Check: $field_LSNODEVPD[1]");}
      my $front_panel_id = $field_LSNODEVPD[1];
      chomp($front_panel_id);
      if ($debug) { message("Front panel ID/Dump name: $front_panel_id"); }
      return $front_panel_id;
    } ## end if ( $field_LSNODEVPD[...])
  } ## end foreach my $line_LSNODEVPD ...
} ## end sub ID_front_panel

sub GetXMLCronFile {
  #$debug = 1;
  my $iostatsdumps_name;
  my $filename;

  # find XML Cron config file on config node
  foreach my $line_LSNODE (`$ssh svcinfo lsnode -nohdr -delim :`) {

    #if($debug_full) {message("The svcinfo lsnode command output: $line_LSNODE");}
    my @field_LSNODE = split( ':', $line_LSNODE );

    # Check none configuration nodes
    if ( length( $field_LSNODE[2] ) == 0 && length( $field_LSNODE[15] ) != 0 ) {
      # Storwize V7000 = UPS_serial_number is blank AND enclosure_serial_number is not blank
      $iostatsdumps_name = "dump_name";    # Storwize V7000 iostats dumps file name
      if ($debug) { message("H/W type, iostats dumps file name (Storwize V7000): $iostatsdumps_name."); }
    } ## end if ( length( $field_LSNODE...))
    elsif ( length( $field_LSNODE[8] ) == 0 && length( $field_LSNODE[15] ) != 0 ) {
      # Storwize V7000 = UPS_unique_id is blank AND enclosure_serial_number is not blank
      $iostatsdumps_name = "dump_name";    # Storwize V7000 iostats dumps file name
      if ($debug) { message("H/W type, iostats dumps file name (Storwize V7000): $iostatsdumps_name."); }
    } ## end if ( length( $field_LSNODE...))
    elsif ( $svc_g1{ $field_LSNODE[9] } ) {
      $iostatsdumps_name = "front_panel_id";    # SVC - Gen1 - with UPS - iostats dumps file name
      if ($debug) { message("H/W type, iostats dumps file name (SVC): $iostatsdumps_name."); }
    }
    elsif ( $svc_g2{ $field_LSNODE[9] } ) {
      $iostatsdumps_name = "front_panel_id";    # SVC - Gen2 - with battery - iostats dumps file name
      if ($debug) { message("H/W type, iostats dumps file name (SVC): $iostatsdumps_name."); }
    }
    else {
      error( "H/W type is not recognized.\n" . $line_LSNODE );
      return undef;
    }
    if ( $field_LSNODE[4] eq "online" and $field_LSNODE[7] eq "yes" ) {
      my $node = $field_LSNODE[0];
      my $front_panel_id = &ID_front_panel( $node, $iostatsdumps_name );
      $filename = "svc.config.cron.xml_" . $front_panel_id;
      if ($debug) { message( "node: " . $node . " ID: " . $front_panel_id . " XML filename name: " . $filename ); }
      last;
    } ## end if ( $field_LSNODE[4] ...)
  } ## end foreach my $line_LSNODE (`$ssh svcinfo lsnode -nohdr -delim :`)
  if ($debug) { message( $scp . ":/dumps/" . $filename . " " . $dir . "/" . $filename ); }
  system( $scp. ":/dumps/" . $filename . " " . $dir . "/" . $filename ) == 0 or &error("svcconfig.pl: ERROR: Command \"$scp $dir\" failed.\n") && return undef;
  return ( $dir . "/" . $filename );
} ## end sub GetXMLCronFile

sub GetXMLConfigTime {
  my $filename = shift;
  if ( not -r $filename ) { &message( "Config file " . $filename . " is not readable." ); }
  open( FILE, "<$filename" ) || &message( "Cannot open file " . $filename . "." ) && return undef;
  my $epoch;
  while ( my $line = <FILE> ) {
    chomp($line);

    # timestamp="2016/10/08 01:00:10" >
    if ( $line =~ /^\s*timestamp="(.*)"\s+>\s*$/ ) {
      my @t = split( /\s+/, $1 );
      if ( defined $t[2] and exists( $tzoffset{ $t[2] } ) ) {
        $t[2] = $tzoffset{ $t[2] };
      }
      $epoch = str2time( join( " ", @t ) );
      close(FILE);
      if ( not defined $epoch ) { &error( "Get time \"" . join( " ", @t ) . "\" failed. Unknown time format.\n" ) }
      return ($epoch);
    } ## end if ( $line =~ /^\s*timestamp="(.*)"\s+>\s*$/)
  } ## end while ( my $line = <FILE>)
  close(FILE);
  return (undef);
} ## end sub GetXMLConfigTime

sub GetCodeLevel {
  my $line;
  my $delim = ":";
  my %c;
  my $cmd = $ssh . " svcinfo lssystem -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  # code_level:7.3.0.7 (build 97.5.1410080000)
  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^\s*code_level$delim(\d+).(\d+).(\d+).(\d+)\s+\(build\s+.*\)\s*$/ ) {
      $c{version} = $1;
      $c{release} = $2;
      $c{major}   = $3;
      $c{minor}   = $4;
      $c{build}   = $5;
      close(CMDOUT);
      return ( \%c );
    } ## end if ( $line =~ /^\s*code_level$delim(\d+).(\d+).(\d+).(\d+)\s+\(build\s+.*\)\s*$/)
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
  return (undef);
} ## end sub GetCodeLevel

sub GetConfigFilename {
  my @a;
  foreach my $line_LS (`$cmd_ls $dir`) {
    chomp($line_LS);

    # Select Config Data file
    if ( $line_LS =~ /^${storage}_svcconf_\d+_\d+.data$/ ) {
      push( @a, $line_LS );
    }
  } ## end foreach my $line_LS (`$cmd_ls $dir`)
  my @b = sort @a;
  return ( pop @b );
} ## end sub GetConfigFilename

sub GetCronXMLConfigFile {

  # Change STOUT buffer attribute
  $| = 1;    # disable STDOUT buffer

  # Clear SVC/Storwize V7000 iostats files  (NOTE: channge to script)
  foreach my $line_LSNODE (`$ssh svcinfo lsnode -nohdr -delim :`) {

    # if($debug_full) {message("The svcinfo lsnode command output: $line_LSNODE");}
    my @field_LSNODE = split( ':', $line_LSNODE );
    system("$ssh svctask cleardumps -prefix /dumps/iostats $field_LSNODE[0]");
    if ($debug) { message("Clear iostas files: node $field_LSNODE[1]"); }
  } ## end foreach my $line_LSNODE (`$ssh svcinfo lsnode -nohdr -delim :`)

} ## end sub GetCronXMLConfigFile

sub CheckConfigFile {
  my $xmlfile = shift;
  my $tmpfile = $dir . "/" . "svc.config.backup.xml.check";
  if ($debug) { &message(".") }
  open( my $data, '<', $xmlfile ) || die "svcconfig.pl: Cannot open file " . $xmlfile . ". Exiting.";
  open( my $out,  '>', $tmpfile ) || die "svcconfig.pl: Cannot open file " . $tmpfile . ". Exiting.";
  if ($debug) { &message( "Check XML file: " . $xmlfile ) }
  while ( my $line = <$data> ) {
    chomp($line);

    #$line =~ s/[:^print:]//g;
    $line =~ s{ ( &\# ( x[0-9A-Z]+ | [0-9]+ ) ; ) } {
            my ($entity, $code) = ($1, $2);
            $code = hex $code if $code =~ s/x//i;
            chr($code) =~ $html_chars ? $entity : '';
        }eixg;

    #if ($debug) {print("*")};
    foreach my $exclude (@html_exclude_string) {
      $line =~ s/$exclude/_/g;
    }
    print $out $line;
  } ## end while ( my $line = <$data>)
  if ($debug) { print("\n") }
  close($data);
  close($out);
} ## end sub CheckConfigFile

sub ReadConfigFile {
  my $xmldata;
  my $xmlfile = $dir . "/" . "svc.config.backup.xml.check";
  my $simple = XML::Simple->new( keyattr => [], ForceArray => 1 );
  eval { $xmldata = $simple->XMLin($xmlfile); };
  if ($@) {
    warning("XML parsing error: $@");
    message("XML parsing error. Trying to recover XML with xmllint");
    eval {
      my $linted = `xmllint --recover $xmlfile`;
      $xmldata = $simple->XMLin($linted);
    };
    if ($@) {
      &error( "XML parsing error: " . $@ . __FILE__ . ":" . __LINE__ . " file:" . $xmlfile );
      &message( "XML parsing error: File: " . $xmlfile );
      exit(1);
    }
  } ## end if ($@)
  my $data = $xmldata->{'object'};
  return $data;
} ## end sub ReadConfigFile

sub InsertDataCli {

  # Save time of full config.
  if ($fullcfgflg) {
    if ( not defined $cfg_ref->{'fullcfg_epoch'} ) { $cfg_ref->{'fullcfg_epoch'} = $local_epoch; }
    if ($debug) { &message( "Insert Full Config Epoch " . $cfg_ref->{'fullcfg_epoch'} . " to config data." ) }
  }
  else {
    $cfg_ref->{'fullcfg_epoch'} = $prevcfg_ref->{'fullcfg_epoch'};
    if ($debug) { &message( "Insert Full Config Epoch " . $cfg_ref->{'fullcfg_epoch'} . " to config data." ) }
  }
  &GetSystemDataCli();
  &GetNodeDataCli();
  &GetEnclosureDataCli();
  &GetPortfcDataCli();
  &GetDriveDataCli();
  &GetMdiskDataCli();
  &GetMdiskGrpDataCli();
  &GetVdiskDataCli();
  &GetRcrelDataCli();
  &GetFcmapDataCli();
  &GetEventDataCli();
} ## end sub InsertDataCli

sub InsertDataXml {

  # Save time of full config.
  if ($fullcfgflg) {
    if ( not defined $cfg_ref->{'fullcfg_epoch'} ) { $cfg_ref->{'fullcfg_epoch'} = $local_epoch; }
    if ($debug) { &message( "Insert Full Config Epoch " . $cfg_ref->{'fullcfg_epoch'} . " to config data." ) }
  }
  else {
    $cfg_ref->{'fullcfg_epoch'} = $prevcfg_ref->{'fullcfg_epoch'};
    if ($debug) { &message( "Insert Full Config Epoch " . $cfg_ref->{'fullcfg_epoch'} . " to config data." ) }
  }
  &GetSystemDataXml();
  &GetNodeDataXml();
  &GetEnclosureDataXml();
  &GetPortfcDataXml();
  &GetDriveDataXml();
  &GetMdiskDataXml();
  &GetMdiskGrpDataXml();
  &GetVdiskDataXml();
  &GetVdiskHostMapDataXml();
  &GetHostDataXml();
  &GetRcrelDataXml();
  &GetFcmapDataXml();
  &GetEventDataCli();
  &GetFabricDataCli();
} ## end sub InsertDataXml

sub StoreData {
  #
  nstore( $cfg_ref, $tmp_conf_file ) or die("svcconfig.pl: Store data to file $tmp_conf_file failed.");
  if ($fullcfgflg) {
    nstore( $cfg_ref, $tmp_conf_full_file ) or die("svcconfig.pl: Store data to file $tmp_conf_file failed.");
  }
} ## end sub StoreData

sub RetriveData {
  if ( -r $tmp_conf_full_file ) {
    eval { $fullcfg_ref = retrieve($tmp_conf_full_file); };
    if ($@) {
      &message( $@ . __FILE__ . ":" . __LINE__ . " file:" . $tmp_conf_full_file );
      &message("Retrieve data from file $tmp_conf_full_file failed.");
      system( "$cmd_rm -f " . $dir . "/" . $tmp_conf_full_file );
    }
  } ## end if ( -r $tmp_conf_full_file)
} ## end sub RetriveData

sub PrintData {
  #
  if ($debug) { &message("Output file: ${out_conf_file}.tmp.") }
  open( CONFOUT, ">${out_conf_file}.tmp" ) or die("Print data to file ${out_conf_file}.tmp failed.");
  &PrintSystemData();
  &PrintNodeData();
  &PrintEnclosureData();
  &PrintPortfcData();
  &PrintDriveData();
  &PrintMdiskData();
  &PrintMdiskGrpData();
  &PrintVdiskData();
  &PrintRcrelData();
  &PrintFcmapData();

  if ($fullcfgflg) {
    &PrintHostData() if ( $codevrmn < 6400 || $codevrmn > 7101 );
    &PrintFabricData();
  }
  &PrintEventData();
  close(CONFOUT);
  if ($debug) { &message("Rename output file to: ${out_conf_file}.") }
  if ( not -f ${out_conf_file} . ".tmp" ) { return; }
  my $ret = system( "$cmd_mv " . ${out_conf_file} . ".tmp " . ${out_conf_file} );
  if ($ret) { error( "svcconfig.pl: Couldn't rename file " . ${tmp_conf_file} . ".tmp ." ); return; }
} ## end sub PrintData

# -------------------------------------------------------------------------------------------------
# CLI

sub GetSystemDataCli {
  my $line;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lssystem -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || error( "svcconfig.pl: Command $cmd failed.", 1 ) && return "";
  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^(\w+)$delim(.*)$/ ) {    # system
      $cfg_ref->{svc_system}->{$1} = $2;
    }
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
  $cfg_ref->{info} = \%storage_info;
  # IBM Storwize ('product_name' => 'IBM Storwize V7000')
  if ( defined $cfg_ref->{svc_system}->{product_name} && $cfg_ref->{svc_system}->{product_name} =~ m/^IBM\s+Storwize\s+V7000/ ) {
    $cfg_ref->{info}->{is_storwize} = 1;
  }
  # IBM FlashSystem ('product_name' => 'IBM SAN Volume Controller')
  if ( defined $cfg_ref->{svc_system}->{product_name} && $cfg_ref->{svc_system}->{product_name} =~ m/^IBM\s+SAN\s+Volume\s+Controller/ ) {
    $cfg_ref->{info}->{is_svc} = 1;
  }
  # IBM FlashSystem V9000 ('product_name' => 'IBM FlashSystem V9000')
  if ( defined $cfg_ref->{svc_system}->{product_name} && $cfg_ref->{svc_system}->{product_name} =~ m/^IBM\s+FlashSystem\s+V9000$/ ) {
    $cfg_ref->{info}->{is_v9000} = 1;
  }
  # IBM FlashSystem 840/900
  if ( defined $cfg_ref->{svc_system}->{product_name} && $cfg_ref->{svc_system}->{product_name} =~ m/^IBM\s+FlashSystem\s+840$/ || $cfg_ref->{svc_system}->{product_name} =~ m/^IBM\s+FlashSystem\s+900$/ ) {
    $cfg_ref->{info}->{is_fs840} = 1;
    $cfg_ref->{svc_system}->{ssd_tier_capacity} = $cfg_ref->{svc_system}->{space_in_mdisk_grps};
    $cfg_ref->{svc_system}->{ssd_tier_free_capacity} = $cfg_ref->{svc_system}->{total_free_space};
    #
    $cfg_ref->{svc_system}->{enterprise_tier_capacity} = "0.0GB";
    $cfg_ref->{svc_system}->{enterprise_tier_free_capacity} = "0.0GB";
    $cfg_ref->{svc_system}->{nearline_tier_capacity} = "0.0GB";
    $cfg_ref->{svc_system}->{nearline_tier_free_capacity} = "0.0GB";
    $cfg_ref->{svc_system}->{ssdnl_tier_capacity} = "0.0GB";
    $cfg_ref->{svc_system}->{ssdnl_tier_free_capacity} = "0.0GB";
  }  
} ## end sub GetSystemDataCli

sub GetNodeDataCli {
  my $line;
  my @header;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lsnode -bytes -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || error( "svcconfig.pl: Command $cmd failed. Exiting.", 1 ) && exit 1;

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}name${delim}UPS_serial_number.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      $cfg_ref->{svc_node}->{ $record{name} } = \%record;
    }
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetNodeDataCli

sub GetEnclosureDataCli {
  my $line;
  my @header;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lsenclosure -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || error( "svcconfig.pl: Command $cmd failed. Exiting.", 1 ) && exit 1;

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}status${delim}type.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      $cfg_ref->{svc_enclosure}->{ $record{id} } = \%record;
    }
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetEnclosureDataCli

sub GetPortfcDataCli {
  my $line;
  my @header;
  my $delim = ",";
  my $cmd   = $ssh . " svcinfo lsportfc -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}fc_io_port_id${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    } elsif ( $line =~ /^id${delim}adapter_id${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    } else {
      my $id;
      my %record;
      @record{@header} = split( /$delim/, $line );
      if ( length( $record{node_id} ) == 0 ) { next; }
      if ( $cfg_ref->{info}->{is_fs840} and defined $record{adapter_id} ) {
        $id = $record{node_name} . "_p" . sprintf( "%d", $record{adapter_id} ) . sprintf( "%d", $record{port_id} );
        my $inerface_id = $record{node_name} . "_interface" . sprintf("%d", $record{adapter_id});
        if ( not defined $cfg_ref->{svc_interfacefc}->{$inerface_id} ) {
          my %h;
          $h{name} = $inerface_id;
          $h{node_name} = $record{node_name};
          $h{adapter_id} = $record{adapter_id};
          $cfg_ref->{svc_interfacefc}->{$inerface_id}=\%h;
        }
      } else {
        $id = $record{node_name} . "_p" . sprintf( "%02d", $record{port_id} );
      }
      $cfg_ref->{svc_portfc}->{$id} = \%record;
    } ## end else [ if ( $line =~ /^id${delim}fc_io_port_id${delim}.*$/)]
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetPortfcDataCli

sub GetDriveDataCli {
  my $line;
  my @header;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lsdrive -bytes -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}status${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_drive}->{$id} = \%record;
    } ## end else [ if ( $line =~ /^id${delim}status${delim}.*$/)]
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetDriveDataCli

sub GetMdiskDataCli {
  my $line;
  my @header;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lsmdisk -bytes -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}name${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    elsif ( $line =~ /^id${delim}status${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      if ( $cfg_ref->{info}->{is_fs840} ) {
        $record{mdisk_grp_id} = 0;
        $record{mdisk_grp_name} = "mdiskgrp0";
      }
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_mdsk}->{$id} = \%record;
    } ## end else [ if ( $line =~ /^id${delim}name${delim}.*$/)]
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetMdiskDataCli

sub InitMdiskGrpRecord {
  my $c = shift;
  $c->{capacity}                          = 0;
  $c->{free_capacity}                     = 0;
  $c->{virtual_capacity}                  = 0;
  $c->{used_capacity}                     = 0;
  $c->{real_capacity}                     = 0;
  $c->{tier_0_capacity}                   = 0;
  $c->{tier_0_free_capacity}              = 0;
  $c->{tier_1_capacity}                   = 0;
  $c->{tier_1_free_capacity}              = 0;
  $c->{tier_2_capacity}                   = 0;
  $c->{tier_2_free_capacity}              = 0;
  $c->{compression_active}                = "no";
  $c->{compression_virtual_capacity}      = 0;
  $c->{compression_compressed_capacity}   = 0;
  $c->{compression_uncompressed_capacity} = 0;
  $c->{extent_size} = 0;
  $c->{overallocation} = 0;
  $c->{warning} = 0;
} ## end sub InitMdiskGrpRecord

sub GetMdiskGrpDataCli {
  my $line;
  my @header;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lsmdiskgrp -bytes -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}name${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    elsif ( $line =~ /^id${delim}status${delim}mdisk_count${delim}vdisk_count${delim}.*$/ ) {    # header fs840
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      &InitMdiskGrpRecord( \%record );
      @record{@header} = split( /$delim/, $line );
      if ( not defined $record{easy_tier} ) { $record{easy_tier} = "no"; }
      if ( not defined $record{easy_tier_status} ) { $record{easy_tier_status} = "no"; }
      if ( not defined $record{compression_active} ) { $record{compression_active} = "no"; }
      if ( not defined $record{name} and defined $record{parent_mdisk_grp_name} ) { $record{name} = $record{parent_mdisk_grp_name} } # FS840
      my $id = $record{name};
      $cfg_ref->{svc_pool}->{$id} = \%record;
    } ## end else [ if ( $line =~ /^id${delim}name${delim}.*$/)]
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetMdiskGrpDataCli

sub GetVdiskDataCli {
  my $line;
  my @header;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lsvdisk -bytes -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}name${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_vdsk}->{$id} = \%record;
    } ## end else [ if ( $line =~ /^id${delim}name${delim}.*$/)]
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetVdiskDataCli

sub GetRcrelDataCli {
  my $line;
  my @header;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lsrcrelationship -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}name${delim}master_cluster_id${delim}.*$/ ) {    # header - id,name,master_cluster_id,master_cluster_name
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_rcrel}->{$id} = \%record;
    } ## end else [ if ( $line =~ /^id${delim}name${delim}master_cluster_id${delim}.*$/)]
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetRcrelDataCli

sub GetFcmapDataCli {
  my $line;
  my @header;
  my $delim = ":";
  my $cmd   = $ssh . " svcinfo lsfcmap -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^id${delim}name${delim}source_vdisk_id${delim}.*$/ ) {    # header - id,name,source_vdisk_id,source_vdisk_name,target_vdisk_id,target_vdisk_name,
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_fcmap}->{$id} = \%record;
    } ## end else [ if ( $line =~ /^id${delim}name${delim}source_vdisk_id{delim}.*$/)]
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetFcmapDataCli

sub GetEventDataCli {
  my $line;
  my @header;
  my $delim = ":";

  # lseventlog -delim , -alert yes -fixed no -message no -monitoring no -expired no -order date
  my $cmd = $ssh . " svcinfo lseventlog -alert yes -fixed no -message no -monitoring no -expired no -order date -delim " . $delim;
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  # sequence_number,last_timestamp,object_type,object_id,object_name,copy_id,status,fixed,event_id,error_code,description
  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^sequence_number${delim}last_timestamp${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      my $id = sprintf( "%06d", $record{sequence_number} );
      $cfg_ref->{svc_event}->{$id} = \%record;
    } ## end else [ if ( $line =~ /^sequence_number${delim}last_timestamp${delim}.*$/)]
  } ## end while ( $line = <CMDOUT> )
  close(CMDOUT);
} ## end sub GetEventDataCli

sub GetFabricDataCli {
  my $line;
  my @data;
  my @header;
  my $delim = ":";
  my $cmd;
  if ( $cfg_ref->{info}->{is_fs840} ) {
    $cmd   = $ssh . " svcinfo lsfabric840 -delim " . $delim;
  } else {
    $cmd   = $ssh . " svcinfo lsfabric -delim " . $delim;
  }
  if ($debug) { &message("Command: $cmd.") }
  open( CMDOUT, "$cmd |" ) || die "svcconfig.pl: Command $cmd failed. Exiting.";

  while ( $line = <CMDOUT> ) {
    chomp($line);
    if ( $line =~ /^remote_wwpn${delim}remote_nportid${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    elsif ( $line =~ /^remote_uid${delim}local_uid${delim}.*$/ ) {    # header
      @header = split( /$delim/, $line );
    }
    else {
      my %record;
      @record{@header} = split( /$delim/, $line );
      push( @data, \%record );
    }
  } ## end while ( $line = <CMDOUT> )
  $cfg_ref->{svc_fabric} = \@data;
  close(CMDOUT);
} ## end sub GetFabricDataCli

# -------------------------------------------------------------------------------------------------
# XML
sub GetSystemDataXml {
  my $p;
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "cluster" ) {
      $p = $n->{'property'};
    }
  }
  my $tiername;  my $tierunknown;
  foreach my $m ( @{$p} ) {
    if ( $m->{'name'} eq "tier" ) {
      if ( $m->{'value'} =~ /ssd/ || $m->{'value'} =~ /tier0_flash/ ) {
        $tiername = "ssd";
      }
      elsif ( $m->{'value'} =~ /tier1_flash/ ) {
        $tiername = "ssdnl";
      }
      elsif ( $m->{'value'} =~ /enterprise/ || $m->{'value'} =~ /hdd/ ) {
        $tiername = "enterprise";
      }
      elsif ( $m->{'value'} =~ /nearline/ || $m->{'value'} =~ /Nearline/ ) {
        $tiername = "nearline";
      }
      else {
        $tierunknown = "Unknown tiername " . $m->{'value'} . ".";
        $tiername = "tier_unknown";
      }
    } ## end if ( $m->{'name'} eq "tier")
    elsif ( $m->{'name'} eq "tier_capacity" ) {
      $cfg_ref->{svc_system}->{ $tiername . "_" . $m->{'name'} } = $m->{'value'};
    }
    elsif ( $m->{'name'} eq "tier_free_capacity" ) {
      $cfg_ref->{svc_system}->{ $tiername . "_" . $m->{'name'} } = $m->{'value'};
    }
    else {
      $cfg_ref->{svc_system}->{ $m->{'name'} } = $m->{'value'};
    }
  } ## end foreach my $m ( @{$p} )
  if ( !defined $cfg_ref->{svc_system}->{ssd_tier_capacity} )             { $cfg_ref->{svc_system}->{ssd_tier_capacity}             = 0; }
  if ( !defined $cfg_ref->{svc_system}->{ssdnl_tier_capacity} )           { $cfg_ref->{svc_system}->{ssdnl_tier_capacity}           = 0; }
  if ( !defined $cfg_ref->{svc_system}->{enterprise_tier_capacity} )      { $cfg_ref->{svc_system}->{enterprise_tier_capacity}      = 0; }
  if ( !defined $cfg_ref->{svc_system}->{nearline_tier_capacity} )        { $cfg_ref->{svc_system}->{nearline_tier_capacity}        = 0; }
  if ( !defined $cfg_ref->{svc_system}->{ssd_tier_free_capacity} )        { $cfg_ref->{svc_system}->{ssd_tier_free_capacity}        = 0; }
  if ( !defined $cfg_ref->{svc_system}->{ssdnl_tier_free_capacity} )      { $cfg_ref->{svc_system}->{ssdnl_tier_free_capacity}      = 0; }
  if ( !defined $cfg_ref->{svc_system}->{enterprise_tier_free_capacity} ) { $cfg_ref->{svc_system}->{enterprise_tier_free_capacity} = 0; }
  if ( !defined $cfg_ref->{svc_system}->{nearline_tier_free_capacity} )   { $cfg_ref->{svc_system}->{nearline_tier_free_capacity}   = 0; }
  #
  $cfg_ref->{info} = \%storage_info;
  # IBM Storwize ('product_name' => 'IBM Storwize V7000')
  if ( defined $cfg_ref->{svc_system}->{product_name} && $cfg_ref->{svc_system}->{product_name} =~ m/^IBM\s+Storwize\s+V7000/ ) {
    $cfg_ref->{info}->{is_storwize} = 1;
  }
  # IBM FlashSystem ('product_name' => 'IBM SAN Volume Controller')
  if ( defined $cfg_ref->{svc_system}->{product_name} && $cfg_ref->{svc_system}->{product_name} =~ m/^IBM\s+SAN\s+Volume\s+Controller/ ) {
    $cfg_ref->{info}->{is_svc} = 1;
  }
  # IBM FlashSystem
  if ( defined $cfg_ref->{svc_system}->{product_name} && $cfg_ref->{svc_system}->{product_name} =~ m/^IBM\s+FlashSystem/ ) {
    $cfg_ref->{info}->{is_fs840} = 1;
    $cfg_ref->{svc_system}->{ssd_tier_capacity} = $cfg_ref->{svc_system}->{space_in_mdisk_grps};
    $cfg_ref->{svc_system}->{ssd_tier_free_capacity} = $cfg_ref->{svc_system}->{total_free_space};
  } else {
    if ( $tierunknown ) { &warning($tierunknown); }
  }
} ## end sub GetSystemDataXml

sub GetNodeDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "node" ) {
      my %record;
      foreach my $m ( @{ $n->{'property'} } ) {
        $record{ $m->{'name'} } = $m->{'value'};
      }
      $cfg_ref->{svc_node}->{ $record{name} } = \%record;
    } ## end if ( $n->{'type'} eq "node")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetNodeDataXml

sub GetEnclosureDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "enclosure" ) {
      my %record;
      foreach my $m ( @{ $n->{'property'} } ) {
        $record{ $m->{'name'} } = $m->{'value'};
      }
      $cfg_ref->{svc_enclosure}->{ $record{id} } = \%record;
    } ## end if ( $n->{'type'} eq "enclosure")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetEnclosureDataXml

sub GetPortfcDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "portfc" ) {
      my %record;
      my $id;
      foreach my $m ( @{ $n->{'property'} } ) {
        $record{ $m->{'name'} } = $m->{'value'};
      }
      if ( length( $record{node_id} ) == 0 ) { next; }
      if ( $cfg_ref->{info}->{is_fs840} and defined $record{adapter_id} ) {
      	$id = $record{node_name} . "_p" . sprintf( "%d", $record{adapter_id} ) . sprintf( "%d", $record{port_id} );
        my $inerface_id = $record{node_name} . "_interface" . sprintf("%d", $record{adapter_id});
        if ( not defined $cfg_ref->{svc_interfacefc}->{$inerface_id} ) {
          my %h;
          $h{name} = $inerface_id;
          $h{node_name} = $record{node_name};
          $h{adapter_id} = $record{adapter_id};
          $cfg_ref->{svc_interfacefc}->{$inerface_id}=\%h;
        }
      } else {
        $id = $record{node_name} . "_p" . sprintf( "%02d", $record{port_id} );
      }
      $cfg_ref->{svc_portfc}->{$id} = \%record;
    } ## end if ( $n->{'type'} eq "portfc")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetPortfcDataXml

sub GetDriveDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "drive" ) {
      my %record;
      foreach my $m ( @{ $n->{'property'} } ) {
        $record{ $m->{'name'} } = $m->{'value'};
      }
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_drive}->{$id} = \%record;
    } ## end if ( $n->{'type'} eq "drive")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetDriveDataXml

sub GetMdiskDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "mdisk" ) {
      my %record;
      foreach my $m ( @{ $n->{'property'} } ) {
        $record{ $m->{'name'} } = $m->{'value'};
      }
      if ( $cfg_ref->{info}->{is_fs840} ) {
        $record{mdisk_grp_id} = 0;
        $record{mdisk_grp_name} = "mdiskgrp0";
      }
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_mdsk}->{$id} = \%record;
    } ## end if ( $n->{'type'} eq "mdisk")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetMdiskDataXml

sub GetMdiskGrpDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "mdisk_grp" ) {
      my %record;
      my $tiername;
      foreach my $m ( @{ $n->{'property'} } ) {
        if ( $m->{'name'} eq "tier" ) {
          if ( $m->{'value'} =~ /ssd/ || $m->{'value'} =~ /tier0_flash/ ) {

            # SSD
            $tiername = "tier_0";
          }
          elsif ( $m->{'value'} =~ /tier1_flash/ ) {

            # SSD
            $tiername = "tier_0n";
          }
          elsif ( $m->{'value'} =~ /enterprise/ || $m->{'value'} =~ /hdd/ ) {

            # Enterprise
            $tiername = "tier_1";
          }
          elsif ( $m->{'value'} =~ /nearline/ || $m->{'value'} =~ /Nearline/ ) {

            # Nearline
            $tiername = "tier_2";
          }
          else {
            &warning( "Unknown tiername " . $m->{'value'} . "." );
            $tiername = "tier_unknown";
          }
          $record{$tiername} = $m->{'value'};
        } ## end if ( $m->{'name'} eq "tier")
        elsif ( $m->{'name'} eq "tier_mdisk_count" ) {
          $record{ $tiername . "_mdisk_count" } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "tier_capacity" ) {
          $record{ $tiername . "_capacity" } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "tier_free_capacity" ) {
          $record{ $tiername . "_free_capacity" } = $m->{'value'};
        }
        else {
          $record{ $m->{'name'} } = $m->{'value'};
        }
      } ## end foreach my $m ( @{ $n->{'property'...}})
      if ( not defined $record{easy_tier} ) { $record{easy_tier} = "no"; }
      if ( not defined $record{easy_tier_status} ) { $record{easy_tier_status} = "no"; }
      if ( not defined $record{compression_active} ) { $record{compression_active} = "no"; }
      if ( not defined $record{extent_size} ) { $record{extent_size} = 0; }
      if ( not defined $record{overallocation} ) { $record{overallocation} = 0; }
      if ( not defined $record{warning} ) { $record{warning} = 0; }
      my $id = $record{name};
      $cfg_ref->{svc_pool}->{$id} = \%record;
    } ## end if ( $n->{'type'} eq "mdisk_grp")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetMdiskGrpDataXml

sub GetVdiskDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "vdisk" ) {
      my %record;

      #         foreach my $m ( @{$n->{'property'}} ) {
      #             $record{$m->{'name'}} = $m->{'value'};
      #         }
      #         my $id = sprintf("%04d",$record{id});
      #         $cfg_ref->{svc_vdsk}->{$id} = \%record;
      my $tiername;
      my $copyid = 0;
      foreach my $m ( @{ $n->{'property'} } ) {

        #$record{$m->{'name'}} = $m->{'value'};
        if ( $m->{'name'} eq "copy_id" ) {
          $copyid = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "tier" ) {
          if ( $m->{'value'} =~ /ssd/ || $m->{'value'} =~ /tier0_flash/ ) {

            # SSD
            $tiername = "tier_0";
            $record{ "copy_" . $copyid . "_" . $tiername } = $m->{'value'};
          } ## end if ( $m->{'value'} =~ ...)
          elsif ( $m->{'value'} =~ /tier1_flash/ ) {

            # SSD
            $tiername = "tier_0n";
            $record{ "copy_" . $copyid . "_" . $tiername } = $m->{'value'};
          } ## end elsif ( $m->{'value'} =~ ...)
          elsif ( $m->{'value'} =~ /enterprise/ || $m->{'value'} =~ /hdd/ ) {

            # Enterprise
            $tiername = "tier_1";
            $record{ "copy_" . $copyid . "_" . $tiername } = $m->{'value'};
          } ## end elsif ( $m->{'value'} =~ ...)
          elsif ( $m->{'value'} =~ /nearline/ || $m->{'value'} =~ /Nearline/ ) {

            # Nearline
            $tiername = "tier_2";
            $record{ "copy_" . $copyid . "_" . $tiername } = $m->{'value'};
          } ## end elsif ( $m->{'value'} =~ ...)
          else {
            &warning( "Unknown tiername " . $m->{'value'} . "." );
            $tiername = "tier_unknown";
          }
        } ## end elsif ( $m->{'name'} eq "tier")
        elsif ( $m->{'name'} eq "tier_capacity" ) {
          $record{ "copy_" . $copyid . "_" . $tiername . "_capacity" } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "mdisk_grp_id" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "mdisk_grp_name" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "used_capacity" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "real_capacity" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "free_capacity" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "se_copy" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "overallocation" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "compressed_copy" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "uncompressed_used_capacity" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "easy_tier" ) {
          $record{ "copy_" . $copyid . "_" . $m->{'name'} } = $m->{'value'};
        }
        else {
          $record{ $m->{'name'} } = $m->{'value'};
        }
      } ## end foreach my $m ( @{ $n->{'property'...}})
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_vdsk}->{$id} = \%record;

      #         $cfg{$id} = \%record;

    } ## end if ( $n->{'type'} eq "vdisk")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetVdiskDataXml

sub GetFcmapDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "fcmap" ) {
      my %record;
      foreach my $m ( @{$n->{'property'}} ) {
        $record{$m->{'name'}} = $m->{'value'};
      }       
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_fcmap}->{$id} = \%record;
    } ## end if ( $n->{'type'} eq "fcmap")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetFcmapDataXml

sub GetRcrelDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "rcrelationship" ) {
      my %record;
      foreach my $m ( @{$n->{'property'}} ) {
        $record{$m->{'name'}} = $m->{'value'};
      }       
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_rcrel}->{$id} = \%record;
    } ## end if ( $n->{'type'} eq "rcrelationship")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetRcrelDataXml

sub GetHostDataXml {
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "host" ) {
      my %record;
      my $i = 0;
      foreach my $m ( @{ $n->{'property'} } ) {
        if ( $m->{'name'} eq "WWPN" ) {
          $record{ "FC" . $i . "_" . $m->{'name'} } = $m->{'value'};
          $i++;
        }
        elsif ( $m->{'name'} eq "node_logged_in_count" ) {
          $record{ "FC" . $i . "_" . $m->{'name'} } = $m->{'value'};
        }
        elsif ( $m->{'name'} eq "state" ) {
          $record{ "FC" . $i . "_" . $m->{'name'} } = $m->{'value'};
        }
        else {
          $record{ $m->{'name'} } = $m->{'value'};
        }
      } ## end foreach my $m ( @{ $n->{'property'...}})
      my $id = sprintf( "%04d", $record{id} );
      $cfg_ref->{svc_host}->{$id} = \%record;
    } ## end if ( $n->{'type'} eq "host")
  } ## end foreach my $n ( @{$cfgdata})
} ## end sub GetHostDataXml

sub GetVdiskHostMapDataXml {
  my @a;
  foreach my $n ( @{$cfgdata} ) {
    if ( $n->{'type'} eq "vdiskhostmap" ) {
      my %record;
      my $i = 0;
      foreach my $m ( @{ $n->{'property'} } ) {
        $record{ $m->{'name'} } = $m->{'value'};
      }
      my $id = sprintf( "%04d", $record{vdisk_id} );
      $record{vdisk_id} = $id;
      push( @a, \%record );
    } ## end if ( $n->{'type'} eq "vdiskhostmap")
  } ## end foreach my $n ( @{$cfgdata})
  $cfg_ref->{svc_vdiskhostmap} = \@a;
} ## end sub GetVdiskHostMapDataXml

# -------------------------------------------------------------------------------------------------

sub PrintSystemData {
  print CONFOUT "\nConfiguration Data\n";
  print CONFOUT "------------------\n";
  print CONFOUT "\tMachine Name:                    ", $cfg_ref->{svc_system}->{name}, "\n";
  print CONFOUT "\tMachine Type-Model:              ", &MachineTypeModel, "\n";
  print CONFOUT "\tMachine Serial:                  ", &MachineSerial,    "\n";
  print CONFOUT "\tCode Level:                      ", $cfg_ref->{svc_system}->{code_level},, "\n";
  if(defined $cfg_ref->{svc_system}->{'product_name'}) {
  print CONFOUT "\tProduct Name:                    ", $cfg_ref->{svc_system}->{'product_name'}, "\n"
  }
  print CONFOUT "\n";
  print CONFOUT "\tPerformance Statistic Status:    ", $cfg_ref->{svc_system}->{statistics_status},,    "\n";
  print CONFOUT "\tPerformance Statistic Frequency: ", $cfg_ref->{svc_system}->{statistics_frequency},, "\n";
  print CONFOUT "\n";
  print CONFOUT "\tDate:           ", $cfg_ref->{date}, "\n";
  print CONFOUT "\n";
  print CONFOUT "\tTotal Mdisk Capacity:            ", $cfg_ref->{svc_system}->{total_mdisk_capacity},      "\n";
  print CONFOUT "\tSpace in Pools:                  ", $cfg_ref->{svc_system}->{space_in_mdisk_grps},       "\n";
  print CONFOUT "\tSpace Allocated to Volumes:      ", $cfg_ref->{svc_system}->{space_allocated_to_vdisks}, "\n";
  print CONFOUT "\n";
  print CONFOUT "\tTotal Free_space:                ", $cfg_ref->{svc_system}->{total_free_space},                "\n";
  print CONFOUT "\tTotal Used_capacity:             ", $cfg_ref->{svc_system}->{total_used_capacity},             "\n";
  print CONFOUT "\tTotal Overallocation:            ", $cfg_ref->{svc_system}->{total_overallocation},            "\n";
  print CONFOUT "\tTotal Volume Capacity:           ", $cfg_ref->{svc_system}->{total_vdisk_capacity},            "\n";
  print CONFOUT "\tTotal Volumecopy Capacity:       ", $cfg_ref->{svc_system}->{total_vdiskcopy_capacity},        "\n";
  print CONFOUT "\tTotal Allocated Extent Capacity: ", $cfg_ref->{svc_system}->{total_allocated_extent_capacity}, "\n";
  print CONFOUT "\n";

  if ( defined $cfg_ref->{svc_system}->{ssd_tier_capacity} ) {
    print CONFOUT "\t                                 Flash  \tEnterprise     \tNearline_HDD     \tNearline_Flash\n";
    print CONFOUT "\tTier Capacity:                   ", $cfg_ref->{svc_system}->{ssd_tier_capacity},      "    \t", $cfg_ref->{svc_system}->{enterprise_tier_capacity},      "    \t", $cfg_ref->{svc_system}->{nearline_tier_capacity},      "    \t", $cfg_ref->{svc_system}->{ssdnl_tier_capacity},      "\n";
    print CONFOUT "\tTier Free Capacity:              ", $cfg_ref->{svc_system}->{ssd_tier_free_capacity}, "    \t", $cfg_ref->{svc_system}->{enterprise_tier_free_capacity}, "    \t", $cfg_ref->{svc_system}->{nearline_tier_free_capacity}, "    \t", $cfg_ref->{svc_system}->{ssdnl_tier_free_capacity}, "\n";
  } ## end if ( defined $cfg_ref->...)
  print CONFOUT "\n";
} ## end sub PrintSystemData

sub PrintNodeData {
  my $delim = ",";

  # Add product_mtm
  # my @keylist = split(/,/,"id,name,UPS_serial_number,WWNN,status,IO_group_id,IO_group_name,config_node,UPS_unique_id,hardware,iscsi_name,iscsi_alias,panel_name,enclosure_id,canister_id,enclosure_serial_number");
  my @keylist = split( /,/, "id,name,UPS_serial_number,WWNN,status,IO_group_id,IO_group_name,config_node,UPS_unique_id,hardware,iscsi_name,iscsi_alias,panel_name,enclosure_id,canister_id,product_mtm,enclosure_serial_number" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nNode Level Configuration\n";
  print CONFOUT "------------------------\n";
  print CONFOUT $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_node} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_node}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_node}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintNodeData

sub MachineTypeModel {
  my @a;
  if ( defined $cfg_ref->{svc_enclosure} ) {
    my @reclist = keys( %{ $cfg_ref->{svc_enclosure} } );
    foreach my $rec ( sort(@reclist) ) {
      if ( $cfg_ref->{svc_enclosure}->{$rec}->{type} eq "control" ) {
        push( @a, $cfg_ref->{svc_enclosure}->{$rec}->{product_MTM} );
      }
    }
  } ## end if ( defined $cfg_ref->...)
  else {
    my @reclist = keys( %{ $cfg_ref->{svc_node} } );
    foreach my $rec ( sort(@reclist) ) {
      if ( defined $cfg_ref->{svc_node}->{$rec}->{product_mtm} ) {
        push( @a, $cfg_ref->{svc_node}->{$rec}->{product_mtm} );
      }
      else {
        push( @a, "2145-" . $cfg_ref->{svc_node}->{$rec}->{hardware} );
      }

    } ## end foreach my $rec ( sort(@reclist...))
  } ## end else [ if ( defined $cfg_ref->...)]
  return ( join( ",", @a ) );
} ## end sub MachineTypeModel

sub MachineSerial {
  my @a;
  if ( defined $cfg_ref->{svc_enclosure} ) {
    my @reclist = keys( %{ $cfg_ref->{svc_enclosure} } );
    foreach my $rec ( sort(@reclist) ) {
      if ( $cfg_ref->{svc_enclosure}->{$rec}->{type} eq "control" ) {
        push( @a, $cfg_ref->{svc_enclosure}->{$rec}->{serial_number} );
      }
    }
  } ## end if ( defined $cfg_ref->...)
  else {
    my @reclist = keys( %{ $cfg_ref->{svc_node} } );
    foreach my $rec ( sort(@reclist) ) {
      if ( defined $cfg_ref->{svc_node}->{$rec}->{enclosure_serial_number} ) {
        push( @a, $cfg_ref->{svc_node}->{$rec}->{enclosure_serial_number} );
      }
    }
  } ## end else [ if ( defined $cfg_ref->...)]
  if (@a) {
    return ( join( ",", @a ) );
  }
  else {
    return;
  }
} ## end sub MachineSerial

sub PrintEnclosureData {
  if ( not defined $cfg_ref->{svc_enclosure} ) { return; }
  my $delim   = ",";
  my @keylist = split( /,/, "id,status,type,managed,IO_group_id,IO_group_name,product_MTM,serial_number,total_canisters,online_canisters,total_PSUs,online_PSUs,drive_slots,machine_part_number" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nEnclosure Level Configuration(V7000 Series)\n";
  print CONFOUT "-------------------------------------------\n";
  print CONFOUT $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_enclosure} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_enclosure}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_enclosure}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintEnclosureData

sub PrintPortfcData {
  my $delim   = ",";
  my @keylist;
  if ( $cfg_ref->{'info'}->{'is_fs840'} ) {
    @keylist = split( /,/, "id,adapter_id,port_id,type,port_speed,node_id,node_name,WWPN,nportid,status,attachment" );
  } else {
    @keylist = split( /,/, "id,fc_io_port_id,port_id,type,port_speed,node_id,node_name,WWPN,nportid,status,attachment" );
  }
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nPort Level Configuration - Fibre Channel Ports\n";
  print CONFOUT "-----------------------------------------------\n";
  print CONFOUT "port_name", $delim, $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_portfc} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    push( @a, $rec );
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_portfc}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_portfc}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintPortfcData

sub PrintDriveData {
  my $delim   = ",";
  my @keylist = split( /,/, "id,status,error_sequence_number,use,tech_type,capacity,mdisk_id,mdisk_name,member_id,enclosure_id,slot_id,node_id,node_name" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nDrive Level Configuration\n";
  print CONFOUT "------------------------\n";
  print CONFOUT "drive_id", $delim, $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_drive} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    push( @a, $rec );
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_drive}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_drive}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintDriveData

sub PrintMdiskData {
  my $delim   = ",";
  my @keylist = split( /,/, "id,name,status,mode,mdisk_grp_id,mdisk_grp_name,capacity,ctrl_LUN_#,controller_name,UID,tier" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nManaged Disk Level Configuration\n";
  print CONFOUT "------------------------\n";
  print CONFOUT "mdisk_id", $delim, $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_mdsk} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    push( @a, $rec );
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_mdsk}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_mdsk}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintMdiskData

sub PrintMdiskGrpData {
  my $delim   = ",";
  my @keylist = split( /,/, "name,id,status,mdisk_count,vdisk_count,capacity,extent_size,free_capacity,virtual_capacity,used_capacity,real_capacity,overallocation,warning,easy_tier,easy_tier_status,compression_active,compression_virtual_capacity,compression_compressed_capacity,compression_uncompressed_capacity" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nPool Level Configuration\n";
  print CONFOUT "------------------------\n";
  print CONFOUT $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_pool} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_pool}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_pool}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintMdiskGrpData

sub PrintVdiskData {
  my $delim   = ",";
  my @keylist = split( /,/, "id,name,IO_group_id,IO_group_name,status,mdisk_grp_id,mdisk_grp_name,capacity,type,FC_id,FC_name,RC_id,RC_name,vdisk_UID,fc_map_count,copy_count,fast_write_state,se_copy_count,RC_change,compressed_copy_count" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nVolume Level Configuration\n";
  print CONFOUT "--------------------------\n";
  print CONFOUT "volume_id", $delim, $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_vdsk} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    push( @a, $rec );
    foreach my $key (@keylist) {
      if ( $key eq "mdisk_grp_id" or $key eq "mdisk_grp_name" ) {
        my @b;
        if ( not defined $cfg_ref->{svc_vdsk}->{$rec}->{$key} ) {
          if ( defined $cfg_ref->{svc_vdsk}->{$rec}->{ "copy_0_" . $key } ) {
            push( @b, $cfg_ref->{svc_vdsk}->{$rec}->{ "copy_0_" . $key } );
          }
          if ( defined $cfg_ref->{svc_vdsk}->{$rec}->{ "copy_1_" . $key } ) {
            push( @b, $cfg_ref->{svc_vdsk}->{$rec}->{ "copy_1_" . $key } );
          }
          if ( scalar(@b) ) {
            push( @a, join( " ", @b ) );
          }
          else {
            push( @a, "" );
          }
        } ## end if ( not defined $cfg_ref...)
        elsif ( $cfg_ref->{svc_vdsk}->{$rec}->{$key} eq "many" ) {
          if ( defined $fullcfg_ref->{svc_vdsk}->{$rec}->{ "copy_0_" . $key } ) {
            push( @b, $fullcfg_ref->{svc_vdsk}->{$rec}->{ "copy_0_" . $key } );
          }
          if ( defined $fullcfg_ref->{svc_vdsk}->{$rec}->{ "copy_1_" . $key } ) {
            push( @b, $fullcfg_ref->{svc_vdsk}->{$rec}->{ "copy_1_" . $key } );
          }
          if ( scalar(@b) ) {
            push( @a, join( " ", @b ) );
          }
          else {
            push( @a, "" );
          }
        } ## end elsif ( $cfg_ref->{svc_vdsk...})
        else {
          push( @a, $cfg_ref->{svc_vdsk}->{$rec}->{$key} );
        }
      } ## end if ( $key eq "mdisk_grp_id"...)
      elsif ( defined $cfg_ref->{svc_vdsk}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_vdsk}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintVdiskData

sub PrintHostData {
  my $delim   = ",";
  my @keylist = split( /,/, "id,name,port_count,iogrp_count,status" );
  my $hdrline = join( $delim, @keylist ) . $delim . "IQN WWPN" . $delim . "Volume IDs" . $delim . "Volume Names" . $delim ."host_cluster_id" . $delim ."host_cluster_name";

  print CONFOUT "\nHost Level Configuration\n";
  print CONFOUT "--------------------------\n";
  print CONFOUT "host_id", $delim, $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_host} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    my @w;
    my @d;
    my @n;
    push( @a, $rec );
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_host}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_host}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)

    # Add IQN and WWPNs
    my $i = 0;
    while ( $i < $cfg_ref->{svc_host}->{$rec}->{port_count} ) {
      if ( $cfg_ref->{svc_host}->{$rec}->{'iscsi_name'} ) {
        push( @w, $cfg_ref->{svc_host}->{$rec}->{'iscsi_name'} );
      }
      elsif ( $cfg_ref->{svc_host}->{$rec}->{ "FC" . $i . "_WWPN" } ) {
        push( @w, $cfg_ref->{svc_host}->{$rec}->{ "FC" . $i . "_WWPN" } );
      }
      $i++;
    } ## end while ( $i < $cfg_ref->{svc_host...})
    push( @a, join( " ", @w ) );

    # Add Vdisk IDs
    foreach my $map ( @{ $cfg_ref->{svc_vdiskhostmap} } ) {
      if ( $cfg_ref->{svc_host}->{$rec}->{'id'} eq $map->{'host_id'} ) {
        push( @d, $map->{'vdisk_id'} );
      }
    }
    @d = sort(@d);
    push( @a, join( " ", @d ) );

    # Add Vdisk Names
    foreach my $id (@d) {
      $id = sprintf( "%04d", $id );
      push( @n, $cfg_ref->{svc_vdsk}->{$id}->{'name'} );
    }
    push( @a, join( " ", @n ) );
    
    # Add Host Cluster 
    if ( defined $cfg_ref->{svc_host}->{$rec}->{'host_cluster_id'} ) {
    	push( @a, $cfg_ref->{svc_host}->{$rec}->{'host_cluster_id'}, $cfg_ref->{svc_host}->{$rec}->{'host_cluster_name'} );
    } else {
    	push( @a, "", "" );
    }
    
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintHostData

sub PrintRcrelData {
  my $delim   = ",";
  my @keylist = split( /,/, "id,name,master_cluster_id,master_cluster_name,master_vdisk_id,master_vdisk_name,aux_cluster_id,aux_cluster_name,aux_vdisk_id,aux_vdisk_name,primary,consistency_group_id,consistency_group_name,state,bg_copy_priority,progress,copy_type,cycling_mode,freeze_time" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nRC Relationship Level Configuration\n";
  print CONFOUT "------------------------\n";
  print CONFOUT $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_rcrel} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_rcrel}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_rcrel}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintRcrelData

sub PrintFcmapData {
  my $delim   = ",";
  my @keylist = split( /,/, "id,name,source_vdisk_id,source_vdisk_name,target_vdisk_id,target_vdisk_name,group_id,group_name,status,progress,copy_rate,clean_progress,incremental,partner_FC_id,partner_FC_name,restoring,start_time,rc_controlled" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nFC Map Level Configuration\n";
  print CONFOUT "------------------------\n";
  print CONFOUT $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_fcmap} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    # Suppress list of the FC Maps for GlobalMirror w/Change Volumes 
    if ( $cfg_ref->{svc_fcmap}->{$rec}->{'rc_controlled'} eq "yes" ) { next; }
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_fcmap}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_fcmap}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintFcmapData

sub PrintEventData {
  my $delim   = ",";
  my @keylist = split( /,/, "sequence_number,last_timestamp,object_type,object_id,object_name,copy_id,status,fixed,event_id,error_code,description" );
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nEvent Log Level\n";
  print CONFOUT "------------------------\n";
  print CONFOUT "event_id", $delim, $hdrline, "\n";

  my @reclist = keys( %{ $cfg_ref->{svc_event} } );
  foreach my $rec ( sort(@reclist) ) {
    my @a;
    push( @a, $rec );
    foreach my $key (@keylist) {
      if ( defined $cfg_ref->{svc_event}->{$rec}->{$key} ) {
        push( @a, $cfg_ref->{svc_event}->{$rec}->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( sort(@reclist...))
  print CONFOUT "\n";
} ## end sub PrintEventData

sub PrintFabricData {
  my $delim   = ",";
  #
  my @keylist;
  if ( $cfg_ref->{info}->{is_fs840} ) {
    @keylist = split( /:/, "remote_uid:local_uid:canister_id:adapter_id:port_id:node_id:node_name:type" );
  } else {
    @keylist = split( /:/, "remote_wwpn:remote_nportid:id:node_name:local_wwpn:local_port:local_nportid:state:name:cluster_name:type" );
  }
  my $hdrline = join( $delim, @keylist );

  print CONFOUT "\nFabric Level Configuration\n";
  print CONFOUT "------------------------\n";
  print CONFOUT $hdrline, "\n";
  
  foreach my $rec ( @{ $cfg_ref->{svc_fabric} } ) {
    my @a;
    foreach my $key (@keylist) {
      if ( defined $rec->{$key} ) {
        push( @a, $rec->{$key} );
      }
      else {
        push( @a, "" );
      }
    } ## end foreach my $key (@keylist)
    print CONFOUT join( $delim, @a ), "\n";
  } ## end foreach my $rec ( @{ $cfg_ref...})
  print CONFOUT "\n";
} ## end sub PrintFabricData

sub error_die {
  my $message  = shift;
  my $act_time = localtime();
  print STDERR "$act_time: $message : $!\n";
  exit(1);
} ## end sub error_die

sub remove_whitespace {
  my $value = shift;
  $value =~ s/^\s+|\s+$//g;
  return $value;

} ## end sub remove_whitespace

sub get_array_data {
  my $file = shift;
  open( FH, "< $file" ) || error_die( "Cannot read $file: $!" . __FILE__ . ":" . __LINE__ );
  my @file_all = <FH>;
  close(FH);
  return @file_all;
} ## end sub get_array_data

sub get_header_index {
  my $line   = shift;
  my @header = split( ",", $line );
  my $index  = -1;
  my %header_index;

  foreach my $element (@header) {
    chomp $element;
    $index++;
    if ( $element eq "" ) { next; }
    $element = remove_whitespace($element);
    if ( defined $header_index{$element} ) { next; }
    $header_index{$element} = $index;
  } ## end foreach my $element (@header)
  return %header_index;

} ## end sub get_header_index

sub set_data_by_header_array {
  my ( $data_line_tmp, $header_index_tmp, $health_status_tmp, $section ) = @_;
  my @data_line     = @{$data_line_tmp};
  my %header_index  = %{$header_index_tmp};
  my %health_status = %{$health_status_tmp};
  my $id            = "";
  if ( defined $header_index{event_id} && $section eq "EVENT" ) {
    $id = $data_line[ $header_index{event_id} ];
  }
  else {
    if ( defined $data_line[0] && $data_line[0] ne "" ) {
      $id = $data_line[0];
    }
    else {
      return;
    }
  } ## end else [ if ( defined $header_index...)]
  foreach my $element ( keys %header_index ) {
    my $value = $data_line[ $header_index{$element} ];

    #$value =~ s/,/ /g;
    my $name_property = $element;
    $health_status{$section}{$id}{$name_property} = $value;
  } ## end foreach my $element ( keys ...)
  return %health_status;
} ## end sub set_data_by_header_array

sub get_event_log {
  my $conf_file = shift;
  my %health_status;
  my $active = 0;
  my %header_index;
  my $section = "";

  if ( !( -e $conf_file ) && -z $conf_file ) {
    return %health_status;
  }
  my @data = get_array_data($conf_file);

  foreach my $line (@data) {
    chomp $line;
    if ($line eq ""){
      $active = 0;
    }
    if ( $line eq "" || $line =~ m/-----------/ ) { next; }
    $line = remove_whitespace($line);
    if ( $line =~ m/^Event Log level/i ) {
      $active = 1;
      next;
    }
    if ( $line =~ m/Level Configuration/ && $line !~ m/RC Relationship|FC Map/) {
      $active  = 3;
      $section = $line;
      $section =~ s/Level Configuration.*//g;
      $section = remove_whitespace($section);
      next;
    } ## end if ( $line =~ m/Level Configuration/)
    if ( $active == 3 ) {
      %header_index = get_header_index($line);
      $active       = 4;
      next;
    }
    if ( $active == 4 && $section ne "" ) {
      my @data_line = split( ",", $line );
      $section_check++;
      %health_status = set_data_by_header_array( \@data_line, \%header_index, \%health_status, $section );
      next;
    } ## end if ( $active == 4 && $section...)
    if ( $active == 1 ) {
      %header_index = get_header_index($line);
      $active       = 2;
      next;
    }
    if ( $active == 2 ) {
      my @data_line = split( ",", $line );
      $event_check++;
      $section       = "EVENT";
      %health_status = set_data_by_header_array( \@data_line, \%header_index, \%health_status, $section );
      $section       = "";
      next;
    } ## end if ( $active == 2 )
  } ## end foreach my $line (@data)

  return %health_status;

} ## end sub get_event_log

sub get_health_status_html {
  my $table = "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
  my $thead = "<thead><tr><th class = \"sortable\" title=\"Date/Time\" nowrap=\"\">Date/Time</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"event_id\" nowrap=\"\">event_id</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"object_type\" nowrap=\"\">object_type</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"object_name\" nowrap=\"\">object_name</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"description\" nowrap=\"\">description</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"status\" nowrap=\"\">status</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"fixed\" nowrap=\"\">fixed</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"seq_number\" nowrap=\"\">seq_number</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"object_id\" nowrap=\"\">object_id</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"copy_id\" nowrap=\"\">copy_id</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"error_code\" nowrap=\"\">error_code</th>\n";
  $thead = $thead . "</tr>\n</thead>\n";
  my $tbody = "<tbody>\n";

  foreach my $id ( sort keys %{ $health_status{EVENT} } ) {
    my $tr = "<tr>";

    my $property = "last_timestamp";
    my $td       = "";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "event_id";
    $td = get_value_to_table( $id, $property, $td,, "EVENT" );

    $property = "object_type";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "object_name";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "description";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "status";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "fixed";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "sequence_number";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "object_id";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "copy_id";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    $property = "error_code";
    $td = get_value_to_table( $id, $property, $td, "EVENT" );

    my $val = $td;
    my $index = 0;
    my $line = "";
    my @vals = split("\n",$val);
    my $size = scalar @vals;
    foreach my $element (@vals){
      $index++;
      $element = remove_whitespace($element);
      if ($element eq "" && $index != $size){
        $line = $line . " : ";
        next;
      }
      $element  =~ s/<.*">//g;
      $element =~ s/<\/td>//g;
      $element =~ s/<td>//g;
      $element = remove_whitespace($element);
      if ($line eq ""){
        $line = $element;
      }
      else{
        $line = $line . " : " . $element;
      }
      if ($index == $size && $element eq ""){
        $line = "";
        last;
      }
    }
    if ($line ne ""){
      push (@error_message,"$line\n");
    }

    $tr    = $tr . $td . "</tr>\n";
    $tbody = $tbody . "$tr\n";

  } ## end foreach my $id ( sort keys ...)
  $table = $table . $thead . "$tbody</tbody>\n" . "</table><br><br><br>\n";

  my $table_section = "<table class =\"tabcfgsum tablesorter tablesortercfgsum\">\n";
  $thead = "<thead><tr><th class = \"sortable\" title=\"section\" nowrap=\"\">section</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"id\" nowrap=\"\">id</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"name\" nowrap=\"\">name</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"status\" nowrap=\"\">status</th>\n";
  $thead = $thead . "<th class = \"sortable\" title=\"description\" nowrap=\"\">description</th>\n";
  $thead = $thead . "</tr>\n</thead>\n";
  $tbody = "<tbody>\n";

  my $row_element = 0;
  foreach my $section ( keys %health_status ) {
    if ( $section eq "EVENT" ) { next; }
    foreach my $id ( keys %{ $health_status{$section} } ) {
      if ( defined $health_status{$section}{$id}{"status"} && $health_status{$section}{$id}{"status"} =~ m/online|degraded|active|inactive_unconfigured|degraded_paths|degraded_ports/i ) {
        next;    ### we show only rows with status error
      }
      $row_element++;
      my $td       = "";
      my $tr       = "<tr>";
      my $property = "section";
      $td = get_value_to_table( $id, $property, $td, $section );

      $property = "id";
      $td = get_value_to_table( $id, $property, $td, $section );

      $property = "name";
      $td = get_value_to_table( $id, $property, $td, $section );

      $property = "status";
      $td = get_value_to_table( $id, $property, $td, $section );

      $property = "description";
      $td = get_value_to_table( $id, $property, $td, $section );

      my $val = $td;
      my $index = 0;
      my $line = "";
      my @vals = split("\n",$val);
      my $size = scalar @vals;

      foreach my $element (@vals){
        $index++;
        $element = remove_whitespace($element);
        if ($element eq "" && $index != $size){
          $line = $line . " : ";
          next;
        }
        $element  =~ s/<.*">//g;
        $element =~ s/<\/td>//g;
        $element =~ s/<td>//g;
        $element = remove_whitespace($element);
        if ($line eq ""){
          $line = $element;
        }
        else{
          $line = $line . " : " . $element;
        }
      }
      push (@error_message,"$line\n");

      $tr    = $tr . $td . "</tr>\n";
      $tbody = $tbody . "$tr\n";
    } ## end foreach my $id ( keys %{ $health_status...})
  } ## end foreach my $section ( keys ...)
  $table_section = $table_section . $thead . "$tbody</tbody>\n" . "</table>\n";
  if ( $row_element == 0 ) { $table_section = ""; }

  my $report = "";
  my $time   = localtime();
  $time = $time . "<br>\n";
  if ( $error_status == 0 && $row_element == 0 ) {
    $report = "Storage $storage health status = Optimal\n";
  }
  else {
    $report = "Storage $storage health status = Error\n";
  }
  $table = $time . $report . $table . "$table_section\n";

  return $table;

} ## end sub get_health_status_html

sub get_value_to_table {
  my $id       = shift;
  my $property = shift;
  my $td       = shift;
  my $section  = shift;

  if ( $property eq "description" ) {
    $td = $td . "<td>";
  }
  else {
    $td = $td . "<td nowrap=\"\">";
  }
  if ( defined $health_status{$section}{$id}{$property} ) {
    my $value = $health_status{$section}{$id}{$property};
    if ( $value =~ m/undef/i ) { $value = ""; }
    if ( $property eq "last_timestamp" && $value ne "" ) {
      my @pom = split( "", $value );
      $value = "$pom[2]$pom[3]/$pom[4]$pom[5]/20$pom[0]$pom[1] $pom[6]$pom[7]:$pom[8]$pom[9]:$pom[10]$pom[11]";
    }
    if ( $property eq "error_code" && $value ne "" ) {
      $error_status++;
    }
    if ( $property eq "status" ) {
      if ( $section eq "Node" && $value =~ m/online/i ) {
        $value = "OK";

      }
      if ( $section eq "Node" && $value =~ m/offline|service/i ) {
        $value = "Error";
      }
      if ( $section eq "Node" && $value =~ m/flushing|pending|adding/i ) {
        $value = "Warning";
      }
      if ( $section eq "Enclosure" && $value =~ m/online/i ) {
        $value = "OK";
        $health_status{$section}{$id}{description} = "the canister is present and working normally";

      }
      if ( $section eq "Enclosure" && $value =~ m/degraded/i ) {
        $value = "Warning";
        $health_status{$section}{$id}{description} = "the canister is present but not working normally";
      }
      if ( $section eq "Enclosure" && $value =~ m/offline/i ) {
        $value = "Error";
        $health_status{$section}{$id}{description} = "the canister could not be detected";
      }
      if ( $section eq "Port" && $value =~ m/active|inactive_unconfigured/i ) {
        $value = "OK";

      }
      if ( $section eq "Port" && $value =~ m/inactive_configured/i ) {
        $value = "Error";
      }
      if ( $section eq "Drive" && $value =~ m/online/i ) {
        $value = "OK";

      }
      if ( $section eq "Drive" && $value =~ m/degraded/i ) {
        $value = "Warning";
      }
      if ( $section eq "Drive" && $value =~ m/offline/i ) {
        $value = "Error";
      }
      if ( $section eq "Managed Disk" && $value =~ m/online/i ) {
        $value = "OK";
        $health_status{$section}{$id}{description} = "The MDisk is online and available";

      }
      if ( $section eq "Managed Disk" && $value =~ m/degraded_paths|degraded_ports|degraded/i ) {
        $value = "Warning";
        if ( $value =~ m/degraded_paths/i ) {
          $health_status{$section}{$id}{description} = "One or more paths to the MDisk have been lost; the MDisk is not online to every node in the system";
        }
      } ## end if ( $section eq "Managed Disk"...)
      if ( $value =~ m/degraded_ports/i ) {
        $health_status{$section}{$id}{description} = "MDisk port errors";
      }
      if ( $value =~ m/degraded/i ) {
        $health_status{$section}{$id}{description} = "The array has members that are degraded, or the raid_status is degraded. (applies only to internal MDisks)";
      }
    } ## end if ( $property eq "status")
    if ( $section eq "Managed Disk" && $value =~ m/offline|excluded/i ) {
      $value = "Error";
      if ( $value =~ m/offline/i ) {
        $health_status{$section}{$id}{description} = "All paths to the MDisk are lost";
      }
      if ( $value =~ m/excluded/i ) {
        $health_status{$section}{$id}{description} = "The MDisk is excluded from use by the system; the MDisk port error count exceeded the threshold";
      }
    } ## end if ( $section eq "Managed Disk"...)
    if ( $section eq "Pool" && $value =~ m/online/i ) {
      $value = "OK";
      $health_status{$section}{$id}{description} = "Indicates that the storage pool is online and available";
    }
    if ( $section eq "Pool" && $value =~ m/offline/i ) {
      $value = "Error";
      $health_status{$section}{$id}{description} = "Indicates that all paths to the storage pool are lost";
    }
    if ( $section eq "Volume" && $value =~ m/online/i ) {
      $value = "OK";

    }
    if ( $section eq "Volume" && $value =~ m/degraded/i ) {
      $value = "Warning";
    }
    if ( $section eq "Volume" && $value =~ m/offline/i ) {
      $value = "Error";
    }
    if ( $section eq "Host" && $value =~ m/online/i ) {
      $value = "OK";

    }
    if ( $section eq "Host" && $value =~ m/degraded/i ) {
      $value = "Warning";
      if ( $section eq "Host" && $value =~ m/offline/i ) {
        $value = "Error";
      }

    } ## end if ( $section eq "Host"...)
    $td = $td . "$value</td>\n";
  } ## end if ( defined $health_status...)
  else {
    if ( $property eq "section" ) {
      $td = $td . "$section</td>\n";
    }
    else {
      if ( $property eq "name" && $section eq "Port" && $health_status{$section}{$id}{port_name} ) {
        my $value = $health_status{$section}{$id}{port_name};
        $td = $td . "$value</td>\n";
      }
      else {
        $td = $td . "</td>\n";
      }
    } ## end else [ if ( $property eq "section")]
  } ## end else [ if ( defined $health_status...)]
  return $td;

} ## end sub get_value_to_table

sub summary_health_status {
  my $timestamp    = time();
  my $time         = localtime();
  my $alert_status = 0;

  ### test
  #Xorux_lib::health_status_alerting($storage);

  if ( !-d "$summary_dir" ) {
    mkdir( "$summary_dir", 0755 ) || error( "$time: Cannot mkdir $summary_dir: $!" . __FILE__ . ":" . __LINE__ );
  }

  if ( !-f "$summary_dir/$storage.ok" && !-f "$summary_dir/$storage.nok" ) {
    $alert_status = 1;
  }

  if ( -f "$summary_dir/$storage.ok" ) { unlink("$summary_dir/$storage.ok"); $alert_status = 1; }
  if ( -f "$summary_dir/$storage.nok" ) { unlink("$summary_dir/$storage.nok"); }

  if ( $error_status == 0 ) {
    my $report = "STORAGE : $storage : OK : $timestamp";
    write_health_status_html( "$summary_dir/$storage.ok", $report );
    return;
  }
  else {
    my $report = "STORAGE : $storage : NOT_OK : $timestamp";
    write_health_status_html( "$summary_dir/$storage.nok", $report );
    if ( $alert_status == 1 ) {
      ### alert health status
      if ( exists &Xorux_lib::health_status_alerting ) {
        Xorux_lib::health_status_alerting($storage,\@error_message);
      }
    } ## end if ( $alert_status == ...)
    return;
  } ## end else [ if ( $error_status == ...)]

} ## end sub summary_health_status

sub write_health_status_html {
  my $file  = shift;
  my $table = shift;
  open( DATA, ">$file" ) or error("Cannot open file: $file : $!") && return 0;
  print DATA $table;
  close(DATA);

} ## end sub write_health_status_html
